@@ -1,7 +1,7 @@
 /* $RoughId: sha1init.c,v 1.2 2001/07/13 19:49:10 knu Exp $ */
-/* $Id: sha1init.c 19020 2008-09-01 07:48:53Z nobu $ */
+/* $Id: sha1init.c 28341 2010-06-16 09:38:14Z knu $ */
 
-#include "digest.h"
+#include "../digest.h"
 #if defined(HAVE_OPENSSL_SHA_H)
 #include "sha1ossl.h"
 #else
@@ -27,9 +27,9 @@ void
 Init_sha1()
 {
     VALUE mDigest, cDigest_Base, cDigest_SHA1;
- 
+
     rb_require("digest");
- 
+
     mDigest = rb_path2class("Digest");
     cDigest_Base = rb_path2class("Digest::Base");
 
