@@ -1,6 +1,6 @@
-/* $Id: sha1ossl.c 11908 2007-02-27 11:51:55Z knu $ */
+/* $Id: sha1ossl.c 28341 2010-06-16 09:38:14Z knu $ */
 
-#include "defs.h"
+#include "../defs.h"
 #include "sha1ossl.h"
 
 void
