@@ -1,24 +1,33 @@
 /* $RoughId: sha2init.c,v 1.3 2001/07/13 20:00:43 knu Exp $ */
-/* $Id: sha2init.c,v 1.1.2.1 2001/08/16 07:35:43 knu Exp $ */
+/* $Id: sha2init.c 28341 2010-06-16 09:38:14Z knu $ */
 
-#include "digest.h"
+#include "../digest.h"
+#if defined(SHA2_USE_OPENSSL)
+#include "sha2ossl.h"
+#else
 #include "sha2.h"
+#endif
 
 #define FOREACH_BITLEN(func)	func(256) func(384) func(512)
 
 #define DEFINE_ALGO_METADATA(bitlen) \
-static algo_t sha##bitlen = { \
+static const rb_digest_metadata_t sha##bitlen = { \
+    RUBY_DIGEST_API_VERSION, \
     SHA##bitlen##_DIGEST_LENGTH, \
+    SHA##bitlen##_BLOCK_LENGTH, \
     sizeof(SHA##bitlen##_CTX), \
-    (hash_init_func_t)SHA##bitlen##_Init, \
-    (hash_update_func_t)SHA##bitlen##_Update, \
-    (hash_end_func_t)SHA##bitlen##_End, \
-    (hash_final_func_t)SHA##bitlen##_Final, \
-    (hash_equal_func_t)SHA##bitlen##_Equal, \
+    (rb_digest_hash_init_func_t)SHA##bitlen##_Init, \
+    (rb_digest_hash_update_func_t)SHA##bitlen##_Update, \
+    (rb_digest_hash_finish_func_t)SHA##bitlen##_Finish, \
 };
 
 FOREACH_BITLEN(DEFINE_ALGO_METADATA)
 
+/*
+ * Classes for calculating message digests using the SHA-256/384/512
+ * Secure Hash Algorithm(s) by NIST (the US' National Institute of
+ * Standards and Technology), described in FIPS PUB 180-2.
+ */
 void
 Init_sha2()
 {
@@ -30,7 +39,7 @@ Init_sha2()
 
     FOREACH_BITLEN(DECLARE_ALGO_CLASS)
 
-    rb_require("digest.so");
+    rb_require("digest");
 
     id_metadata = rb_intern("metadata");
 
@@ -40,8 +49,8 @@ Init_sha2()
 #define DEFINE_ALGO_CLASS(bitlen) \
     cDigest_SHA##bitlen = rb_define_class_under(mDigest, "SHA" #bitlen, cDigest_Base); \
 \
-    rb_cvar_declare(cDigest_SHA##bitlen, id_metadata, \
-		    Data_Wrap_Struct(rb_cObject, 0, 0, &sha##bitlen));
+    rb_ivar_set(cDigest_SHA##bitlen, id_metadata, \
+      Data_Wrap_Struct(rb_cObject, 0, 0, (void *)&sha##bitlen));
 
     FOREACH_BITLEN(DEFINE_ALGO_CLASS)
 }
