@@ -4,7 +4,9 @@ have_library("sun", "getpwnam")	# NIS (== YP) interface for IRIX 4
 a = have_func("getlogin")
 b = have_func("getpwent")
 c = have_func("getgrent")
-if  a or b or c
+sysconfdir = RbConfig.expand(RbConfig::MAKEFILE_CONFIG["sysconfdir"].dup, "prefix"=>"")
+$defs.push("-DSYSCONFDIR=#{Shellwords.escape(sysconfdir.dump)}")
+if  a or b or c or sysconfdir
   have_struct_member('struct passwd', 'pw_gecos', 'pwd.h')
   have_struct_member('struct passwd', 'pw_change', 'pwd.h')
   have_struct_member('struct passwd', 'pw_quota', 'pwd.h')
@@ -24,18 +26,5 @@ if  a or b or c
   have_struct_member('struct passwd', 'pw_expire', 'pwd.h')
   have_struct_member('struct passwd', 'pw_passwd', 'pwd.h')
   have_struct_member('struct group', 'gr_passwd', 'grp.h')
-  [%w"uid_t pwd.h", %w"gid_t grp.h"].each do |t, *h|
-    h.unshift("sys/types.h")
-    f = "INT2NUM"
-    if have_type(t, h)
-      if try_static_assert("sizeof(#{t}) > sizeof(long)", h)
-	f = "LL2NUM"
-      end
-      if try_static_assert("(#{t})-1 > 0", h)
-	f = "U#{f}"
-      end
-    end
-    $defs.push("-DPW_#{t.chomp('_t').upcase}2VAL=#{f}")
-  end
   create_makefile("etc")
 end
