@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_engine.c,v 1.4.2.6 2005/09/18 22:56:11 gotoyuzo Exp $
+ * $Id: ossl_engine.c 27457 2010-04-23 08:37:55Z usa $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2003  GOTOU Yuuzou <gotoyuzo@notwork.org>
  * All rights reserved.
@@ -29,7 +29,7 @@
     GetPKCS7(obj, engine); \
 } while (0)
 
-/* 
+/*
  * Classes
  */
 VALUE cEngine;
@@ -40,7 +40,7 @@ VALUE eEngineError;
  */
 #define OSSL_ENGINE_LOAD_IF_MATCH(x) \
 do{\
-  if(!strcmp(#x, RSTRING(name)->ptr)){\
+  if(!strcmp(#x, RSTRING_PTR(name))){\
     ENGINE_load_##x();\
     return Qtrue;\
   }\
@@ -61,21 +61,39 @@ ossl_engine_s_load(int argc, VALUE *argv, VALUE klass)
     }
     StringValue(name);
 #ifndef OPENSSL_NO_STATIC_ENGINE
+#if HAVE_ENGINE_LOAD_DYNAMIC
     OSSL_ENGINE_LOAD_IF_MATCH(dynamic);
+#endif
+#if HAVE_ENGINE_LOAD_CSWIFT
     OSSL_ENGINE_LOAD_IF_MATCH(cswift);
+#endif
+#if HAVE_ENGINE_LOAD_CHIL
     OSSL_ENGINE_LOAD_IF_MATCH(chil);
+#endif
+#if HAVE_ENGINE_LOAD_ATALLA
     OSSL_ENGINE_LOAD_IF_MATCH(atalla);
+#endif
+#if HAVE_ENGINE_LOAD_NURON
     OSSL_ENGINE_LOAD_IF_MATCH(nuron);
+#endif
+#if HAVE_ENGINE_LOAD_UBSEC
     OSSL_ENGINE_LOAD_IF_MATCH(ubsec);
+#endif
+#if HAVE_ENGINE_LOAD_AEP
     OSSL_ENGINE_LOAD_IF_MATCH(aep);
+#endif
+#if HAVE_ENGINE_LOAD_SUREWARE
     OSSL_ENGINE_LOAD_IF_MATCH(sureware);
+#endif
+#if HAVE_ENGINE_LOAD_4758CCA
     OSSL_ENGINE_LOAD_IF_MATCH(4758cca);
 #endif
+#endif
 #ifdef HAVE_ENGINE_LOAD_OPENBSD_DEV_CRYPTO
     OSSL_ENGINE_LOAD_IF_MATCH(openbsd_dev_crypto);
 #endif
     OSSL_ENGINE_LOAD_IF_MATCH(openssl);
-    rb_warning("no such builtin loader for `%s'", RSTRING(name)->ptr);
+    rb_warning("no such builtin loader for `%s'", RSTRING_PTR(name));
     return Qnil;
 #endif /* HAVE_ENGINE_LOAD_BUILTIN_ENGINES */
 }
@@ -112,14 +130,14 @@ ossl_engine_s_by_id(VALUE klass, VALUE id)
 
     StringValue(id);
     ossl_engine_s_load(1, &id, klass);
-    if(!(e = ENGINE_by_id(RSTRING(id)->ptr)))
+    if(!(e = ENGINE_by_id(RSTRING_PTR(id))))
 	ossl_raise(eEngineError, NULL);
     WrapEngine(klass, obj, e);
     if(rb_block_given_p()) rb_yield(obj);
     if(!ENGINE_init(e))
 	ossl_raise(eEngineError, NULL);
     ENGINE_ctrl(e, ENGINE_CTRL_SET_PASSWORD_CALLBACK,
-		0, NULL, (void(*)())ossl_pem_passwd_cb);
+		0, NULL, (void(*)(void))ossl_pem_passwd_cb);
     ERR_clear_error();
 
     return obj;
@@ -166,10 +184,10 @@ ossl_engine_finish(VALUE self)
     return Qnil;
 }
 
+#if defined(HAVE_ENGINE_GET_CIPHER)
 static VALUE
 ossl_engine_get_cipher(VALUE self, VALUE name)
 {
-#if defined(HAVE_ENGINE_GET_CIPHER)
     ENGINE *e;
     const EVP_CIPHER *ciph, *tmp;
     char *s;
@@ -184,15 +202,15 @@ ossl_engine_get_cipher(VALUE self, VALUE name)
     if(!ciph) ossl_raise(eEngineError, NULL);
 
     return ossl_cipher_new(ciph);
-#else
-    rb_notimplement();
-#endif
 }
+#else
+#define ossl_engine_get_cipher rb_f_notimplement
+#endif
 
+#if defined(HAVE_ENGINE_GET_DIGEST)
 static VALUE
 ossl_engine_get_digest(VALUE self, VALUE name)
 {
-#if defined(HAVE_ENGINE_GET_DIGEST)
     ENGINE *e;
     const EVP_MD *md, *tmp;
     char *s;
@@ -207,10 +225,10 @@ ossl_engine_get_digest(VALUE self, VALUE name)
     if(!md) ossl_raise(eEngineError, NULL);
 
     return ossl_digest_new(md);
-#else
-    rb_notimplement();
-#endif
 }
+#else
+#define ossl_engine_get_digest rb_f_notimplement
+#endif
 
 static VALUE
 ossl_engine_load_privkey(int argc, VALUE *argv, VALUE self)
@@ -281,8 +299,8 @@ ossl_engine_ctrl_cmd(int argc, VALUE *argv, VALUE self)
     rb_scan_args(argc, argv, "11", &cmd, &val);
     StringValue(cmd);
     if (!NIL_P(val)) StringValue(val);
-    ret = ENGINE_ctrl_cmd_string(e, RSTRING(cmd)->ptr,
-				 NIL_P(val) ? NULL : RSTRING(val)->ptr, 0);
+    ret = ENGINE_ctrl_cmd_string(e, RSTRING_PTR(cmd),
+				 NIL_P(val) ? NULL : RSTRING_PTR(val), 0);
     if (!ret) ossl_raise(eEngineError, NULL);
 
     return self;
@@ -326,8 +344,8 @@ static VALUE
 ossl_engine_inspect(VALUE self)
 {
     VALUE str;
-    char *cname = rb_class2name(rb_obj_class(self));
-    
+    const char *cname = rb_class2name(rb_obj_class(self));
+
     str = rb_str_new2("#<");
     rb_str_cat2(str, cname);
     rb_str_cat2(str, " id=\"");
