@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_pkey_dh.c,v 1.4.2.3 2005/04/08 09:26:54 gotoyuzo Exp $
+ * $Id: ossl_pkey_dh.c 27440 2010-04-22 08:21:01Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -42,7 +42,7 @@ dh_instance(VALUE klass, DH *dh)
 {
     EVP_PKEY *pkey;
     VALUE obj;
-	
+
     if (!dh) {
 	return Qfalse;
     }
@@ -85,7 +85,7 @@ static DH *
 dh_generate(int size, int gen)
 {
     DH *dh;
-    
+
     dh = DH_generate_parameters(size, gen,
 	    rb_block_given_p() ? ossl_generate_cb : NULL,
 	    NULL);
@@ -99,17 +99,26 @@ dh_generate(int size, int gen)
     return dh;
 }
 
+/*
+ *  call-seq:
+ *     DH.generate(size [, generator]) -> dh
+ *
+ *  === Parameters
+ *  * +size+ is an integer representing the desired key size.  Keys smaller than 1024 should be considered insecure.
+ *  * +generator+ is a small number > 1, typically 2 or 5.
+ *
+ */
 static VALUE
 ossl_dh_s_generate(int argc, VALUE *argv, VALUE klass)
 {
     DH *dh ;
     int g = 2;
     VALUE size, gen, obj;
-	
+
     if (rb_scan_args(argc, argv, "11", &size, &gen) == 2) {
-	g = FIX2INT(gen);
+	g = NUM2INT(gen);
     }
-    dh = dh_generate(FIX2INT(size), g);
+    dh = dh_generate(NUM2INT(size), g);
     obj = dh_instance(klass, dh);
     if (obj == Qfalse) {
 	DH_free(dh);
@@ -119,6 +128,21 @@ ossl_dh_s_generate(int argc, VALUE *argv, VALUE klass)
     return obj;
 }
 
+/*
+ *  call-seq:
+ *     DH.new([size [, generator] | string]) -> dh
+ *
+ *  === Parameters
+ *  * +size+ is an integer representing the desired key size.  Keys smaller than 1024 should be considered insecure.
+ *  * +generator+ is a small number > 1, typically 2 or 5.
+ *  * +string+ contains the DER or PEM encoded key.
+ *
+ *  === Examples
+ *  * DH.new -> dh
+ *  * DH.new(1024) -> dh
+ *  * DH.new(1024, 5) -> dh
+ *  * DH.new(File.read('key.pem')) -> dh
+ */
 static VALUE
 ossl_dh_initialize(int argc, VALUE *argv, VALUE self)
 {
@@ -134,7 +158,7 @@ ossl_dh_initialize(int argc, VALUE *argv, VALUE self)
     }
     else if (FIXNUM_P(arg)) {
 	if (!NIL_P(gen)) {
-	    g = FIX2INT(gen);
+	    g = NUM2INT(gen);
 	}
 	if (!(dh = dh_generate(FIX2INT(arg), g))) {
 	    ossl_raise(eDHError, NULL);
@@ -145,7 +169,7 @@ ossl_dh_initialize(int argc, VALUE *argv, VALUE self)
 	in = ossl_obj2bio(arg);
 	dh = PEM_read_bio_DHparams(in, NULL, NULL, NULL);
 	if (!dh){
-	    BIO_reset(in);
+	    (void)BIO_reset(in);
 	    dh = d2i_DHparams_bio(in, NULL);
 	}
 	BIO_free(in);
@@ -158,29 +182,41 @@ ossl_dh_initialize(int argc, VALUE *argv, VALUE self)
     return self;
 }
 
+/*
+ *  call-seq:
+ *     dh.public? -> true | false
+ *
+ */
 static VALUE
 ossl_dh_is_public(VALUE self)
 {
     EVP_PKEY *pkey;
 
     GetPKeyDH(self, pkey);
-    /*
-     * Do we need to check dhp->dh->public_pkey?
-     * return Qtrue;
-     */
+
     return (pkey->pkey.dh->pub_key) ? Qtrue : Qfalse;
 }
 
+/*
+ *  call-seq:
+ *     dh.private? -> true | false
+ *
+ */
 static VALUE
 ossl_dh_is_private(VALUE self)
 {
     EVP_PKEY *pkey;
 
     GetPKeyDH(self, pkey);
-	
+
     return (DH_PRIVATE(pkey->pkey.dh)) ? Qtrue : Qfalse;
 }
 
+/*
+ *  call-seq:
+ *     dh.to_pem -> aString
+ *
+ */
 static VALUE
 ossl_dh_export(VALUE self)
 {
@@ -201,9 +237,14 @@ ossl_dh_export(VALUE self)
     return str;
 }
 
+/*
+ *  call-seq:
+ *     dh.to_der -> aString
+ *
+ */
 static VALUE
 ossl_dh_to_der(VALUE self)
-{       
+{
     EVP_PKEY *pkey;
     unsigned char *p;
     long len;
@@ -213,7 +254,7 @@ ossl_dh_to_der(VALUE self)
     if((len = i2d_DHparams(pkey->pkey.dh, NULL)) <= 0)
 	ossl_raise(eDHError, NULL);
     str = rb_str_new(0, len);
-    p = RSTRING(str)->ptr;
+    p = (unsigned char *)RSTRING_PTR(str);
     if(i2d_DHparams(pkey->pkey.dh, &p) < 0)
 	ossl_raise(eDHError, NULL);
     ossl_str_adjust(str, p);
@@ -222,6 +263,9 @@ ossl_dh_to_der(VALUE self)
 }
 
 /*
+ *  call-seq:
+ *     dh.params -> hash
+ *
  * Stores all parameters of key to the hash
  * INSECURE: PRIVATE INFORMATIONS CAN LEAK OUT!!!
  * Don't use :-)) (I's up to you)
@@ -240,11 +284,14 @@ ossl_dh_get_params(VALUE self)
     rb_hash_aset(hash, rb_str_new2("g"), ossl_bn_new(pkey->pkey.dh->g));
     rb_hash_aset(hash, rb_str_new2("pub_key"), ossl_bn_new(pkey->pkey.dh->pub_key));
     rb_hash_aset(hash, rb_str_new2("priv_key"), ossl_bn_new(pkey->pkey.dh->priv_key));
-    
+
     return hash;
 }
 
 /*
+ *  call-seq:
+ *     dh.to_text -> aString
+ *
  * Prints all parameters of key to buffer
  * INSECURE: PRIVATE INFORMATIONS CAN LEAK OUT!!!
  * Don't use :-)) (I's up to you)
@@ -270,7 +317,10 @@ ossl_dh_to_text(VALUE self)
 }
 
 /*
- * Makes new instance DH PUBLIC_KEY from PRIVATE_KEY
+ *  call-seq:
+ *     dh.public_key -> aDH
+ *
+ *  Makes new instance DH PUBLIC_KEY from PRIVATE_KEY
  */
 static VALUE
 ossl_dh_to_public_key(VALUE self)
@@ -278,7 +328,7 @@ ossl_dh_to_public_key(VALUE self)
     EVP_PKEY *pkey;
     DH *dh;
     VALUE obj;
-	
+
     GetPKeyDH(self, pkey);
     dh = DHparams_dup(pkey->pkey.dh); /* err check perfomed by dh_instance */
     obj = dh_instance(CLASS_OF(self), dh);
@@ -290,13 +340,18 @@ ossl_dh_to_public_key(VALUE self)
     return obj;
 }
 
+/*
+ *  call-seq:
+ *     dh.check_params -> true | false
+ *
+ */
 static VALUE
 ossl_dh_check_params(VALUE self)
 {
     DH *dh;
     EVP_PKEY *pkey;
     int codes;
-    
+
     GetPKeyDH(self, pkey);
     dh = pkey->pkey.dh;
 
@@ -307,6 +362,11 @@ ossl_dh_check_params(VALUE self)
     return codes == 0 ? Qtrue : Qfalse;
 }
 
+/*
+ *  call-seq:
+ *     dh.generate_key -> self
+ *
+ */
 static VALUE
 ossl_dh_generate_key(VALUE self)
 {
@@ -321,6 +381,18 @@ ossl_dh_generate_key(VALUE self)
     return self;
 }
 
+/*
+ *  call-seq:
+ *     dh.compute_key(pub_bn) -> aString
+ *
+ *  === Parameters
+ *  * +pub_bn+ is a OpenSSL::BN.
+ *
+ *  Returns aString containing a shared secret computed from the other parties public value.
+ *
+ *  See DH_compute_key() for further information.
+ *
+ */
 static VALUE
 ossl_dh_compute_key(VALUE self, VALUE pub)
 {
@@ -335,19 +407,18 @@ ossl_dh_compute_key(VALUE self, VALUE pub)
     pub_key = GetBNPtr(pub);
     len = DH_size(dh);
     str = rb_str_new(0, len);
-    if ((len = DH_compute_key(RSTRING(str)->ptr, pub_key, dh)) < 0) {
+    if ((len = DH_compute_key((unsigned char *)RSTRING_PTR(str), pub_key, dh)) < 0) {
 	ossl_raise(eDHError, NULL);
     }
-    RSTRING(str)->len = len;
-    RSTRING(str)->ptr[len] = 0;
+    rb_str_set_len(str, len);
 
     return str;
 }
 
-OSSL_PKEY_BN(dh, p);
-OSSL_PKEY_BN(dh, g);
-OSSL_PKEY_BN(dh, pub_key);
-OSSL_PKEY_BN(dh, priv_key);
+OSSL_PKEY_BN(dh, p)
+OSSL_PKEY_BN(dh, g)
+OSSL_PKEY_BN(dh, pub_key)
+OSSL_PKEY_BN(dh, priv_key)
 
 /*
  * -----BEGIN DH PARAMETERS-----
@@ -365,10 +436,10 @@ static unsigned char DEFAULT_DH_512_PRIM[] = {
     0x08, 0x04, 0x8c, 0x52, 0x8f, 0xe3, 0x4a, 0x31,
     0x44, 0x47, 0x19, 0xa1, 0x4a, 0xc8, 0x8b, 0xcb,
 };
-static unsigned char DEFAULT_DH_512_GEN[] = { 0x02 }; 
+static unsigned char DEFAULT_DH_512_GEN[] = { 0x02 };
 DH *OSSL_DEFAULT_DH_512 = NULL;
 
-/* 
+/*
  * -----BEGIN DH PARAMETERS-----
  * MIGHAoGBAJ0lOVy0VIr/JebWn0zDwY2h+rqITFOpdNr6ugsgvkDXuucdcChhYExJ
  * AV/ZD2AWPbrTqV76mGRgJg4EddgT1zG0jq3rnFdMj2XzkBYx3BVvfR0Arnby0RHR
@@ -406,7 +477,7 @@ ossl_create_dh(unsigned char *p, size_t plen, unsigned char *g, size_t glen)
     dh->g = BN_bin2bn(g, glen, NULL);
     if (dh->p == NULL || dh->g == NULL){
         DH_free(dh);
-        ossl_raise(eDHError, NULL);
+	ossl_raise(eDHError, NULL);
     }
 
     return dh;
@@ -418,6 +489,11 @@ ossl_create_dh(unsigned char *p, size_t plen, unsigned char *g, size_t glen)
 void
 Init_ossl_dh()
 {
+#if 0 /* let rdoc know about mOSSL and mPKey */
+    mOSSL = rb_define_module("OpenSSL");
+    mPKey = rb_define_module_under(mOSSL, "PKey");
+#endif
+
     eDHError = rb_define_class_under(mPKey, "DHError", ePKeyError);
     cDH = rb_define_class_under(mPKey, "DH", cPKey);
     rb_define_singleton_method(cDH, "generate", ossl_dh_s_generate, -1);
@@ -448,11 +524,9 @@ Init_ossl_dh()
 }
 
 #else /* defined NO_DH */
-#   warning >>> OpenSSL is compiled without DH support <<<
 void
 Init_ossl_dh()
 {
-    rb_warning("OpenSSL is compiled without DH support");
 }
 #endif /* NO_DH */
 
