@@ -15,7 +15,7 @@
 	OSSL_Check_Kind(obj, cSSLSession); \
 	GetSSLSession(obj, sess); \
 } while (0)
-        
+
 
 VALUE cSSLSession;
 static VALUE eSSLSession;
@@ -72,6 +72,16 @@ static VALUE ossl_ssl_session_initialize(VALUE self, VALUE arg1)
 	return self;
 }
 
+#if HAVE_SSL_SESSION_CMP == 0
+int SSL_SESSION_cmp(const SSL_SESSION *a,const SSL_SESSION *b)
+{
+    if (a->ssl_version != b->ssl_version ||
+	a->session_id_length != b->session_id_length)
+	return 1;
+    return memcmp(a->session_id,b-> session_id, a->session_id_length);
+}
+#endif
+
 /*
  * call-seq:
  *    session1 == session2 -> boolean
@@ -107,7 +117,7 @@ static VALUE ossl_ssl_session_get_time(VALUE self)
 	if (t == 0)
 		return Qnil;
 
-	return rb_funcall(rb_cTime, rb_intern("at"), 1, LONG2NUM(t));
+	return rb_funcall(rb_cTime, rb_intern("at"), 1, TIMET2NUM(t));
 }
 
 /*
@@ -126,14 +136,14 @@ static VALUE ossl_ssl_session_get_timeout(VALUE self)
 
 	t = SSL_SESSION_get_timeout(ctx);
 
-	return ULONG2NUM(t);
+	return TIMET2NUM(t);
 }
 
 #define SSLSESSION_SET_TIME(func)						\
 	static VALUE ossl_ssl_session_set_##func(VALUE self, VALUE time_v)	\
 	{									\
 		SSL_SESSION *ctx;						\
-		time_t t;							\
+		unsigned long t;						\
 										\
 		GetSSLSession(self, ctx);					\
 										\
@@ -214,7 +224,7 @@ static VALUE ossl_ssl_session_to_pem(VALUE self)
 	BUF_MEM *buf;
 	VALUE str;
 	int i;
-         
+
 	GetSSLSession(self, ctx);
 
 	if (!(out = BIO_new(BIO_s_mem()))) {
@@ -246,7 +256,7 @@ static VALUE ossl_ssl_session_to_text(VALUE self)
 	BIO *out;
 	BUF_MEM *buf;
 	VALUE str;
-         
+
 	GetSSLSession(self, ctx);
 
 	if (!(out = BIO_new(BIO_s_mem()))) {
@@ -264,7 +274,7 @@ static VALUE ossl_ssl_session_to_text(VALUE self)
 
 	return str;
 }
-                                                     
+
 
 void Init_ossl_ssl_session(void)
 {
