@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509attr.c 18168 2008-07-22 15:34:23Z nobu $
+ * $Id: ossl_x509attr.c 27437 2010-04-22 08:04:13Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001 Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -41,7 +41,7 @@ ossl_x509attr_new(X509_ATTRIBUTE *attr)
 {
     X509_ATTRIBUTE *new;
     VALUE obj;
-    
+
     if (!attr) {
 	new = X509_ATTRIBUTE_new();
     } else {
@@ -77,7 +77,7 @@ ossl_x509attr_alloc(VALUE klass)
     X509_ATTRIBUTE *attr;
     VALUE obj;
 
-    if (!(attr = X509_ATTRIBUTE_new())) 
+    if (!(attr = X509_ATTRIBUTE_new()))
 	ossl_raise(eX509AttrError, NULL);
     WrapX509Attr(klass, obj, attr);
 
@@ -92,7 +92,7 @@ static VALUE
 ossl_x509attr_initialize(int argc, VALUE *argv, VALUE self)
 {
     VALUE oid, value;
-    X509_ATTRIBUTE *attr;
+    X509_ATTRIBUTE *attr, *x;
     const unsigned char *p;
 
     GetX509Attr(self, attr);
@@ -100,7 +100,9 @@ ossl_x509attr_initialize(int argc, VALUE *argv, VALUE self)
 	oid = ossl_to_der_if_possible(oid);
 	StringValue(oid);
 	p = (unsigned char *)RSTRING_PTR(oid);
-	if(!d2i_X509_ATTRIBUTE(&attr, &p, RSTRING_LEN(oid)) && (DATA_PTR(self) = attr, 1)){
+	x = d2i_X509_ATTRIBUTE(&attr, &p, RSTRING_LEN(oid));
+	DATA_PTR(self) = attr;
+	if(!x){
 	    ossl_raise(eX509AttrError, NULL);
 	}
 	return self;
@@ -121,14 +123,14 @@ ossl_x509attr_set_oid(VALUE self, VALUE oid)
     X509_ATTRIBUTE *attr;
     ASN1_OBJECT *obj;
     char *s;
- 
+
     s = StringValuePtr(oid);
     obj = OBJ_txt2obj(s, 0);
     if(!obj) obj = OBJ_txt2obj(s, 1);
     if(!obj) ossl_raise(eX509AttrError, NULL);
     GetX509Attr(self, attr);
     X509_ATTRIBUTE_set1_object(attr, obj);
- 
+
     return oid;
 }
 
@@ -155,7 +157,7 @@ ossl_x509attr_get_oid(VALUE self)
 	i2a_ASN1_OBJECT(out, oid);
 	ret = ossl_membio2str(out);
     }
-  
+
     return ret;
 }
 
@@ -216,8 +218,9 @@ ossl_x509attr_get_value(VALUE self)
 	ossl_str_adjust(str, p);
     }
     else{
-	length = i2d_ASN1_SET_OF_ASN1_TYPE(attr->value.set, NULL,
-			i2d_ASN1_TYPE, V_ASN1_SET, V_ASN1_UNIVERSAL, 0);
+	length = i2d_ASN1_SET_OF_ASN1_TYPE(attr->value.set,
+			(unsigned char **) NULL, i2d_ASN1_TYPE,
+			V_ASN1_SET, V_ASN1_UNIVERSAL, 0);
 	str = rb_str_new(0, length);
 	p = (unsigned char *)RSTRING_PTR(str);
 	i2d_ASN1_SET_OF_ASN1_TYPE(attr->value.set, &p,
@@ -248,7 +251,7 @@ ossl_x509attr_to_der(VALUE self)
     p = (unsigned char *)RSTRING_PTR(str);
     if(i2d_X509_ATTRIBUTE(attr, &p) <= 0)
 	ossl_raise(eX509AttrError, NULL);
-    rb_str_set_len(str, p - (unsigned char*)RSTRING_PTR(str)); 
+    rb_str_set_len(str, p - (unsigned char*)RSTRING_PTR(str));
 
     return str;
 }
