@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509name.c 18702 2008-08-19 08:34:42Z akr $
+ * $Id: ossl_x509name.c 27440 2010-04-22 08:21:01Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001 Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -41,7 +41,7 @@ VALUE eX509NameError;
 /*
  * Public
  */
-VALUE 
+VALUE
 ossl_x509name_new(X509_NAME *name)
 {
     X509_NAME *new;
@@ -56,7 +56,7 @@ ossl_x509name_new(X509_NAME *name)
 	ossl_raise(eX509NameError, NULL);
     }
     WrapX509Name(cX509Name, obj, new);
-    
+
     return obj;
 }
 
@@ -78,7 +78,7 @@ ossl_x509name_alloc(VALUE klass)
 {
     X509_NAME *name;
     VALUE obj;
-	
+
     if (!(name = X509_NAME_new())) {
 	ossl_raise(eX509NameError, NULL);
     }
@@ -137,13 +137,14 @@ ossl_x509name_initialize(int argc, VALUE *argv, VALUE self)
 	else{
 	    const unsigned char *p;
 	    VALUE str = ossl_to_der_if_possible(arg);
-	    X509_NAME *x = DATA_PTR(self);
+	    X509_NAME *x;
 	    StringValue(str);
 	    p = (unsigned char *)RSTRING_PTR(str);
-	    if(!d2i_X509_NAME(&x, &p, RSTRING_LEN(str)) && (DATA_PTR(self) = x, 1)){
+	    x = d2i_X509_NAME(&name, &p, RSTRING_LEN(str));
+	    DATA_PTR(self) = name;
+	    if(!x){
 		ossl_raise(eX509NameError, NULL);
 	    }
-            DATA_PTR(self) = x;
 	}
     }
 
@@ -221,7 +222,7 @@ ossl_x509name_to_s(int argc, VALUE *argv, VALUE self)
  * call-seq:
  *    name.to_a => [[name, data, type], ...]
  */
-static VALUE 
+static VALUE
 ossl_x509name_to_a(VALUE self)
 {
     X509_NAME *name;
@@ -230,7 +231,7 @@ ossl_x509name_to_a(VALUE self)
     char long_name[512];
     const char *short_name;
     VALUE ary, ret;
-	
+
     GetX509Name(self, name);
     entries = X509_NAME_entry_count(name);
     if (entries < 0) {
@@ -332,7 +333,7 @@ ossl_x509name_to_der(VALUE self)
 /*
  * INIT
  */
-void 
+void
 Init_ossl_x509name()
 {
     VALUE utf8str, ptrstr, ia5str, hash;
