@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509store.c 11708 2007-02-12 23:01:19Z shyouhei $
+ * $Id: ossl_x509store.c 27437 2010-04-22 08:04:13Z nobu $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -9,7 +9,6 @@
  * (See the file 'LICENCE'.)
  */
 #include "ossl.h"
-#include <rubysig.h>
 
 #define WrapX509Store(klass, obj, st) do { \
     if (!st) { \
@@ -55,7 +54,7 @@ VALUE eX509StoreError;
 /*
  * Public functions
  */
-VALUE 
+VALUE
 ossl_x509store_new(X509_STORE *store)
 {
     VALUE obj;
@@ -77,19 +76,19 @@ GetX509StorePtr(VALUE obj)
 
 X509_STORE *
 DupX509StorePtr(VALUE obj)
-{   
+{
     X509_STORE *store;
 
     SafeGetX509Store(obj, store);
     CRYPTO_add(&store->references, 1, CRYPTO_LOCK_X509_STORE);
-    
+
     return store;
 }
 
 /*
  * Private functions
  */
-static VALUE 
+static VALUE
 ossl_x509store_alloc(VALUE klass)
 {
     X509_STORE *store;
@@ -118,12 +117,20 @@ ossl_x509store_set_vfy_cb(VALUE self, VALUE cb)
     return cb;
 }
 
+
+/*
+ * call-seq:
+ *    X509::Store.new => store
+ *
+ */
 static VALUE
 ossl_x509store_initialize(int argc, VALUE *argv, VALUE self)
 {
     X509_STORE *store;
 
+/* BUG: This method takes any number of arguments but appears to ignore them. */
     GetX509Store(self, store);
+    store->ex_data.sk = NULL;
     X509_STORE_set_verify_cb_func(store, ossl_verify_cb);
     ossl_x509store_set_vfy_cb(self, Qnil);
 
@@ -164,7 +171,7 @@ ossl_x509store_set_purpose(VALUE self, VALUE purpose)
 #if (OPENSSL_VERSION_NUMBER >= 0x00907000L)
     X509_STORE *store;
     long p = NUM2LONG(purpose);
-    
+
     GetX509Store(self, store);
     X509_STORE_set_purpose(store, p);
 #else
@@ -190,14 +197,14 @@ ossl_x509store_set_trust(VALUE self, VALUE trust)
     return trust;
 }
 
-static VALUE 
+static VALUE
 ossl_x509store_set_time(VALUE self, VALUE time)
 {
     rb_iv_set(self, "@time", time);
     return time;
 }
 
-static VALUE 
+static VALUE
 ossl_x509store_add_file(VALUE self, VALUE file)
 {
     X509_STORE *store;
@@ -205,8 +212,8 @@ ossl_x509store_add_file(VALUE self, VALUE file)
     char *path = NULL;
 
     if(file != Qnil){
-        Check_SafeStr(file);
-	path = RSTRING(file)->ptr;
+        SafeStringValue(file);
+	path = RSTRING_PTR(file);
     }
     GetX509Store(self, store);
     lookup = X509_STORE_add_lookup(store, X509_LOOKUP_file());
@@ -218,7 +225,7 @@ ossl_x509store_add_file(VALUE self, VALUE file)
     return self;
 }
 
-static VALUE 
+static VALUE
 ossl_x509store_add_path(VALUE self, VALUE dir)
 {
     X509_STORE *store;
@@ -226,8 +233,8 @@ ossl_x509store_add_path(VALUE self, VALUE dir)
     char *path = NULL;
 
     if(dir != Qnil){
-        Check_SafeStr(dir);
-	path = RSTRING(dir)->ptr;
+        SafeStringValue(dir);
+	path = RSTRING_PTR(dir);
     }
     GetX509Store(self, store);
     lookup = X509_STORE_add_lookup(store, X509_LOOKUP_hash_dir());
@@ -286,7 +293,7 @@ static VALUE ossl_x509stctx_get_err(VALUE);
 static VALUE ossl_x509stctx_get_err_string(VALUE);
 static VALUE ossl_x509stctx_get_chain(VALUE);
 
-static VALUE 
+static VALUE
 ossl_x509store_verify(int argc, VALUE *argv, VALUE self)
 {
     VALUE cert, chain;
@@ -343,7 +350,7 @@ ossl_x509stctx_free(X509_STORE_CTX *ctx)
     X509_STORE_CTX_free(ctx);
 }
 
-static VALUE 
+static VALUE
 ossl_x509stctx_alloc(VALUE klass)
 {
     X509_STORE_CTX *ctx;
@@ -435,7 +442,7 @@ ossl_x509stctx_get_chain(VALUE self)
     return ary;
 }
 
-static VALUE 
+static VALUE
 ossl_x509stctx_get_err(VALUE self)
 {
     X509_STORE_CTX *ctx;
@@ -451,12 +458,12 @@ ossl_x509stctx_set_error(VALUE self, VALUE err)
     X509_STORE_CTX *ctx;
 
     GetX509StCtx(self, ctx);
-    X509_STORE_CTX_set_error(ctx, FIX2INT(err));
+    X509_STORE_CTX_set_error(ctx, NUM2INT(err));
 
     return err;
 }
 
-static VALUE 
+static VALUE
 ossl_x509stctx_get_err_string(VALUE self)
 {
     X509_STORE_CTX *ctx;
@@ -468,7 +475,7 @@ ossl_x509stctx_get_err_string(VALUE self)
     return rb_str_new2(X509_verify_cert_error_string(err));
 }
 
-static VALUE 
+static VALUE
 ossl_x509stctx_get_err_depth(VALUE self)
 {
     X509_STORE_CTX *ctx;
@@ -478,7 +485,7 @@ ossl_x509stctx_get_err_depth(VALUE self)
     return INT2FIX(X509_STORE_CTX_get_error_depth(ctx));
 }
 
-static VALUE 
+static VALUE
 ossl_x509stctx_get_curr_cert(VALUE self)
 {
     X509_STORE_CTX *ctx;
@@ -550,6 +557,10 @@ ossl_x509stctx_set_trust(VALUE self, VALUE trust)
     return trust;
 }
 
+/*
+ * call-seq:
+ *    storectx.time = time => time
+ */
 static VALUE
 ossl_x509stctx_set_time(VALUE self, VALUE time)
 {
@@ -566,7 +577,7 @@ ossl_x509stctx_set_time(VALUE self, VALUE time)
 /*
  * INIT
  */
-void 
+void
 Init_ossl_x509store()
 {
     VALUE x509stctx;
