@@ -4,6 +4,7 @@ case RUBY_PLATFORM
 when /(ms|bcc)win32|mingw/
   test_func = "WSACleanup"
   have_library("ws2_32", "WSACleanup")
+  $defs << "-DHAVE_SOCKETPAIR"
 when /cygwin/
   test_func = "socket"
 when /beos/
@@ -33,11 +34,13 @@ if have_header("arpa/inet.h")
 end
 
 ipv6 = false
-default_ipv6 = /cygwin|beos|haiku/ !~ RUBY_PLATFORM
+default_ipv6 = /mswin|cygwin|beos|haiku/ !~ RUBY_PLATFORM
 if enable_config("ipv6", default_ipv6)
   if checking_for("ipv6") {try_link(<<EOF)}
 #include <sys/types.h>
+#ifndef _WIN32
 #include <sys/socket.h>
+#endif
 int
 main()
 {
@@ -50,6 +53,9 @@ EOF
 end
 
 if ipv6
+  if $mingw
+    $CPPFLAGS << " -D_WIN32_WINNT=0x501"
+  end
   ipv6lib = nil
   class << (fmt = "unknown")
     def %(s) s || self end
@@ -92,24 +98,33 @@ end
 #   doug's fix, NOW add -Dss_family... only if required!
 doug = proc {have_struct_member("struct sockaddr_storage", "ss_family", headers)}
 if (doug[] or
-    with_cppflags($CPPFLAGS + " -Dss_family=__ss_family -Dss_len=__ss_len", &doug))
+    with_cppflags($CPPFLAGS + " -Dss_family=__ss_family", &doug))
   $defs[-1] = "-DHAVE_SOCKADDR_STORAGE"
+  doug = proc {have_struct_member("struct sockaddr_storage", "ss_len", headers)}
+  doug[] or with_cppflags($CPPFLAGS + " -Dss_len=__ss_len", &doug)
 end
 
 if have_struct_member("struct sockaddr", "sa_len", headers)
   $defs[-1] = "-DHAVE_SA_LEN "
 end
 
-have_header("netinet/tcp.h") if not /cygwin/ =~ RUBY_PLATFORM # for cygwin 1.1.5
+have_header("netinet/tcp.h") if /cygwin/ !~ RUBY_PLATFORM # for cygwin 1.1.5
 have_header("netinet/udp.h")
 
-if have_func("sendmsg") | have_func("recvmsg")
+if !have_macro("IPPROTO_IPV6", headers) && have_const("IPPROTO_IPV6", headers)
+  IO.read(File.join(File.dirname(__FILE__), "mkconstants.rb")).sub(/\A.*^__END__$/m, '').split(/\r?\n/).grep(/\AIPPROTO_\w*/){$&}.each {|name|
+    have_const(name, headers) unless $defs.include?("-DHAVE_CONST_#{name.upcase}")
+  }
+end
+
+if (have_func("sendmsg") | have_func("recvmsg")) && /64-darwin/ !~ RUBY_PLATFORM
+  # CMSG_ macros are broken on 64bit darwin, because of use of __DARWIN_ALIGN.
   have_struct_member('struct msghdr', 'msg_control', ['sys/types.h', 'sys/socket.h'])
   have_struct_member('struct msghdr', 'msg_accrights', ['sys/types.h', 'sys/socket.h'])
 end
 
-getaddr_info_ok = enable_config("wide-getaddrinfo") do
-  checking_for("wide getaddrinfo") {try_run(<<EOF)}
+getaddr_info_ok = (enable_config("wide-getaddrinfo") && :wide) ||
+  (checking_for("wide getaddrinfo") {try_run(<<EOF)} && :os)
 #{cpp_include(headers)}
 #include <stdlib.h>
 
@@ -217,7 +232,6 @@ main()
   exit(EXIT_FAILURE);
 }
 EOF
-end
 if ipv6 and not getaddr_info_ok
   abort <<EOS
 
@@ -242,20 +256,11 @@ Fatal: invalid value for --with-lookup-order-hack (expected INET, INET6 or UNSPE
 EOS
 end
 
-$objs = ["socket.#{$OBJEXT}"]
-
-unless getaddr_info_ok and have_func("getnameinfo", headers) and have_func("getaddrinfo", headers)
-  if have_struct_member("struct in6_addr", "s6_addr8", headers)
-    $defs[-1] = "-DHAVE_ADDR8"
-  end
-  $CPPFLAGS="-I. "+$CPPFLAGS
-  $objs += ["getaddrinfo.#{$OBJEXT}"]
-  $objs += ["getnameinfo.#{$OBJEXT}"]
-  have_func("inet_ntop") or have_func("inet_ntoa")
-  have_func("inet_pton") or have_func("inet_aton")
-  have_func("getservbyport")
-  if have_func("gai_strerror")
-    unless checking_for("gai_strerror() returns const pointer") {!try_compile(<<EOF)}
+have_type("struct addrinfo", headers)
+have_func("freehostent")
+have_func("freeaddrinfo")
+if /haiku/ !~ RUBY_PLATFORM and have_func("gai_strerror")
+  if checking_for("gai_strerror() returns const pointer") {!try_compile(<<EOF)}
 #{cpp_include(headers)}
 #include <stdlib.h>
 void
@@ -264,26 +269,87 @@ conftest_gai_strerror_is_const()
     *gai_strerror(0) = 0;
 }
 EOF
-      $defs << "-DGAI_STRERROR_CONST"
-    end
+    $defs << "-DGAI_STRERROR_CONST"
   end
+end
+
+$objs = [
+  "init.#{$OBJEXT}",
+  "constants.#{$OBJEXT}",
+  "basicsocket.#{$OBJEXT}",
+  "socket.#{$OBJEXT}",
+  "ipsocket.#{$OBJEXT}",
+  "tcpsocket.#{$OBJEXT}",
+  "tcpserver.#{$OBJEXT}",
+  "sockssocket.#{$OBJEXT}",
+  "udpsocket.#{$OBJEXT}",
+  "unixsocket.#{$OBJEXT}",
+  "unixserver.#{$OBJEXT}",
+  "option.#{$OBJEXT}",
+  "ancdata.#{$OBJEXT}",
+  "raddrinfo.#{$OBJEXT}"
+]
+
+if getaddr_info_ok == :wide or
+    !have_func("getnameinfo", headers) or !have_func("getaddrinfo", headers)
+  if have_struct_member("struct in6_addr", "s6_addr8", headers)
+    $defs[-1] = "s6_addr=s6_addr8"
+  end
+  if ipv6 == "kame" && have_struct_member("struct in6_addr", "s6_addr32", headers)
+    $defs[-1] = "-DFAITH"
+  end
+  $CPPFLAGS="-I. "+$CPPFLAGS
+  $objs += ["getaddrinfo.#{$OBJEXT}"]
+  $objs += ["getnameinfo.#{$OBJEXT}"]
+  $defs << "-DGETADDRINFO_EMU"
+  have_func("inet_ntop") or have_func("inet_ntoa")
+  have_func("inet_pton") or have_func("inet_aton")
+  have_func("getservbyport")
   have_header("arpa/nameser.h")
   have_header("resolv.h")
 end
 
+have_header("ifaddrs.h")
+have_func("getifaddrs")
+have_header("sys/ioctl.h")
+have_header("sys/sockio.h")
+have_header("net/if.h", headers)
+
+have_header("sys/param.h", headers)
+have_header("sys/ucred.h", headers)
+
 unless have_type("socklen_t", headers)
   $defs << "-Dsocklen_t=int"
 end
 
 have_header("sys/un.h")
 have_header("sys/uio.h")
+have_type("struct in_pktinfo", headers) {|src|
+  src.sub(%r'^/\*top\*/', '\1'"\n#if defined(IPPROTO_IP) && defined(IP_PKTINFO)") <<
+  "#else\n" << "#error\n" << ">>>>>> no in_pktinfo <<<<<<\n" << "#endif\n"
+} and have_struct_member("struct in_pktinfo", "ipi_spec_dst", headers)
+have_type("struct in6_pktinfo", headers) {|src|
+  src.sub(%r'^/\*top\*/', '\1'"\n#if defined(IPPROTO_IPV6) && defined(IPV6_PKTINFO)") <<
+  "#else\n" << "#error\n" << ">>>>>> no in6_pktinfo <<<<<<\n" << "#endif\n"
+}
 
-$distcleanfiles << "constants.h"
+have_type("struct sockcred", headers)
+have_type("struct cmsgcred", headers)
+
+have_func("getpeereid")
+
+have_header("ucred.h", headers)
+have_func("getpeerucred")
+
+# workaround for recent Windows SDK
+$defs << "-DIPPROTO_IPV6=IPPROTO_IPV6" if $defs.include?("-DHAVE_CONST_IPPROTO_IPV6") && !have_macro("IPPROTO_IPV6")
+
+$distcleanfiles << "constants.h" << "constdefs.*"
 
 if have_func(test_func)
   have_func("hsterror")
   have_func("getipnodebyname") or have_func("gethostbyname2")
-  have_func("socketpair")
+  have_func("socketpair") unless $defs.include?("-DHAVE_SOCKETPAIR")
   unless have_func("gethostname")
     have_func("uname")
   end
