@@ -1,15 +1,14 @@
 /*
  * emitter.c
  *
- * $Author: matz $
- * $Date: 2006/05/03 17:29:34 $
+ * $Author: nobu $
  *
  * Copyright (C) 2003 why the lucky stiff
- * 
+ *
  * All Base64 code from Ruby's pack.c.
- * Ruby is Copyright (C) 1993-2003 Yukihiro Matsumoto 
+ * Ruby is Copyright (C) 1993-2007 Yukihiro Matsumoto
  */
-#include "ruby.h"
+#include "ruby/ruby.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -18,7 +17,7 @@
 
 #define DEFAULT_ANCHOR_FORMAT "id%03d"
 
-const char hex_table[] = 
+const char hex_table[] =
 "0123456789ABCDEF";
 static char b64_table[] =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -98,7 +97,7 @@ syck_base64dec( char *s, long len )
         }
     }
     *end = '\0';
-    /*RSTRING(buf)->len = ptr - RSTRING(buf)->ptr;*/
+    /*RSTRING_LEN(buf) = ptr - RSTRING_PTR(buf);*/
     return ptr;
 }
 
@@ -106,7 +105,7 @@ syck_base64dec( char *s, long len )
  * Allocate an emitter
  */
 SyckEmitter *
-syck_new_emitter()
+syck_new_emitter(void)
 {
     SyckEmitter *e;
     e = S_ALLOC( SyckEmitter );
@@ -132,7 +131,7 @@ syck_new_emitter()
     e->output_handler = NULL;
     e->lvl_idx = 0;
     e->lvl_capa = ALLOC_CT;
-    e->levels = S_ALLOC_N( SyckLevel, e->lvl_capa ); 
+    e->levels = S_ALLOC_N( SyckLevel, e->lvl_capa );
     syck_emitter_reset_levels( e );
     e->bonus = NULL;
     return e;
@@ -198,7 +197,7 @@ syck_emitter_pop_level( SyckEmitter *e )
     free( e->levels[e->lvl_idx].domain );
 }
 
-void 
+void
 syck_emitter_add_level( SyckEmitter *e, int len, enum syck_level_status status )
 {
     ASSERT( e != NULL );
@@ -282,20 +281,20 @@ syck_emitter_clear( SyckEmitter *e )
  * Raw write to the emitter buffer.
  */
 void
-syck_emitter_write( SyckEmitter *e, char *str, long len )
+syck_emitter_write( SyckEmitter *e, const char *str, long len )
 {
     long at;
-    ASSERT( str != NULL )
+    ASSERT( str != NULL );
     if ( e->buffer == NULL )
     {
         syck_emitter_clear( e );
     }
-    
+
     /*
      * Flush if at end of buffer
      */
     at = e->marker - e->buffer;
-    if ( len + at >= e->bufsize )
+    if ( len + at >= (long)e->bufsize )
     {
         syck_emitter_flush( e, 0 );
 	for (;;) {
@@ -327,7 +326,7 @@ syck_emitter_flush( SyckEmitter *e, long check_room )
      */
     if ( check_room > 0 )
     {
-        if ( e->bufsize > ( e->marker - e->buffer ) + check_room )
+        if ( (long)e->bufsize > ( e->marker - e->buffer ) + check_room )
         {
             return;
         }
@@ -361,7 +360,7 @@ syck_emit( SyckEmitter *e, st_data_t n )
     int indent = 0;
     long x = 0;
     SyckLevel *lvl = syck_emitter_current_level( e );
-    
+
     /*
      * Determine headers.
      */
@@ -392,7 +391,7 @@ syck_emit( SyckEmitter *e, st_data_t n )
     /* Look for anchor */
     if ( e->anchors != NULL &&
         st_lookup( e->markers, n, (st_data_t *)&oid ) &&
-        st_lookup( e->anchors, (st_data_t)oid, (st_data_t *)&anchor_name ) )
+        st_lookup( e->anchors, (st_data_t)oid, (void *)&anchor_name ) )
     {
         if ( e->anchored == NULL )
         {
@@ -438,7 +437,7 @@ end_emit:
  * and the implicit tag which would be assigned to this node.  If a tag is
  * required, write the tag.
  */
-void syck_emit_tag( SyckEmitter *e, char *tag, char *ignore )
+void syck_emit_tag( SyckEmitter *e, const char *tag, const char *ignore )
 {
     SyckLevel *lvl;
     if ( tag == NULL ) return;
@@ -457,10 +456,10 @@ void syck_emit_tag( SyckEmitter *e, char *tag, char *ignore )
             int skip = 4 + strlen( YAML_DOMAIN ) + 1;
             syck_emitter_write( e, tag + skip, taglen - skip );
         } else {
-            char *subd = tag + 4;
+            const char *subd = tag + 4;
             while ( *subd != ':' && *subd != '\0' ) subd++;
             if ( *subd == ':' ) {
-                if ( subd - tag > ( strlen( YAML_DOMAIN ) + 5 ) &&
+                if ( subd - tag > ( (long)( strlen( YAML_DOMAIN ) + 5 )) &&
                      strncmp( subd - strlen( YAML_DOMAIN ), YAML_DOMAIN, strlen( YAML_DOMAIN ) ) == 0 ) {
                     syck_emitter_write( e, tag + 4, subd - strlen( YAML_DOMAIN ) - ( tag + 4 ) - 1 );
                     syck_emitter_write( e, "/", 1 );
@@ -486,7 +485,7 @@ void syck_emit_tag( SyckEmitter *e, char *tag, char *ignore )
     lvl->anctag = 1;
 }
 
-/* 
+/*
  * Emit a newline and an appropriately spaced indent.
  */
 void syck_emit_indent( SyckEmitter *e )
@@ -539,7 +538,7 @@ void syck_emit_indent( SyckEmitter *e )
  * Basic printable test for LATIN-1 characters.
  */
 int
-syck_scan_scalar( int req_width, char *cursor, long len )
+syck_scan_scalar( int req_width, const char *cursor, long len )
 {
     long i = 0, start = 0;
     int flags = SCAN_NONE;
@@ -570,8 +569,8 @@ syck_scan_scalar( int req_width, char *cursor, long len )
     } else if ( len > 1 && cursor[len-2] == '\n' ) {
         flags |= SCAN_MANYNL_E;
     }
-    if ( 
-        ( len > 0 && ( cursor[0] == ' ' || cursor[0] == '\t' ) ) ||
+    if (
+        ( len > 0 && ( cursor[0] == ' ' || cursor[0] == '\t' || cursor[0] == '\n' || cursor[0] == '\r' ) ) ||
         ( len > 1 && ( cursor[len-1] == ' ' || cursor[len-1] == '\t' ) )
     ) {
         flags |= SCAN_WHITEEDGE;
@@ -595,7 +594,7 @@ syck_scan_scalar( int req_width, char *cursor, long len )
             flags |= SCAN_NEWLINE;
             if ( len - i >= 3 && strncmp( &cursor[i+1], "---", 3 ) == 0 )
                 flags |= SCAN_DOCSEP;
-            if ( cursor[i+1] == ' ' || cursor[i+1] == '\t' ) 
+            if ( cursor[i+1] == ' ' || cursor[i+1] == '\t' )
                 flags |= SCAN_INDENTED;
             if ( req_width > 0 && i - start > req_width )
                 flags |= SCAN_WIDE;
@@ -619,12 +618,12 @@ syck_scan_scalar( int req_width, char *cursor, long len )
         }
         /* remember, if plain collections get implemented, to add nb-plain-flow-char */
         else if ( ( cursor[i] == ' ' && cursor[i+1] == '#' ) ||
-                  ( cursor[i] == ':' && 
+                  ( cursor[i] == ':' &&
                     ( cursor[i+1] == ' ' || cursor[i+1] == '\n' || i == len - 1 ) ) )
         {
             flags |= SCAN_INDIC_C;
         }
-        else if ( cursor[i] == ',' && 
+        else if ( cursor[i] == ',' &&
                   ( cursor[i+1] == ' ' || cursor[i+1] == '\n' || i == len - 1 ) )
         {
             flags |= SCAN_FLOWMAP;
@@ -639,37 +638,38 @@ syck_scan_scalar( int req_width, char *cursor, long len )
  * All scalars should be emitted through this function, which determines an appropriate style,
  * tag and indent.
  */
-void syck_emit_scalar( SyckEmitter *e, char *tag, enum scalar_style force_style, int force_indent, int force_width,
-                       char keep_nl, char *str, long len )
+void syck_emit_scalar( SyckEmitter *e, const char *tag, enum scalar_style force_style, int force_indent, int force_width,
+                       char keep_nl, const char *str, long len )
 {
     enum scalar_style favor_style = scalar_literal;
     SyckLevel *parent = syck_emitter_parent_level( e );
     SyckLevel *lvl = syck_emitter_current_level( e );
     int scan = 0;
+    const char *match_implicit;
     char *implicit;
-    
+
     if ( str == NULL ) str = "";
 
     /* No empty nulls as map keys */
-    if ( len == 0 && ( parent->status == syck_lvl_map || parent->status == syck_lvl_imap ) && 
-         parent->ncount % 2 == 1 && syck_tagcmp( tag, "tag:yaml.org,2002:null" ) == 0 ) 
+    if ( len == 0 && ( parent->status == syck_lvl_map || parent->status == syck_lvl_imap ) &&
+         parent->ncount % 2 == 1 && syck_tagcmp( tag, "tag:yaml.org,2002:null" ) == 0 )
     {
         str = "~";
         len = 1;
     }
 
     scan = syck_scan_scalar( force_width, str, len );
-    implicit = syck_match_implicit( str, len );
+    match_implicit = syck_match_implicit( str, len );
 
     /* quote strings which default to implicits */
-    implicit = syck_taguri( YAML_DOMAIN, implicit, strlen( implicit ) );
+    implicit = syck_taguri( YAML_DOMAIN, match_implicit, strlen( match_implicit ) );
     if ( syck_tagcmp( tag, implicit ) != 0 && syck_tagcmp( tag, "tag:yaml.org,2002:str" ) == 0 ) {
         force_style = scalar_2quote;
     } else {
         /* complex key */
         if ( parent->status == syck_lvl_map && parent->ncount % 2 == 1 &&
-             ( !( tag == NULL || 
-             ( implicit != NULL && syck_tagcmp( tag, implicit ) == 0 && e->explicit_typing == 0 ) ) ) ) 
+             ( !( tag == NULL ||
+             ( implicit != NULL && syck_tagcmp( tag, implicit ) == 0 && e->explicit_typing == 0 ) ) ) )
         {
             syck_emitter_write( e, "? ", 2 );
             parent->status = syck_lvl_mapx;
@@ -773,7 +773,7 @@ void syck_emit_scalar( SyckEmitter *e, char *tag, enum scalar_style force_style,
 }
 
 void
-syck_emitter_escape( SyckEmitter *e, char *src, long len )
+syck_emitter_escape( SyckEmitter *e, const char *src, long len )
 {
     int i;
     for( i = 0; i < len; i++ )
@@ -786,8 +786,8 @@ syck_emitter_escape( SyckEmitter *e, char *src, long len )
             else
             {
                 syck_emitter_write( e, "x", 1 );
-                syck_emitter_write( e, (char *)hex_table + ((src[i] & 0xF0) >> 4), 1 );
-                syck_emitter_write( e, (char *)hex_table + (src[i] & 0x0F), 1 );
+                syck_emitter_write( e, (const char *)hex_table + ((src[i] & 0xF0) >> 4), 1 );
+                syck_emitter_write( e, (const char *)hex_table + (src[i] & 0x0F), 1 );
             }
         }
         else
@@ -802,12 +802,13 @@ syck_emitter_escape( SyckEmitter *e, char *src, long len )
 /*
  * Outputs a single-quoted block.
  */
-void syck_emit_1quoted( SyckEmitter *e, int width, char *str, long len )
+void
+syck_emit_1quoted( SyckEmitter *e, int width, const char *str, long len )
 {
     char do_indent = 0;
-    char *mark = str;
-    char *start = str;
-    char *end = str;
+    const char *mark = str;
+    const char *start = str;
+    const char *end = str;
     syck_emitter_write( e, "'", 1 );
     while ( mark < str + len ) {
         if ( do_indent ) {
@@ -849,12 +850,13 @@ void syck_emit_1quoted( SyckEmitter *e, int width, char *str, long len )
 /*
  * Outputs a double-quoted block.
  */
-void syck_emit_2quoted( SyckEmitter *e, int width, char *str, long len )
+void
+syck_emit_2quoted( SyckEmitter *e, int width, const char *str, long len )
 {
     char do_indent = 0;
-    char *mark = str;
-    char *start = str;
-    char *end = str;
+    const char *mark = str;
+    const char *start = str;
+    const char *end = str;
     syck_emitter_write( e, "\"", 1 );
     while ( mark < str + len ) {
         if ( do_indent > 0 ) {
@@ -909,11 +911,12 @@ void syck_emit_2quoted( SyckEmitter *e, int width, char *str, long len )
 /*
  * Outputs a literal block.
  */
-void syck_emit_literal( SyckEmitter *e, char keep_nl, char *str, long len )
+void
+syck_emit_literal( SyckEmitter *e, char keep_nl, const char *str, long len )
 {
-    char *mark = str;
-    char *start = str;
-    char *end = str;
+    const char *mark = str;
+    const char *start = str;
+    const char *end = str;
     syck_emitter_write( e, "|", 1 );
     if ( keep_nl == NL_CHOMP ) {
         syck_emitter_write( e, "-", 1 );
@@ -944,11 +947,12 @@ void syck_emit_literal( SyckEmitter *e, char keep_nl, char *str, long len )
 /*
  * Outputs a folded block.
  */
-void syck_emit_folded( SyckEmitter *e, int width, char keep_nl, char *str, long len )
+void
+syck_emit_folded( SyckEmitter *e, int width, char keep_nl, const char *str, long len )
 {
-    char *mark = str;
-    char *start = str;
-    char *end = str;
+    const char *mark = str;
+    const char *start = str;
+    const char *end = str;
     syck_emitter_write( e, ">", 1 );
     if ( keep_nl == NL_CHOMP ) {
         syck_emitter_write( e, "-", 1 );
@@ -993,7 +997,7 @@ void syck_emit_folded( SyckEmitter *e, int width, char keep_nl, char *str, long
 /*
  * Begins emission of a sequence.
  */
-void syck_emit_seq( SyckEmitter *e, char *tag, enum seq_style style )
+void syck_emit_seq( SyckEmitter *e, const char *tag, enum seq_style style )
 {
     SyckLevel *parent = syck_emitter_parent_level( e );
     SyckLevel *lvl = syck_emitter_current_level( e );
@@ -1014,7 +1018,8 @@ void syck_emit_seq( SyckEmitter *e, char *tag, enum seq_style style )
 /*
  * Begins emission of a mapping.
  */
-void syck_emit_map( SyckEmitter *e, char *tag, enum map_style style )
+void
+syck_emit_map( SyckEmitter *e, const char *tag, enum map_style style )
 {
     SyckLevel *parent = syck_emitter_parent_level( e );
     SyckLevel *lvl = syck_emitter_current_level( e );
@@ -1218,10 +1223,10 @@ syck_emitter_mark_node( SyckEmitter *e, st_data_t n )
             e->anchors = st_init_numtable();
         }
 
-        if ( ! st_lookup( e->anchors, (st_data_t)oid, (st_data_t *)&anchor_name ) )
+        if ( ! st_lookup( e->anchors, (st_data_t)oid, (void *)&anchor_name ) )
         {
             int idx = 0;
-            char *anc = ( e->anchor_format == NULL ? DEFAULT_ANCHOR_FORMAT : e->anchor_format );
+            const char *anc = ( e->anchor_format == NULL ? DEFAULT_ANCHOR_FORMAT : e->anchor_format );
 
             /*
              * Second time hitting this object, let's give it an anchor
