@@ -15,10 +15,6 @@ module Tk
   end
 end
 
-Tk.__set_toplevel_aliases__(:Ttk, Tk::Tile::Labelframe, 
-                            :TkLabelframe, :TkLabelFrame)
-
-
 class Tk::Tile::TLabelframe < Tk::Tile::TFrame
   include Tk::Tile::TileWidget
 
@@ -28,9 +24,15 @@ class Tk::Tile::TLabelframe < Tk::Tile::TFrame
     TkCommandNames = ['::tlabelframe'.freeze].freeze
   end
   WidgetClassName = 'TLabelframe'.freeze
-  WidgetClassNames[WidgetClassName] = self
+  WidgetClassNames[WidgetClassName] ||= self
 
   def self.style(*args)
     [self::WidgetClassName, *(args.map!{|a| _get_eval_string(a)})].join('.')
   end
 end
+
+#Tk.__set_toplevel_aliases__(:Ttk, Tk::Tile::Labelframe,
+#                            :TkLabelframe, :TkLabelFrame)
+Tk.__set_loaded_toplevel_aliases__('tkextlib/tile/tlabelframe.rb',
+                                   :Ttk, Tk::Tile::Labelframe,
+                                   :TkLabelframe, :TkLabelFrame)
