@@ -1,3 +1,4 @@
+# -*- coding: euc-jp -*-
 #
 # arrowhead widget demo (called by 'widget')
 #
@@ -24,67 +25,75 @@ def arrowSetup(c)
 
   # Create the arrow and outline.
   c.delete('all')
-  TkcLine.new(c, v.x1, v.y, v.x2, v.y, 
-              { 'width'=>10 * v.width, 
-                'arrowshape'=>[10*v.a, 10*v.b, 10*v.c], 
-                'arrow'=>'last' 
+  TkcLine.new(c, v.x1, v.y, v.x2, v.y,
+              { 'width'=>10 * v.width,
+                'arrowshape'=>[10*v.a, 10*v.b, 10*v.c],
+                'arrow'=>'last'
               }.update(v.bigLineStyle) )
   xtip = v.x2 - 10*v.b
   deltaY = 10*v.c + 5*v.width
-  TkcLine.new(c, v.x2, v.y, xtip, v.y + deltaY, 
-              v.x2 - 10*v.a, v.y, xtip, v.y - deltaY, v.x2, v.y, 
+  TkcLine.new(c, v.x2, v.y, xtip, v.y + deltaY,
+              v.x2 - 10*v.a, v.y, xtip, v.y - deltaY, v.x2, v.y,
               'width'=>2, 'capstyle'=>'round', 'joinstyle'=>'round')
 
   # Create the boxes for reshaping the line and arrowhead.
-  TkcRectangle.new(c, v.x2-10*v.a-5, v.y-5, v.x2-10*v.a+5, v.y+5, 
+  TkcRectangle.new(c, v.x2-10*v.a-5, v.y-5, v.x2-10*v.a+5, v.y+5,
                    {'tags'=>['box1', $arrowTag_box]}.update(v.boxStyle) )
-  TkcRectangle.new(c, xtip-5, v.y-deltaY-5, xtip+5, v.y-deltaY+5, 
+  TkcRectangle.new(c, xtip-5, v.y-deltaY-5, xtip+5, v.y-deltaY+5,
                    {'tags'=>['box2', $arrowTag_box]}.update(v.boxStyle) )
-  TkcRectangle.new(c, v.x1-5, v.y-5*v.width-5, v.x1+5, v.y-5*v.width+5, 
+  TkcRectangle.new(c, v.x1-5, v.y-5*v.width-5, v.x1+5, v.y-5*v.width+5,
                    {'tags'=>['box3', $arrowTag_box]}.update(v.boxStyle) )
   c.itemconfigure cur, v.activeStyle if cur
 
   # Create three arrows in actual size with the same parameters
   TkcLine.new(c, v.x2+50, 0, v.x2+50, 1000, 'width'=>2)
   tmp = v.x2+100
-  TkcLine.new(c, tmp, v.y-125, tmp, v.y-75, 'width'=>v.width, 
+  TkcLine.new(c, tmp, v.y-125, tmp, v.y-75, 'width'=>v.width,
               'arrow'=>'both', 'arrowshape'=>[v.a, v.b, v.c])
-  TkcLine.new(c, tmp-25, v.y, tmp+25, v.y, 'width'=>v.width, 
+  TkcLine.new(c, tmp-25, v.y, tmp+25, v.y, 'width'=>v.width,
               'arrow'=>'both', 'arrowshape'=>[v.a, v.b, v.c])
-  TkcLine.new(c, tmp-25, v.y+75, tmp+25, v.y+125, 'width'=>v.width, 
+  TkcLine.new(c, tmp-25, v.y+75, tmp+25, v.y+125, 'width'=>v.width,
               'arrow'=>'both', 'arrowshape'=>[v.a, v.b, v.c])
 
-  # Create a bunch of other arrows and text items showing the 
+  # Create a bunch of other arrows and text items showing the
   # current dimensions.
   tmp = v.x2+10
-  TkcLine.new(c, tmp, v.y-5*v.width, tmp, v.y-deltaY, 
+  TkcLine.new(c, tmp, v.y-5*v.width, tmp, v.y-deltaY,
               'arrow'=>'both', 'arrowshape'=>v.smallTips)
   TkcText.new(c, v.x2+15, v.y-deltaY+5*v.c, 'text'=>v.c, 'anchor'=>'w')
   tmp = v.x1-10
-  TkcLine.new(c, tmp, v.y-5*v.width, tmp, v.y+5*v.width, 
+  TkcLine.new(c, tmp, v.y-5*v.width, tmp, v.y+5*v.width,
               'arrow'=>'both', 'arrowshape'=>v.smallTips)
   TkcText.new(c, v.x1-15, v.y, 'text'=>v.width, 'anchor'=>'e')
   tmp = v.y+5*v.width+10*v.c+10
-  TkcLine.new(c, v.x2-10*v.a, tmp, v.x2, tmp, 
+  TkcLine.new(c, v.x2-10*v.a, tmp, v.x2, tmp,
               'arrow'=>'both', 'arrowshape'=>v.smallTips)
   TkcText.new(c, v.x2-5*v.a, tmp+5, 'text'=>v.a, 'anchor'=>'n')
   tmp = tmp+25
-  TkcLine.new(c, v.x2-10*v.b, tmp, v.x2, tmp, 
+  TkcLine.new(c, v.x2-10*v.b, tmp, v.x2, tmp,
               'arrow'=>'both', 'arrowshape'=>v.smallTips)
   TkcText.new(c, v.x2-5*v.b, tmp+5, 'text'=>v.b, 'anchor'=>'n')
 
-  TkcText.new(c, v.x1, 310, 'text'=>"'width'=>#{v.width}", 'anchor'=>'w', 
-              'font'=>'-*-Helvetica-Medium-R-Normal--*-180-*-*-*-*-*-*')
-  TkcText.new(c, v.x1, 330, 
-              'text'=>"'arrowshape'=>[#{v.a}, #{v.b}, #{v.c}]", 'anchor'=>'w', 
-              'font'=>'-*-Helvetica-Medium-R-Normal--*-180-*-*-*-*-*-*')
+  if $tk_version =~ /^4.*/
+    TkcText.new(c, v.x1, 310, 'text'=>"'width'=>#{v.width}", 'anchor'=>'w',
+                'font'=>'-*-Helvetica-Medium-R-Normal--*-180-*-*-*-*-*-*')
+    TkcText.new(c, v.x1, 330,
+                'text'=>"'arrowshape'=>[#{v.a}, #{v.b}, #{v.c}]",'anchor'=>'w',
+                'font'=>'-*-Helvetica-Medium-R-Normal--*-180-*-*-*-*-*-*')
+  else
+    TkcText.new(c, v.x1, 310, 'text'=>"'width'=>#{v.width}", 'anchor'=>'w',
+                'font'=>'Helvetica 18')
+    TkcText.new(c, v.x1, 330,
+                'text'=>"'arrowshape'=>[#{v.a}, #{v.b}, #{v.c}]",
+                'anchor'=>'w', 'font'=>'Helvetica 18')
+  end
 
   v.count += 1
 end
 
 # toplevel widget ¸ߤк
 if defined?($arrow_demo) && $arrow_demo
-  $arrow_demo.destroy 
+  $arrow_demo.destroy
   $arrow_demo = nil
 end
 
@@ -95,14 +104,16 @@ $arrow_demo = TkToplevel.new {|w|
   positionWindow(w)
 }
 
+base_frame = TkFrame.new($arrow_demo).pack(:fill=>:both, :expand=>true)
+
 # label 
-TkLabel.new($arrow_demo, 'font'=>$font, 'wraplength'=>'5i', 'justify'=>'left', 
+TkLabel.new(base_frame, 'font'=>$font, 'wraplength'=>'5i', 'justify'=>'left',
             'text'=>" widget ǡХǻȤ饤ˤĤ͡ƬηƤߤ뤳ȤǤޤηѤˤϡ礵줿ˤĤƤ 3ĤλͳѤɥåƤ¦̤礭ǤΥץ򼨤ƤޤΥƥȤϥ饤󥢥ƥФꥪץǤ"){
   pack('side'=>'top')
 }
 
 # frame 
-$arrow_buttons = TkFrame.new($arrow_demo) {|frame|
+$arrow_buttons = TkFrame.new(base_frame) {|frame|
   TkButton.new(frame) {
     #text 'λ'
     text 'Ĥ'
@@ -121,14 +132,14 @@ $arrow_buttons = TkFrame.new($arrow_demo) {|frame|
 $arrow_buttons.pack('side'=>'bottom', 'fill'=>'x', 'pady'=>'2m')
 
 # canvas 
-$arrow_canvas = TkCanvas.new($arrow_demo, 'width'=>500, 'height'=>350, 
+$arrow_canvas = TkCanvas.new(base_frame, 'width'=>500, 'height'=>350,
                              'relief'=>'sunken', 'borderwidth'=>2)
 $arrow_canvas.pack('expand'=>'yes', 'fill'=>'both')
 
 # 
 unless Struct.const_defined?("ArrowInfo")
-  $demo_arrowInfo = Struct.new("ArrowInfo", :a, :b, :c, :width, :motionProc, 
-                               :x1, :x2, :y, :smallTips, :count, 
+  $demo_arrowInfo = Struct.new("ArrowInfo", :a, :b, :c, :width, :motionProc,
+                               :x1, :x2, :y, :smallTips, :count,
                                :bigLineStyle, :boxStyle, :activeStyle).new
 end
 $demo_arrowInfo.a = 8
@@ -146,7 +157,7 @@ if TkWinfo.depth($arrow_canvas) > 1
   $demo_arrowInfo.boxStyle = {'fill'=>'', 'outline'=>'black', 'width'=>1}
   $demo_arrowInfo.activeStyle = {'fill'=>'red', 'outline'=>'black', 'width'=>1}
 else
-  $demo_arrowInfo.bigLineStyle = {'fill'=>'black', 
+  $demo_arrowInfo.bigLineStyle = {'fill'=>'black',
     'stipple'=>'@'+[$demo_dir,'..','images','grey.25'].join(File::Separator)}
   $demo_arrowInfo.boxStyle = {'fill'=>'', 'outline'=>'black', 'width'=>1}
   $demo_arrowInfo.activeStyle = {'fill'=>'black','outline'=>'black','width'=>1}
@@ -157,16 +168,16 @@ $arrowTag_box.bind('Enter', proc{$arrow_canvas.itemconfigure('current', $demo_ar
 $arrowTag_box.bind('Leave', proc{$arrow_canvas.itemconfigure('current', $demo_arrowInfo.boxStyle)})
 $arrowTag_box.bind('B1-Enter', proc{})
 $arrowTag_box.bind('B1-Leave', proc{})
-$arrow_canvas.itembind('box1', '1', 
+$arrow_canvas.itembind('box1', '1',
                        proc{$demo_arrowInfo.motionProc \
                          = proc{|x,y| arrowMove1 $arrow_canvas, x, y}})
-$arrow_canvas.itembind('box2', '1', 
+$arrow_canvas.itembind('box2', '1',
                        proc{$demo_arrowInfo.motionProc \
                          = proc{|x,y| arrowMove2 $arrow_canvas, x, y}})
-$arrow_canvas.itembind('box3', '1', 
+$arrow_canvas.itembind('box3', '1',
                        proc{$demo_arrowInfo.motionProc \
                          = proc{|x,y| arrowMove3 $arrow_canvas, x, y}})
-$arrowTag_box.bind('B1-Motion', 
+$arrowTag_box.bind('B1-Motion',
                   proc{|x,y| $demo_arrowInfo.motionProc.call(x,y)}, "%x %y")
 $arrow_canvas.bind('Any-ButtonRelease-1', proc{arrowSetup $arrow_canvas})
 
