@@ -1,10 +1,11 @@
+# -*- coding: euc-jp -*-
 #
 # iconic button widget demo (called by 'widget')
 #
 
 # toplevel widget ¸ߤк
 if defined?($icon_demo) && $icon_demo
-  $icon_demo.destroy 
+  $icon_demo.destroy
   $icon_demo = nil
 end
 
@@ -15,8 +16,10 @@ $icon_demo = TkToplevel.new {|w|
   positionWindow(w)
 }
 
+base_frame = TkFrame.new($icon_demo).pack(:fill=>:both, :expand=>true)
+
 # label 
-msg = TkLabel.new($icon_demo) {
+msg = TkLabel.new(base_frame) {
   font $font
   wraplength '5i'
   justify 'left'
@@ -25,7 +28,7 @@ msg = TkLabel.new($icon_demo) {
 msg.pack('side'=>'top')
 
 # frame 
-TkFrame.new($icon_demo) {|frame|
+TkFrame.new(base_frame) {|frame|
   TkButton.new(frame) {
     #text 'λ'
     text 'Ĥ'
@@ -60,16 +63,18 @@ TkBitmapImage.new('file'=>[$demo_dir,'..',
 letters = TkVariable.new
 
 # frame 
-TkFrame.new($icon_demo, 'borderwidth'=>10){|w|
+TkFrame.new(base_frame, 'borderwidth'=>10){|w|
   TkFrame.new(w) {|f|
-    TkRadioButton.new(f){
+    # TkRadioButton.new(f){
+    Tk::RadioButton.new(f){
       bitmap '@' + [$demo_dir,'..',
                      'images','letters.xbm'].join(File::Separator)
       variable letters
       value 'full'
     }.pack('side'=>'top', 'expand'=>'yes')
 
-    TkRadioButton.new(f){
+    # TkRadioButton.new(f){
+    Tk::RadioButton.new(f){
       bitmap '@' + [$demo_dir,'..',
                      'images','noletter.xbm'].join(File::Separator)
       variable letters
@@ -78,14 +83,16 @@ TkFrame.new($icon_demo, 'borderwidth'=>10){|w|
 
   }.pack('side'=>'left', 'expand'=>'yes', 'padx'=>'5m')
 
-  TkCheckButton.new(w) {
+  # TkCheckButton.new(w) {
+  Tk::CheckButton.new(w) {
     image flagdown
     selectimage flagup
     indicatoron 0
     selectcolor self['background']
   }.pack('side'=>'left', 'expand'=>'yes', 'padx'=>'5m')
 
-  TkCheckButton.new(w) {
+  # TkCheckButton.new(w) {
+  Tk::CheckButton.new(w) {
     bitmap '@' + [$demo_dir,'..',
                    'images','letters.xbm'].join(File::Separator)
     indicatoron 0
