@@ -1,5 +1,5 @@
 #
-# kroc.rb 
+# kroc.rb
 #
 # based on:
 #   >> kroc.tcl - Copyright (C) 2004 David Zolli <kroc@kroc.tk>
@@ -15,30 +15,30 @@ else
 end
 
 def kroc_rb_settings
-  # Tk::Tile::Style.default(TkRoot, :background=>'#FCB64F', 
+  # Tk::Tile::Style.default(TkRoot, :background=>'#FCB64F',
   #                         :troughcolor=>'#F8C278', :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default(Tk.root, :background=>'#FCB64F', 
+  # Tk::Tile::Style.default(Tk.root, :background=>'#FCB64F',
   #                         :troughcolor=>'#F8C278', :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default('.', :background=>'#FCB64F', 
+  # Tk::Tile::Style.default('.', :background=>'#FCB64F',
   #                         :troughcolor=>'#F8C278', :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default(nil, :background=>'#FCB64F', 
+  # Tk::Tile::Style.default(nil, :background=>'#FCB64F',
   #                         :troughcolor=>'#F8C278', :borderwidth=>1)
   #   or
-  Tk::Tile::Style.default(:background=>'#FCB64F', :troughcolor=>'#F8C278', 
+  Tk::Tile::Style.default(:background=>'#FCB64F', :troughcolor=>'#F8C278',
                           :borderwidth=>1)
-  # Tk::Tile::Style.default(TkRoot, :font=>Tk::Tile::Font::Default, 
+  # Tk::Tile::Style.default(TkRoot, :font=>Tk::Tile::Font::Default,
   #                         :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default(Tk.root, :font=>Tk::Tile::Font::Default, 
+  # Tk::Tile::Style.default(Tk.root, :font=>Tk::Tile::Font::Default,
   #                         :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default('.', :font=>Tk::Tile::Font::Default, 
+  # Tk::Tile::Style.default('.', :font=>Tk::Tile::Font::Default,
   #                         :borderwidth=>1)
   #   or
-  # Tk::Tile::Style.default(nil, :font=>Tk::Tile::Font::Default, 
+  # Tk::Tile::Style.default(nil, :font=>Tk::Tile::Font::Default,
   #                         :borderwidth=>1)
   #   or
   Tk::Tile::Style.default(:font=>Tk::Tile::Font::Default, :borderwidth=>1)
@@ -57,37 +57,37 @@ def kroc_rb_settings
   # Tk::Tile::Style.default('TButton', :padding=>[10,4])
   Tk::Tile::Style.default(Tk::Tile::TButton, :padding=>[10,4])
 
-  # Tk::Tile::Style.default('TNotebook.Tab', 
-  Tk::Tile::Style.default($TNotebook_Tab, 
+  # Tk::Tile::Style.default('TNotebook.Tab',
+  Tk::Tile::Style.default($TNotebook_Tab,
                           :padding=>[10, 3], :font=>Tk::Tile::Font::Default)
-  # Tk::Tile::Style.map('TNotebook.Tab', 
-  Tk::Tile::Style.map($TNotebook_Tab, 
-                      :background=>[:selected, '#FCB64F', '', '#FFE6BA'], 
-                      :foreground=>['', 'black'], 
+  # Tk::Tile::Style.map('TNotebook.Tab',
+  Tk::Tile::Style.map($TNotebook_Tab,
+                      :background=>[:selected, '#FCB64F', '', '#FFE6BA'],
+                      :foreground=>['', 'black'],
                       :padding=>[:selected, [10, 6, 10, 3]])
 
-  # Tk::Tile::Style.map('TScrollbar', 
-  Tk::Tile::Style.map(Tk::Tile::TScrollbar, 
-                      :background=>[:pressed, '#694418'], 
-                      :arrowcolor=>[:pressed, '#FEF7CB'], 
+  # Tk::Tile::Style.map('TScrollbar',
+  Tk::Tile::Style.map(Tk::Tile::TScrollbar,
+                      :background=>[:pressed, '#694418'],
+                      :arrowcolor=>[:pressed, '#FEF7CB'],
                       :relief=>[:pressed, :sunken])
 
-  # Tk::Tile::Style.layout('Vertical.TScrollbar', 
-  Tk::Tile::Style.layout(Tk::Tile.style('Vertical', Tk::Tile::TScrollbar), 
+  # Tk::Tile::Style.layout('Vertical.TScrollbar',
+  Tk::Tile::Style.layout(Tk::Tile.style('Vertical', Tk::Tile::TScrollbar),
                          ['Scrollbar.trough', {:children=>[
-                             'Scrollbar.uparrow',   {:side=>:top}, 
-                             'Scrollbar.downarrow', {:side=>:bottom}, 
-                             'Scrollbar.uparrow',   {:side=>:bottom}, 
+                             'Scrollbar.uparrow',   {:side=>:top},
+                             'Scrollbar.downarrow', {:side=>:bottom},
+                             'Scrollbar.uparrow',   {:side=>:bottom},
                              'Scrollbar.thumb',  {:side=>:top, :expand=>true}
                            ]}
                          ])
 
-  # Tk::Tile::Style.layout('Horizontal.TScrollbar', 
-  Tk::Tile::Style.layout(Tk::Tile.style('Horizontal', Tk::Tile::TScrollbar), 
+  # Tk::Tile::Style.layout('Horizontal.TScrollbar',
+  Tk::Tile::Style.layout(Tk::Tile.style('Horizontal', Tk::Tile::TScrollbar),
                          ['Scrollbar.trough', {:children=>[
-                             'Scrollbar.leftarrow',   {:side=>:left}, 
-                             'Scrollbar.rightarrow', {:side=>:right}, 
-                             'Scrollbar.leftarrow',   {:side=>:right}, 
+                             'Scrollbar.leftarrow',   {:side=>:left},
+                             'Scrollbar.rightarrow', {:side=>:right},
+                             'Scrollbar.leftarrow',   {:side=>:right},
                              'Scrollbar.thumb',  {:side=>:left, :expand=>true}
                            ]}
                          ])
@@ -95,16 +95,43 @@ def kroc_rb_settings
   #
   # Elements:
   #
-  if TkPackage.vcompare(Tk::Tile.package_version, '0.5') >= 0
-
-    Tk::Tile::Style.element_create('Button.button', 
-                                   :image, $images['button-n'],
-                                   :map=>[
-                                     :pressed, $images['button-p'], 
-                                     :active,  $images['button-h'], 
+  if Tk::Tile::TILE_SPEC_VERSION_ID >= 8
+    Tk::Tile::Style.element_create('Button.button',
+                                   :image,
+                                   [ $images['button-n'],
+                                     :pressed, $images['button-p'],
+                                     :active,  $images['button-h'],
                                    ], :border=>3, :sticky=>:ew)
 
-    Tk::Tile::Style.element_create('Checkbutton.indicator', 
+    Tk::Tile::Style.element_create('Checkbutton.indicator',
+                                   :image,
+                                   [ $images['check-nu'],
+                                     [:pressed, :selected],$images['check-nc'],
+                                     :pressed,             $images['check-nu'],
+                                     [:active, :selected], $images['check-hc'],
+                                     :active,              $images['check-hu'],
+                                     :selected,            $images['check-nc'],
+                                   ], :sticky=>:w)
+
+    Tk::Tile::Style.element_create('Radiobutton.indicator',
+                                   :image,
+                                   [ $images['radio-nu'],
+                                     [:pressed,:selected],$images['radio-nc'],
+                                     :pressed,            $images['radio-nu'],
+                                     [:active,:selected], $images['radio-hc'],
+                                     :active,             $images['radio-hu'],
+                                     :selected,           $images['radio-nc'],
+                                   ], :sticky=>:w)
+
+  elsif TkPackage.vcompare(Tk::Tile.package_version, '0.5') >= 0
+    Tk::Tile::Style.element_create('Button.button',
+                                   :image, $images['button-n'],
+                                   :map=>[
+                                     :pressed, $images['button-p'],
+                                     :active,  $images['button-h'],
+                                   ], :border=>3, :sticky=>:ew)
+
+    Tk::Tile::Style.element_create('Checkbutton.indicator',
                                    :image, $images['check-nu'],
                                    :map=>[
                                      [:pressed, :selected],$images['check-nc'],
@@ -114,7 +141,7 @@ def kroc_rb_settings
                                      :selected,            $images['check-nc'],
                                    ], :sticky=>:w)
 
-    Tk::Tile::Style.element_create('Radiobutton.indicator', 
+    Tk::Tile::Style.element_create('Radiobutton.indicator',
                                    :image, $images['radio-nu'],
                                    :map=>[
                                      [:pressed, :selected],$images['radio-nc'],
@@ -125,15 +152,14 @@ def kroc_rb_settings
                                    ], :sticky=>:w)
 
   else # tile 0.4 or earlier
-
-    Tk::Tile::Style.element_create('Button.button', :pixmap, 
+    Tk::Tile::Style.element_create('Button.button', :pixmap,
                                    :images=>[
-                                     :pressed, $images['button-p'], 
-                                     :active,  $images['button-h'], 
+                                     :pressed, $images['button-p'],
+                                     :active,  $images['button-h'],
                                      '',       $images['button-n']
                                    ], :border=>3, :tiling=>:tile)
 
-    Tk::Tile::Style.element_create('Checkbutton.indicator', :pixmap, 
+    Tk::Tile::Style.element_create('Checkbutton.indicator', :pixmap,
                                    :images=>[
                                      [:pressed, :selected],$images['check-nc'],
                                      :pressed,             $images['check-nu'],
@@ -143,7 +169,7 @@ def kroc_rb_settings
                                      '',                   $images['check-nu'],
                                    ], :tiling=>:fixed)
 
-    Tk::Tile::Style.element_create('Radiobutton.indicator', :pixmap, 
+    Tk::Tile::Style.element_create('Radiobutton.indicator', :pixmap,
                                    :images=>[
                                      [:pressed, :selected],$images['radio-nc'],
                                      :pressed,             $images['radio-nu'],
@@ -158,7 +184,7 @@ def kroc_rb_settings
   #
   # Settings:
   #
-  # Tk::Tile::Style.layout(Tk::Tile::TButton, 
+  # Tk::Tile::Style.layout(Tk::Tile::TButton,
   Tk::Tile::Style.layout('TButton', [
       'Button.button', {:children=>[
            'Button.focus', {:children=>[
@@ -169,12 +195,12 @@ def kroc_rb_settings
       ]}
   ])
 
-  # Tk::Tile::Style.layout(Tk::Tile::TCheckbutton, 
+  # Tk::Tile::Style.layout(Tk::Tile::TCheckbutton,
   Tk::Tile::Style.layout('TCheckbutton', [
       'Checkbutton.background', # this is not needed in tile 0.5 or later
       'Checkbutton.border', {:children=>[
            'Checkbutton.padding', {:children=>[
-                'Checkbutton.indicator', {:side=>:left}, 
+                'Checkbutton.indicator', {:side=>:left},
                 'Checkbutton.focus', {:side=>:left, :children=>[
                     'Checkbutton.label'
                 ]}
@@ -182,12 +208,12 @@ def kroc_rb_settings
       ]}
   ])
 
-  # Tk::Tile::Style.layout(Tk::Tile::TRadiobutton, 
+  # Tk::Tile::Style.layout(Tk::Tile::TRadiobutton,
   Tk::Tile::Style.layout('TRadiobutton', [
       'Radiobutton.background', # this is not needed in tile 0.5 or later
       'Radiobutton.border', {:children=>[
            'Radiobutton.padding', {:children=>[
-                'Radiobutton.indicator', {:side=>:left}, 
+                'Radiobutton.indicator', {:side=>:left},
                 'Radiobutton.focus', {:expand=>true, :sticky=>:w, :children=>[
                     'Radiobutton.label', {:side=>:right, :expand=>true}
                 ]}
@@ -196,5 +222,5 @@ def kroc_rb_settings
   ])
 end
 
-Tk::Tile::Style.theme_create('kroc-rb', :parent=>'alt', 
+Tk::Tile::Style.theme_create('kroc-rb', :parent=>'alt',
                              :settings=>proc{ kroc_rb_settings() })
