@@ -1,6 +1,6 @@
 #----------------------------------
 # extconf.rb
-# $Revision: 21960 $
+# $Revision: 28927 $
 #----------------------------------
 require 'mkmf'
 
@@ -19,7 +19,7 @@ end
 def create_win32ole_makefile
   if have_library("ole32") and
      have_library("oleaut32") and
-     have_library("uuid") and 
+     have_library("uuid") and
      have_library("user32") and
      have_library("kernel32") and
      have_library("advapi32") and
@@ -38,7 +38,5 @@ end
 case RUBY_PLATFORM
 when /mswin/
   $CFLAGS += ' /W3'
-when /cygwin/, /mingw/
-  $defs << '-DNONAMELESSUNION'
 end
 create_win32ole_makefile
