@@ -1,20 +1,19 @@
 #
 #   xmp.rb - irb version of gotoken xmp
 #   	$Release Version: 0.9$
-#   	$Revision: 11708 $
-#   	$Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
+#   	$Revision: 26986 $
 #   	by Keiju ISHITSUKA(Nippon Rational Inc.)
 #
 # --
 #
-#   
+#
 #
 
 require "irb"
 require "irb/frame"
 
 class XMP
-  @RCS_ID='-$Id: xmp.rb 11708 2007-02-12 23:01:19Z shyouhei $-'
+  @RCS_ID='-$Id: xmp.rb 26986 2010-03-20 03:30:59Z nobu $-'
 
   def initialize(bind = nil)
     IRB.init_config(nil)
@@ -73,8 +72,20 @@ class XMP
     end
 
     def puts(exps)
+      if @encoding and exps.encoding != @encoding
+	enc = Encoding.compatible?(@exps.join("\n"), exps)
+	if enc.nil?
+	  raise Encoding::CompatibilityError, "Encoding in which the passed expression is encoded is not compatible to the preceding's one"
+	else
+	  @encoding = enc
+	end
+      else
+	@encoding = exps.encoding
+      end
       @exps.concat exps.split(/\n/)
     end
+
+    attr_reader :encoding
   end
 end
 
