@@ -1,11 +1,11 @@
-# 
+#
 # = net/ftp.rb - FTP Client Library
-# 
+#
 # Written by Shugo Maeda <shugo@ruby-lang.org>.
 #
 # Documentation by Gavin Sinclair, sourced from "Programming Ruby" (Hunt/Thomas)
 # and "Ruby In a Nutshell" (Matsumoto), used with permission.
-# 
+#
 # This library is distributed under the terms of the Ruby license.
 # You can freely distribute/modify this library.
 #
@@ -22,9 +22,10 @@ module Net
   # :stopdoc:
   class FTPError < StandardError; end
   class FTPReplyError < FTPError; end
-  class FTPTempError < FTPError; end 
-  class FTPPermError < FTPError; end 
+  class FTPTempError < FTPError; end
+  class FTPPermError < FTPError; end
   class FTPProtoError < FTPError; end
+  class FTPConnectionError < FTPError; end
   # :startdoc:
 
   #
@@ -34,11 +35,11 @@ module Net
   # advantage of Ruby's style and strengths.
   #
   # == Example
-  # 
+  #
   #   require 'net/ftp'
   #
   # === Example 1
-  #  
+  #
   #   ftp = Net::FTP.new('ftp.netlab.co.jp')
   #   ftp.login
   #   files = ftp.chdir('pub/lang/ruby/contrib')
@@ -71,13 +72,13 @@ module Net
   #
   class FTP
     include MonitorMixin
-    
+
     # :stopdoc:
     FTP_PORT = 21
     CRLF = "\r\n"
     DEFAULT_BLOCKSIZE = 4096
     # :startdoc:
-    
+
     # When +true+, transfers are performed in binary mode.  Default: +true+.
     attr_reader :binary
 
@@ -101,7 +102,7 @@ module Net
 
     # The server's last response.
     attr_reader :last_response
-    
+
     #
     # A synonym for <tt>FTP.new</tt>, but with a mandatory host parameter.
     #
@@ -120,7 +121,7 @@ module Net
         new(host, user, passwd, acct)
       end
     end
-    
+
     #
     # Creates and returns a new +FTP+ object. If a +host+ is given, a connection
     # is made. Additionally, if the +user+ is given, the given user name,
@@ -128,10 +129,12 @@ module Net
     #
     def initialize(host = nil, user = nil, passwd = nil, acct = nil)
       super()
-      @binary = false
+      @binary = true
       @passive = false
       @debug_mode = false
       @resume = false
+      @sock = NullSocket.new
+      @logged_in = false
       if host
 	connect(host)
 	if user
@@ -143,10 +146,19 @@ module Net
     def binary=(newmode)
       if newmode != @binary
         @binary = newmode
-        @binary ? voidcmd("TYPE I") : voidcmd("TYPE A")
+        send_type_command if @logged_in
       end
     end
 
+    def send_type_command
+      if @binary
+        voidcmd("TYPE I")
+      else
+        voidcmd("TYPE A")
+      end
+    end
+    private :send_type_command
+
     def with_binary(newmode)
       oldmode = binary
       self.binary = newmode
@@ -170,15 +182,15 @@ module Net
     end
 
     def open_socket(host, port)
-      if defined? SOCKSsocket and ENV["SOCKS_SERVER"]
+      if defined? SOCKSSocket and ENV["SOCKS_SERVER"]
 	@passive = true
-	return SOCKSsocket.open(host, port)
+	return SOCKSSocket.open(host, port)
       else
 	return TCPSocket.open(host, port)
       end
     end
     private :open_socket
-    
+
     #
     # Establishes an FTP connection to host, optionally overriding the default
     # port. If the environment variable +SOCKS_SERVER+ is set, sets up the
@@ -215,7 +227,7 @@ module Net
       end
     end
     private :sanitize
-    
+
     def putline(line)
       if @debug_mode
 	print "put: ", sanitize(line), "\n"
@@ -224,7 +236,7 @@ module Net
       @sock.write(line)
     end
     private :putline
-    
+
     def getline
       line = @sock.readline # if get EOF, raise EOFError
       line.sub!(/(\r\n|\n|\r)\z/n, "")
@@ -234,7 +246,7 @@ module Net
       return line
     end
     private :getline
-    
+
     def getmultiline
       line = getline
       buff = line
@@ -248,7 +260,7 @@ module Net
       return buff << "\n"
     end
     private :getmultiline
-    
+
     def getresp
       @last_response = getmultiline
       @last_response_code = @last_response[0, 3]
@@ -264,7 +276,7 @@ module Net
       end
     end
     private :getresp
-    
+
     def voidresp
       resp = getresp
       if resp[0] != ?2
@@ -272,7 +284,7 @@ module Net
       end
     end
     private :voidresp
-    
+
     #
     # Sends a command and returns the response.
     #
@@ -282,7 +294,7 @@ module Net
 	return getresp
       end
     end
-    
+
     #
     # Sends a command and expect a response beginning with '2'.
     #
@@ -292,7 +304,7 @@ module Net
 	voidresp
       end
     end
-    
+
     def sendport(host, port)
       af = (@sock.peeraddr)[0]
       if af == "AF_INET"
@@ -305,7 +317,7 @@ module Net
       voidcmd(cmd)
     end
     private :sendport
-    
+
     def makeport
       sock = TCPServer.open(@sock.addr[3], 0)
       port = sock.addr[1]
@@ -314,7 +326,7 @@ module Net
       return sock
     end
     private :makeport
-    
+
     def makepasv
       if @sock.peeraddr[0] == "AF_INET"
 	host, port = parse227(sendcmd("PASV"))
@@ -325,13 +337,13 @@ module Net
       return host, port
     end
     private :makepasv
-    
+
     def transfercmd(cmd, rest_offset = nil)
       if @passive
 	host, port = makepasv
 	conn = open_socket(host, port)
 	if @resume and rest_offset
-	  resp = sendcmd("REST " + rest_offset.to_s) 
+	  resp = sendcmd("REST " + rest_offset.to_s)
 	  if resp[0] != ?3
 	    raise FTPReplyError, resp
 	  end
@@ -345,7 +357,7 @@ module Net
       else
 	sock = makeport
 	if @resume and rest_offset
-	  resp = sendcmd("REST " + rest_offset.to_s) 
+	  resp = sendcmd("REST " + rest_offset.to_s)
 	  if resp[0] != ?3
 	    raise FTPReplyError, resp
 	  end
@@ -362,23 +374,7 @@ module Net
       return conn
     end
     private :transfercmd
-    
-    def getaddress
-      thishost = Socket.gethostname
-      if not thishost.index(".")
-	thishost = Socket.gethostbyname(thishost)[0]
-      end
-      if ENV.has_key?("LOGNAME")
-	realuser = ENV["LOGNAME"]
-      elsif ENV.has_key?("USER")
-	realuser = ENV["USER"]
-      else
-	realuser = "anonymous"
-      end
-      return realuser + "@" + thishost
-    end
-    private :getaddress
-    
+
     #
     # Logs in to the remote host. The session must have been previously
     # connected.  If +user+ is the string "anonymous" and the +password+ is
@@ -389,9 +385,9 @@ module Net
     #
     def login(user = "anonymous", passwd = nil, acct = nil)
       if user == "anonymous" and passwd == nil
-	passwd = getaddress
+	passwd = "anonymous@"
       end
-      
+
       resp = ""
       synchronize do
 	resp = sendcmd('USER ' + user)
@@ -408,9 +404,10 @@ module Net
 	raise FTPReplyError, resp
       end
       @welcome = resp
-      self.binary = true
+      send_type_command
+      @logged_in = true
     end
-    
+
     #
     # Puts the connection into binary (image) mode, issues the given command,
     # and fetches the data returned, passing it to the associated block in
@@ -431,7 +428,7 @@ module Net
         end
       end
     end
-    
+
     #
     # Puts the connection into ASCII (text) mode, issues the given command, and
     # passes the resulting data, one line at a time, to the associated block. If
@@ -445,19 +442,14 @@ module Net
           loop do
             line = conn.gets
             break if line == nil
-            if line[-2, 2] == CRLF
-              line = line[0 .. -3]
-            elsif line[-1] == ?\n
-              line = line[0 .. -2]
-            end
-            yield(line)
+            yield(line.sub(/\r?\n\z/, ""), !line.match(/\n\z/).nil?)
           end
           conn.close
           voidresp
         end
       end
     end
-    
+
     #
     # Puts the connection into binary (image) mode, issues the given server-side
     # command (such as "STOR myfile"), and sends the contents of the file named
@@ -470,7 +462,7 @@ module Net
       end
       synchronize do
 	with_binary(true) do
-          conn = transfercmd(cmd, rest_offset)
+          conn = transfercmd(cmd)
           loop do
             buf = file.read(blocksize)
             break if buf == nil
@@ -489,7 +481,7 @@ module Net
       getresp
       raise
     end
-    
+
     #
     # Puts the connection into ASCII (text) mode, issues the given server-side
     # command (such as "STOR myfile"), and sends the contents of the file
@@ -544,7 +536,7 @@ module Net
       end
       begin
 	f.binmode if localfile
-	retrbinary("RETR " + remotefile, blocksize, rest_offset) do |data|
+	retrbinary("RETR " + remotefile.to_s, blocksize, rest_offset) do |data|
 	  f.write(data) if localfile
 	  yield(data) if block_given?
           result.concat(data) if result
@@ -554,7 +546,7 @@ module Net
 	f.close if localfile
       end
     end
-    
+
     #
     # Retrieves +remotefile+ in ASCII (text) mode, storing the result in
     # +localfile+.
@@ -570,10 +562,11 @@ module Net
         result = ""
       end
       begin
-	retrlines("RETR " + remotefile) do |line|
-	  f.puts(line) if localfile
-	  yield(line) if block_given?
-          result.concat(line + "\n") if result
+	retrlines("RETR " + remotefile) do |line, newline|
+          l = newline ? line + "\n" : line
+	  f.print(l) if localfile
+	  yield(line, newline) if block_given?
+          result.concat(l) if result
 	end
         return result
       ensure
@@ -593,7 +586,7 @@ module Net
 	gettextfile(remotefile, localfile, &block)
       end
     end
-    
+
     #
     # Transfers +localfile+ to the server in binary mode, storing the result in
     # +remotefile+. If a block is supplied, calls it, passing in the transmitted
@@ -613,12 +606,16 @@ module Net
       f = open(localfile)
       begin
 	f.binmode
-	storbinary("STOR " + remotefile, f, blocksize, rest_offset, &block)
+        if rest_offset
+          storbinary("APPE " + remotefile, f, blocksize, rest_offset, &block)
+        else
+          storbinary("STOR " + remotefile, f, blocksize, rest_offset, &block)
+        end
       ensure
 	f.close
       end
     end
-    
+
     #
     # Transfers +localfile+ to the server in ASCII (text) mode, storing the result
     # in +remotefile+. If callback or an associated block is supplied, calls it,
@@ -653,7 +650,7 @@ module Net
       cmd = "ACCT " + account
       voidcmd(cmd)
     end
-    
+
     #
     # Returns an array of filenames in the remote directory.
     #
@@ -668,7 +665,7 @@ module Net
       end
       return files
     end
-    
+
     #
     # Returns an array of file information in the directory (the output is like
     # `ls -l`).  If a block is given, it iterates through the listing.
@@ -676,7 +673,7 @@ module Net
     def list(*args, &block) # :yield: line
       cmd = "LIST"
       args.each do |arg|
-	cmd = cmd + " " + arg
+	cmd = cmd + " " + arg.to_s
       end
       if block
 	retrlines(cmd, &block)
@@ -690,7 +687,7 @@ module Net
     end
     alias ls list
     alias dir list
-    
+
     #
     # Renames a file on the server.
     #
@@ -701,7 +698,7 @@ module Net
       end
       voidcmd("RNTO " + toname)
     end
-    
+
     #
     # Deletes a file on the server.
     #
@@ -715,7 +712,7 @@ module Net
 	raise FTPReplyError, resp
       end
     end
-    
+
     #
     # Changes the (remote) directory.
     #
@@ -733,22 +730,22 @@ module Net
       cmd = "CWD " + dirname
       voidcmd(cmd)
     end
-    
+
     #
     # Returns the size of the given (remote) filename.
     #
     def size(filename)
       with_binary(true) do
         resp = sendcmd("SIZE " + filename)
-        if resp[0, 3] != "213" 
+        if resp[0, 3] != "213"
           raise FTPReplyError, resp
         end
         return resp[3..-1].strip.to_i
       end
     end
-    
+
     MDTM_REGEXP = /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/  # :nodoc:
-    
+
     #
     # Returns the last modification time of the (remote) file.  If +local+ is
     # +true+, it is returned as a local time, otherwise it's a UTC time.
@@ -758,7 +755,7 @@ module Net
       ary = str.scan(MDTM_REGEXP)[0].collect {|i| i.to_i}
       return local ? Time.local(*ary) : Time.gm(*ary)
     end
-    
+
     #
     # Creates a remote directory.
     #
@@ -766,14 +763,14 @@ module Net
       resp = sendcmd("MKD " + dirname)
       return parse257(resp)
     end
-    
+
     #
     # Removes a remote directory.
     #
     def rmdir(dirname)
       voidcmd("RMD " + dirname)
     end
-    
+
     #
     # Returns the current remote directory.
     #
@@ -782,7 +779,7 @@ module Net
       return parse257(resp)
     end
     alias getdir pwd
-    
+
     #
     # Returns system information.
     #
@@ -793,7 +790,7 @@ module Net
       end
       return resp[4 .. -1]
     end
-    
+
     #
     # Aborts the previous command (ABOR command).
     #
@@ -807,7 +804,7 @@ module Net
       end
       return resp
     end
-    
+
     #
     # Returns the status (STAT command).
     #
@@ -817,7 +814,7 @@ module Net
       @sock.send(line, Socket::MSG_OOB)
       return getresp
     end
-    
+
     #
     # Issues the MDTM command.  TODO: more info.
     #
@@ -827,7 +824,7 @@ module Net
 	return resp[3 .. -1].strip
       end
     end
-    
+
     #
     # Issues the HELP command.
     #
@@ -838,7 +835,7 @@ module Net
       end
       sendcmd(cmd)
     end
-    
+
     #
     # Exits the FTP session.
     #
@@ -860,7 +857,7 @@ module Net
       cmd = "SITE " + arg
       voidcmd(cmd)
     end
-    
+
     #
     # Closes the connection.  Further operations are impossible until you open
     # a new connection with #connect.
@@ -868,14 +865,14 @@ module Net
     def close
       @sock.close if @sock and not @sock.closed?
     end
-    
+
     #
     # Returns +true+ iff the connection is closed.
     #
     def closed?
       @sock == nil or @sock.closed?
     end
-    
+
     def parse227(resp)
       if resp[0, 3] != "227"
 	raise FTPReplyError, resp
@@ -894,7 +891,7 @@ module Net
       return host, port
     end
     private :parse227
-    
+
     def parse228(resp)
       if resp[0, 3] != "228"
 	raise FTPReplyError, resp
@@ -922,11 +919,11 @@ module Net
 	end
 	host = v6[0, 8].join(":")
 	port = (numbers[19].to_i << 8) + numbers[20].to_i
-      end 
+      end
       return host, port
     end
     private :parse228
-    
+
     def parse229(resp)
       if resp[0, 3] != "229"
 	raise FTPReplyError, resp
@@ -945,7 +942,7 @@ module Net
       return host, port
     end
     private :parse229
-    
+
     def parse257(resp)
       if resp[0, 3] != "257"
 	raise FTPReplyError, resp
@@ -970,8 +967,15 @@ module Net
       return dirname
     end
     private :parse257
-  end
 
+    # :stopdoc:
+    class NullSocket
+      def method_missing(mid, *args)
+        raise FTPConnectionError, "not connected"
+      end
+    end
+    # :startdoc:
+  end
 end
 
 
