@@ -2,7 +2,7 @@
 
   main.c -
 
-  $Author: nobu $
+  $Author: akr $
   created at: Fri Aug 19 13:19:58 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -11,6 +11,7 @@
 
 #undef RUBY_EXPORT
 #include "ruby.h"
+#include "debug.h"
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
