@@ -1,32 +1,28 @@
 /* public domain rewrite of strchr(3) and strrchr(3) */
 
 char *
-strchr(s, c)
-    char *s;
-    int c;
+strchr(const char *s, int c)
 {
-    if (c == 0) return s + strlen(s);
+    if (c == 0) return (char *)s + strlen(s);
     while (*s) {
 	if (*s == c)
-	    return s;
+	    return (char *)s;
 	s++;
     }
     return 0;
 }
 
 char *
-strrchr(s, c)
-    char *s;
-    int c;
+strrchr(const char *s, int c)
 {
-    char *save;
+    const char *save;
 
-    if (c == 0) return s + strlen(s);
+    if (c == 0) return (char *)s + strlen(s);
     save = 0;
     while (*s) {
 	if (*s == c)
 	    save = s;
 	s++;
     }
-    return save;
+    return (char *)save;
 }
