@@ -22,13 +22,15 @@ end
 
 f = Foo.new
 #Foo.private :printf
-Foo.public :foobar
+class Foo			# redefines foobar's scope
+  public :foobar
+end
 f.foobar
 f.printf "%s\n", Foo
 
 f.quux
 
-class Bar : Foo
+class Bar<Foo
   def quux
     super
     baz()
