@@ -1,4 +1,4 @@
-require 'drbtest'
+require_relative 'drbtest'
 
 class TestDRbCore < Test::Unit::TestCase
   include DRbCore
@@ -6,23 +6,23 @@ end
 
 class TestDRbYield < Test::Unit::TestCase
   def setup
-    @ext = DRbService.manager.service('ut_drb.rb')
+    @ext = DRbService.ext_service('ut_drb.rb')
     @there = @ext.front
   end
 
   def teardown
-    @ext.stop_service
+    @ext.stop_service if @ext
   end
 
   def test_01_one
     @there.echo_yield_1([]) {|one|
       assert_equal([], one)
     }
-    
+
     @there.echo_yield_1(1) {|one|
       assert_equal(1, one)
     }
-    
+
     @there.echo_yield_1(nil) {|one|
       assert_equal(nil, one)
     }
@@ -102,9 +102,18 @@ class TestDRbYield < Test::Unit::TestCase
     @there.xarray_each {|x| assert_kind_of(XArray, x)}
     @there.xarray_each {|*x| assert_kind_of(XArray, x[0])}
   end
+
+  def test_06_taint
+    x = proc {}
+    assert(! x.tainted?)
+    @there.echo_yield(x) {|o|
+      assert_equal(x, o)
+      assert(! x.tainted?)
+    }
+  end
 end
 
-class TestRubyYield < TestDRbYield
+class TestDRbRubyYield < TestDRbYield
   def echo_yield(*arg)
     yield(*arg)
   end
@@ -131,25 +140,25 @@ class TestRubyYield < TestDRbYield
   def setup
     @there = self
   end
-  
+
   def teardown
   end
 end
 
-class TestRuby18Yield < TestRubyYield
+class TestDRbRuby18Yield < TestDRbRubyYield
   class YieldTest18
     def echo_yield(*arg, &proc)
       proc.call(*arg)
     end
-    
+
     def echo_yield_0(&proc)
       proc.call
     end
-    
+
     def echo_yield_1(a, &proc)
       proc.call(a)
     end
-    
+
     def echo_yield_2(a, b, &proc)
       proc.call(a, b)
     end
@@ -172,7 +181,7 @@ end
 
 class TestDRbMServer < Test::Unit::TestCase
   def setup
-    @ext = DRbService.manager.service('ut_drb.rb')
+    @ext = DRbService.ext_service('ut_drb.rb')
     @there = @ext.front
     @server = (1..3).collect do |n|
       DRb::DRbServer.new(nil, Onecky.new(n.to_s))
@@ -183,7 +192,7 @@ class TestDRbMServer < Test::Unit::TestCase
     @server.each do |s|
       s.stop_service
     end
-    @ext.stop_service
+    @ext.stop_service if @ext
   end
 
   def test_01
@@ -191,61 +200,70 @@ class TestDRbMServer < Test::Unit::TestCase
   end
 end
 
-class TestDRbReusePort < TestDRbAry
-  def setup
-    sleep 1.2
-    @ext = DRbService.manager.service('ut_port.rb')
-    @there = @ext.front
-  end
-end
-
 class TestDRbSafe1 < TestDRbAry
   def setup
-    @ext = DRbService.manager.service('ut_safe1.rb')
+    @ext = DRbService.ext_service('ut_safe1.rb')
     @there = @ext.front
   end
 end
 
-class TestDRbEval < Test::Unit::TestCase
+class TestDRbEval # < Test::Unit::TestCase
   def setup
     super
-    @ext = DRbService.manager.service('ut_eval.rb')
+    @ext = DRbService.ext_service('ut_eval.rb')
     @there = @ext.front
   end
 
   def teardown
-    @ext.stop_service
+    @ext.stop_service if @ext
   end
-  
-  def test_01_safe1_eval
-    assert_raises(SecurityError) do
+
+  def test_01_safe1_safe4_eval
+    assert_raise(SecurityError) do
       @there.method_missing(:instance_eval, 'ENV.inspect')
     end
 
-    assert_raises(SecurityError) do
+    assert_raise(SecurityError) do
       @there.method_missing(:send, :eval, 'ENV.inspect')
     end
 
     remote_class = @there.remote_class
 
-    assert_raises(SecurityError) do
+    assert_raise(SecurityError) do
       remote_class.class_eval('ENV.inspect')
     end
 
-    assert_raises(SecurityError) do
+    assert_raise(SecurityError) do
       remote_class.module_eval('ENV.inspect')
     end
+
+    four = @there.four
+    assert_equal(1, four.method_missing(:send, :eval, '1'))
+
+    remote_class = four.remote_class
+
+    assert_equal(1, remote_class.class_eval('1'))
+
+    assert_equal(1, remote_class.module_eval('1'))
+
+    assert_raise(SecurityError) do
+      remote_class.class_eval('ENV = {}')
+    end
+
+    assert_raise(SecurityError) do
+      remote_class.module_eval('ENV = {}')
+    end
   end
 end
 
 class TestDRbLarge < Test::Unit::TestCase
   def setup
-    @ext = DRbService.manager.service('ut_large.rb')
+    @ext = DRbService.ext_service('ut_large.rb')
     @there = @ext.front
   end
 
   def teardown
-    @ext.stop_service
+    @ext.stop_service if @ext
   end
 
   def test_01_large_ary
@@ -265,7 +283,7 @@ class TestDRbLarge < Test::Unit::TestCase
   end
 
   def test_04_many_arg
-    assert_raises(ArgumentError) {
+    assert_raise(DRb::DRbConnError) {
       @there.arg_test(1, 2, 3, 4, 5, 6, 7, 8, 9, 0)
     }
   end
@@ -274,7 +292,7 @@ class TestDRbLarge < Test::Unit::TestCase
     ary = ["Hello, World"] * 102400
     exception = nil
     begin
-      @there.size(ary)      
+      @there.size(ary)
     rescue StandardError
       exception = $!
     end
