@@ -8,13 +8,6 @@ require 'tmpdir'
 require 'enumerator'
 
 class TestPathname < Test::Unit::TestCase
-
-  if RUBY_VERSION < "1.9"
-    FUNCALL = :__send__
-  else
-    FUNCALL = :funcall
-  end
-
   def self.define_assertion(name, &block)
     @defassert_num ||= {}
     @defassert_num[name] ||= 0
@@ -123,7 +116,7 @@ class TestPathname < Test::Unit::TestCase
 
   # has_trailing_separator?(path) -> bool
   def has_trailing_separator?(path)
-    Pathname.allocate.send(FUNCALL, :has_trailing_separator?, path)
+    Pathname.allocate.__send__(:has_trailing_separator?, path)
   end
 
   defassert(:has_trailing_separator?, false, "/")
@@ -132,11 +125,11 @@ class TestPathname < Test::Unit::TestCase
   defassert(:has_trailing_separator?, true, "a/")
 
   def add_trailing_separator(path)
-    Pathname.allocate.send(FUNCALL, :add_trailing_separator, path)
+    Pathname.allocate.__send__(:add_trailing_separator, path)
   end
 
   def del_trailing_separator(path)
-    Pathname.allocate.send(FUNCALL, :del_trailing_separator, path)
+    Pathname.allocate.__send__(:del_trailing_separator, path)
   end
 
   defassert(:del_trailing_separator, "/", "/")
@@ -174,7 +167,10 @@ class TestPathname < Test::Unit::TestCase
 
   if DOSISH
     defassert(:del_trailing_separator, "a", "a\\")
-    defassert(:del_trailing_separator, "\225\\", "\225\\\\") # SJIS
+    require 'Win32API'
+    if Win32API.new('kernel32', 'GetACP', nil, 'L').call == 932
+      defassert(:del_trailing_separator, "\225\\", "\225\\\\") # SJIS
+    end
   end
 
   def plus(path1, path2) # -> path
@@ -273,27 +269,104 @@ class TestPathname < Test::Unit::TestCase
   defassert_raise(:relative_path_from, ArgumentError, "a", "..")
   defassert_raise(:relative_path_from, ArgumentError, ".", "..")
 
-  def realpath(path)
-    Pathname.new(path).realpath.to_s
+  def with_tmpchdir(base=nil)
+    Dir.mktmpdir(base) {|d|
+      d = Pathname.new(d).realpath.to_s
+      Dir.chdir(d) {
+        yield d
+      }
+    }
   end
 
-  def test_realpath
+  def has_symlink?
     begin
       File.symlink(nil, nil)
     rescue NotImplementedError
-      return
+      return false
     rescue TypeError
     end
-    dir = "#{Dir.tmpdir}/tst-pathname-#$$"
-    Dir.mkdir(dir)
-    begin
+    return true
+  end
+
+  def realpath(path, basedir=nil)
+    Pathname.new(path).realpath(basedir).to_s
+  end
+
+  def test_realpath
+    return if !has_symlink?
+    with_tmpchdir('rubytest-pathname') {|dir|
+      assert_raise(Errno::ENOENT) { realpath("#{dir}/not-exist") }
       File.symlink("not-exist-target", "#{dir}/not-exist")
       assert_raise(Errno::ENOENT) { realpath("#{dir}/not-exist") }
+
       File.symlink("loop", "#{dir}/loop")
       assert_raise(Errno::ELOOP) { realpath("#{dir}/loop") }
-    ensure
-      FileUtils.rmtree(dir)
-    end
+      assert_raise(Errno::ELOOP) { realpath("#{dir}/loop", dir) }
+
+      File.symlink("../#{File.basename(dir)}/./not-exist-target", "#{dir}/not-exist2")
+      assert_raise(Errno::ENOENT) { realpath("#{dir}/not-exist2") }
+
+      File.open("#{dir}/exist-target", "w") {}
+      File.symlink("../#{File.basename(dir)}/./exist-target", "#{dir}/exist2")
+      assert_nothing_raised { realpath("#{dir}/exist2") }
+
+      File.symlink("loop-relative", "loop-relative")
+      assert_raise(Errno::ELOOP) { realpath("#{dir}/loop-relative") }
+
+      Dir.mkdir("exist")
+      assert_equal("#{dir}/exist", realpath("exist"))
+      assert_raise(Errno::ELOOP) { realpath("../loop", "#{dir}/exist") }
+
+      File.symlink("loop1/loop1", "loop1")
+      assert_raise(Errno::ELOOP) { realpath("#{dir}/loop1") }
+
+      File.symlink("loop2", "loop3")
+      File.symlink("loop3", "loop2")
+      assert_raise(Errno::ELOOP) { realpath("#{dir}/loop2") }
+
+      Dir.mkdir("b")
+
+      File.symlink("b", "c")
+      assert_equal("#{dir}/b", realpath("c"))
+      assert_equal("#{dir}/b", realpath("c/../c"))
+      assert_equal("#{dir}/b", realpath("c/../c/../c/."))
+
+      File.symlink("..", "b/d")
+      assert_equal("#{dir}/b", realpath("c/d/c/d/c"))
+
+      File.symlink("#{dir}/b", "e")
+      assert_equal("#{dir}/b", realpath("e"))
+
+      Dir.mkdir("f")
+      Dir.mkdir("f/g")
+      File.symlink("f/g", "h")
+      assert_equal("#{dir}/f/g", realpath("h"))
+      File.chmod(0000, "f")
+      assert_raise(Errno::EACCES) { realpath("h") }
+      File.chmod(0755, "f")
+    }
+  end
+
+  def realdirpath(path)
+    Pathname.new(path).realdirpath.to_s
+  end
+
+  def test_realdirpath
+    return if !has_symlink?
+    Dir.mktmpdir('rubytest-pathname') {|dir|
+      rdir = realpath(dir)
+      assert_equal("#{rdir}/not-exist", realdirpath("#{dir}/not-exist"))
+      assert_raise(Errno::ENOENT) { realdirpath("#{dir}/not-exist/not-exist-child") }
+      File.symlink("not-exist-target", "#{dir}/not-exist")
+      assert_equal("#{rdir}/not-exist-target", realdirpath("#{dir}/not-exist"))
+      File.symlink("../#{File.basename(dir)}/./not-exist-target", "#{dir}/not-exist2")
+      assert_equal("#{rdir}/not-exist-target", realdirpath("#{dir}/not-exist2"))
+      File.open("#{dir}/exist-target", "w") {}
+      File.symlink("../#{File.basename(dir)}/./exist-target", "#{dir}/exist")
+      assert_equal("#{rdir}/exist-target", realdirpath("#{dir}/exist"))
+      File.symlink("loop", "#{dir}/loop")
+      assert_raise(Errno::ELOOP) { realdirpath("#{dir}/loop") }
+    }
   end
 
   def descend(path)
@@ -389,6 +462,26 @@ class TestPathname < Test::Unit::TestCase
   def pathsub(path, pat, repl) Pathname.new(path).sub(pat, repl).to_s end
   defassert(:pathsub, "a.o", "a.c", /\.c\z/, ".o")
 
+  def pathsubext(path, repl) Pathname.new(path).sub_ext(repl).to_s end
+  defassert(:pathsubext, 'a.o', 'a.c', '.o')
+  defassert(:pathsubext, 'a.o', 'a.c++', '.o')
+  defassert(:pathsubext, 'a.png', 'a.gif', '.png')
+  defassert(:pathsubext, 'ruby.tar.bz2', 'ruby.tar.gz', '.bz2')
+  defassert(:pathsubext, 'd/a.o', 'd/a.c', '.o')
+  defassert(:pathsubext, 'foo', 'foo.exe', '')
+  defassert(:pathsubext, 'lex.yy.o', 'lex.yy.c', '.o')
+  defassert(:pathsubext, 'fooaa.o', 'fooaa', '.o')
+  defassert(:pathsubext, 'd.e/aa.o', 'd.e/aa', '.o')
+
+  def test_sub_matchdata
+    result = Pathname("abc.gif").sub(/\..*/) {
+      assert_not_nil($~)
+      assert_equal(".gif", $~[0])
+      ".png"
+    }
+    assert_equal("abc.png", result.to_s)
+  end
+
   def root?(path)
     Pathname.new(path).root?
   end
@@ -478,9 +571,38 @@ class TestPathname < Test::Unit::TestCase
     result = []
     Pathname.new("/usr/bin/ruby").each_filename {|f| result << f }
     assert_equal(%w[usr bin ruby], result)
+    assert_equal(%w[usr bin ruby], Pathname.new("/usr/bin/ruby").each_filename.to_a)
   end
 
   def test_kernel_pathname
     assert_equal(Pathname.new("a"), Pathname("a"))
   end
+
+  def test_file_basename
+    assert_equal("bar", File.basename(Pathname.new("foo/bar")))
+  end
+
+  def test_file_dirname
+    assert_equal("foo", File.dirname(Pathname.new("foo/bar")))
+  end
+
+  def test_file_split
+    assert_equal(["foo", "bar"], File.split(Pathname.new("foo/bar")))
+  end
+
+  def test_file_extname
+    assert_equal(".baz", File.extname(Pathname.new("bar.baz")))
+  end
+
+  def test_file_fnmatch
+    assert(File.fnmatch("*.*", Pathname.new("bar.baz")))
+  end
+
+  def test_file_join
+    assert_equal("foo/bar", File.join(Pathname.new("foo"), Pathname.new("bar")))
+    lambda {
+      $SAFE = 1
+      assert_equal("foo/bar", File.join(Pathname.new("foo"), Pathname.new("bar").taint))
+    }.call
+  end
 end
