@@ -12,14 +12,14 @@ class MockClock
   include Singleton
 
   class MyTS < Rinda::TupleSpace
-    def keeper
+    def keeper_thread
       nil
     end
   end
-  
+
   def initialize
     @now = 2
-    @reso = 0.1
+    @reso = 1
     @ts = MyTS.new
     @ts.write([2, :now])
     @inf = 2**31 - 1
@@ -32,7 +32,7 @@ class MockClock
   def at(n)
     n
   end
-    
+
   def _forward(n=nil)
     now ,= @ts.take([nil, :now])
     @now = now + n
@@ -40,10 +40,11 @@ class MockClock
     @ts.write([@now, :now])
   end
 
-  def forward(n=nil)
+  def forward(n)
     while n > 0
       _forward(@reso)
       n -= @reso
+      Thread.pass
     end
   end
 
@@ -56,20 +57,10 @@ class MockClock
   end
 
   def sleep(n=nil)
-    while will_deadlock? 
-      n -= @reso
-      forward
-      return 0 if n <= 0
-    end
     now ,= @ts.read([nil, :now])
     @ts.read([(now + n)..@inf, :now])
     0
   end
-
-  def will_deadlock?
-    sz = Thread.current.group.list.find_all {|x| x.status != 'sleep'}.size
-    sz <= 1
-  end
 end
 
 module Time
@@ -115,7 +106,15 @@ module TupleSpaceTestModule
       Time.sleep(n)
     end
   end
-  
+
+  def thread_join(th)
+    while th.alive?
+      Kernel.sleep(0.1)
+      sleep(1)
+    end
+    th.value
+  end
+
   def test_00_tuple
     tuple = Rinda::TupleEntry.new([1,2,3])
     assert(!tuple.canceled?)
@@ -150,7 +149,7 @@ module TupleSpaceTestModule
     assert(!tmpl.match({"message"=>"Hi", "name"=>"Foo", "age"=>1}))
     assert(!tmpl.match({"message"=>"Hello", "no_name"=>"Foo"}))
 
-    assert_raises(Rinda::InvalidHashTupleKey) do
+    assert_raise(Rinda::InvalidHashTupleKey) do
       tmpl = Rinda::Template.new({:message=>String, "name"=>String})
     end
     tmpl = Rinda::Template.new({"name"=>String})
@@ -183,7 +182,7 @@ module TupleSpaceTestModule
     assert(!tmpl.match({"message"=>"Hi", "name"=>"Foo", "age"=>1}))
     assert(!tmpl.match({"message"=>"Hello", "no_name"=>"Foo"}))
 
-    assert_raises(Rinda::InvalidHashTupleKey) do
+    assert_raise(Rinda::InvalidHashTupleKey) do
       @ts.write({:message=>String, "name"=>String})
     end
 
@@ -194,7 +193,7 @@ module TupleSpaceTestModule
     assert_equal({'1'=>1, '2'=>2, '3'=>3}, @ts.take({'1'=>1, '2'=>2, '3'=>3}))
 
     entry = @ts.write(['1'=>1, '2'=>2, '3'=>3])
-    assert_raises(Rinda::RequestExpiredError) do
+    assert_raise(Rinda::RequestExpiredError) do
       assert_equal({'1'=>1, '2'=>2, '3'=>3}, @ts.read({'1'=>1}, 0))
     end
     entry.cancel
@@ -231,15 +230,37 @@ module TupleSpaceTestModule
   end
 
   def test_inp_rdp
-    assert_raises(Rinda::RequestExpiredError) do
+    assert_raise(Rinda::RequestExpiredError) do
       @ts.take([:empty], 0)
     end
 
-    assert_raises(Rinda::RequestExpiredError) do
+    assert_raise(Rinda::RequestExpiredError) do
       @ts.read([:empty], 0)
     end
   end
 
+  def test_ruby_talk_264062
+    th = Thread.new { @ts.take([:empty], 1) }
+    sleep(10)
+    assert_raise(Rinda::RequestExpiredError) do
+      thread_join(th)
+    end
+
+    th = Thread.new { @ts.read([:empty], 1) }
+    sleep(10)
+    assert_raise(Rinda::RequestExpiredError) do
+      thread_join(th)
+    end
+  end
+
+  def test_symbol_tuple
+    @ts.write([:symbol, :symbol])
+    @ts.write(['string', :string])
+    assert_equal([[:symbol, :symbol]], @ts.read_all([:symbol, nil]))
+    assert_equal([[:symbol, :symbol]], @ts.read_all([Symbol, nil]))
+    assert_equal([], @ts.read_all([:nil, nil]))
+  end
+
   def test_core_01
     5.times do |n|
       @ts.write([:req, 2])
@@ -252,7 +273,7 @@ module TupleSpaceTestModule
       s = 0
       while true
 	begin
-	  tuple = @ts.take([:req, Integer], 0.5)
+	  tuple = @ts.take([:req, Integer], 1)
 	  assert_equal(2, tuple[1])
 	  s += tuple[1]
 	rescue Rinda::RequestExpiredError
@@ -262,11 +283,10 @@ module TupleSpaceTestModule
       @ts.write([:ans, s])
       s
     end
-    
-    sleep(20)
+
+    assert_equal(10, thread_join(taker))
     tuple = @ts.take([:ans, nil])
     assert_equal(10, tuple[1])
-    assert_equal(10, taker.value)
   end
 
   def test_core_02
@@ -274,7 +294,7 @@ module TupleSpaceTestModule
       s = 0
       while true
 	begin
-	  tuple = @ts.take([:req, Integer], 1.0)
+	  tuple = @ts.take([:req, Integer], 1)
 	  assert_equal(2, tuple[1])
 	  s += tuple[1]
 	rescue Rinda::RequestExpiredError
@@ -289,17 +309,16 @@ module TupleSpaceTestModule
       @ts.write([:req, 2])
     end
 
-    sleep(20)
+    assert_equal(10, thread_join(taker))
     tuple = @ts.take([:ans, nil])
     assert_equal(10, tuple[1])
-    assert_equal(10, taker.value)
     assert_equal([], @ts.read_all([nil, nil]))
   end
-  
+
   def test_core_03_notify
     notify1 = @ts.notify(nil, [:req, Integer])
-    notify2 = @ts.notify(nil, [:ans, Integer], 5)
-    notify3 = @ts.notify(nil, {"message"=>String, "name"=>String}, 5)
+    notify2 = @ts.notify(nil, [:ans, Integer], 8)
+    notify3 = @ts.notify(nil, {"message"=>String, "name"=>String}, 8)
 
     @ts.write({"message"=>"first", "name"=>"3"}, 3)
     @ts.write({"message"=>"second", "name"=>"1"}, 1)
@@ -328,19 +347,18 @@ module TupleSpaceTestModule
       result = nil
       lv = 0
       n = 0
-      notify2.each  do |ev|
+      notify2.each do |ev, tuple|
 	n += 1
-	if ev[0] == 'write'
+	if ev == 'write'
 	  lv = lv + 1
-	elsif ev[0] == 'take'
+	elsif ev == 'take'
 	  lv = lv - 1
-	elsif ev[0] == 'close'
+	elsif ev == 'close'
 	  result = [lv, n]
-	else
 	  break
 	end
 	assert(lv >= 0)
-	assert_equal([:ans, 10], ev[1])
+	assert_equal([:ans, 10], tuple)
       end
       result
     end
@@ -349,7 +367,7 @@ module TupleSpaceTestModule
       s = 0
       while true
 	begin
-	  tuple = @ts.take([:req, Integer], 1.0)
+	  tuple = @ts.take([:req, Integer], 1)
 	  s += tuple[1]
 	rescue Rinda::RequestExpiredError
 	  break
@@ -359,26 +377,25 @@ module TupleSpaceTestModule
       s
     end
 
-    writer = Thread.new do
-      5.times do |n|
-	@ts.write([:req, 2])
-	sleep 0.1
-      end
+    5.times do |n|
+      @ts.write([:req, 2])
     end
 
     @ts.take({"message"=>"first", "name"=>"3"})
 
     sleep(4)
+    assert_equal(10, thread_join(taker))
+    # notify2 must not expire until this @ts.take.
+    # sleep(4) might be short enough for the timeout of notify2 (8 secs)
     tuple = @ts.take([:ans, nil])
     assert_equal(10, tuple[1])
-    assert_equal(10, taker.value)
     assert_equal([], @ts.read_all([nil, nil]))
-    
+
     notify1.cancel
-    sleep(3) # notify2 expired
-    
-    assert_equal([0, 11], listener1.value)
-    assert_equal([0, 3], listener2.value)
+    sleep(7) # notify2 expired (sleep(4)+sleep(7) > 8)
+
+    assert_equal([0, 11], thread_join(listener1))
+    assert_equal([0, 3], thread_join(listener2))
 
     ary = []
     ary.push(["write", {"message"=>"first", "name"=>"3"}])
@@ -400,26 +417,27 @@ module TupleSpaceTestModule
     assert_equal([[:removeme, 1]], @ts.read_all([nil, nil]))
     entry.cancel
     assert_equal([], @ts.read_all([nil, nil]))
-    
+
     template = nil
     taker = Thread.new do
-      @ts.take([:take, nil], 10) do |template|
+      @ts.take([:take, nil], 10) do |t|
+        template = t
 	Thread.new do
-	  sleep 0.2
 	  template.cancel
 	end
       end
     end
-    
-    sleep(1)
-    assert(template.canceled?)
-    
-    @ts.write([:take, 1])
 
-    assert_raises(Rinda::RequestCanceledError) do
-      assert_nil(taker.value)
+    sleep(2)
+
+    assert_raise(Rinda::RequestCanceledError) do
+      assert_nil(thread_join(taker))
     end
 
+    assert(template.canceled?)
+
+    @ts.write([:take, 1])
+
     assert_equal([[:take, 1]], @ts.read_all([nil, nil]))
   end
 
@@ -431,23 +449,24 @@ module TupleSpaceTestModule
 
     template = nil
     reader = Thread.new do
-      @ts.read([:take, nil], 10) do |template|
+      @ts.read([:take, nil], 10) do |t|
+        template = t
 	Thread.new do
-	  sleep 0.2
 	  template.cancel
 	end
       end
     end
 
-    sleep(1)
-    assert(template.canceled?)
-    
-    @ts.write([:take, 1])
+    sleep(2)
 
-    assert_raises(Rinda::RequestCanceledError) do
-      assert_nil(reader.value)
+    assert_raise(Rinda::RequestCanceledError) do
+      assert_nil(thread_join(reader))
     end
 
+    assert(template.canceled?)
+
+    @ts.write([:take, 1])
+
     assert_equal([[:take, 1]], @ts.read_all([nil, nil]))
   end
 
@@ -456,7 +475,7 @@ module TupleSpaceTestModule
       @sec = sec
       @n = n
     end
-    
+
     def renew
       return -1 if @n <= 0
       @n -= 1
@@ -469,7 +488,7 @@ module TupleSpaceTestModule
     assert(!tuple.canceled?)
     assert(tuple.expired?)
     assert(!tuple.alive?)
-    
+
     tuple = Rinda::TupleEntry.new([1,2,3], 1)
     assert(!tuple.canceled?)
     assert(!tuple.expired?)
@@ -478,15 +497,16 @@ module TupleSpaceTestModule
     assert(tuple.expired?)
     assert(!tuple.alive?)
 
-    tuple = Rinda::TupleEntry.new([1,2,3], SimpleRenewer.new(1,2))
+    @renewer = SimpleRenewer.new(1,2)
+    tuple = Rinda::TupleEntry.new([1,2,3], @renewer)
     assert(!tuple.canceled?)
     assert(!tuple.expired?)
     assert(tuple.alive?)
-    sleep(1.5)
+    sleep(1)
     assert(!tuple.canceled?)
     assert(!tuple.expired?)
     assert(tuple.alive?)
-    sleep(1.5)
+    sleep(2)
     assert(tuple.expired?)
     assert(!tuple.alive?)
   end
@@ -499,6 +519,10 @@ class TupleSpaceTest < Test::Unit::TestCase
     ThreadGroup.new.add(Thread.current)
     @ts = Rinda::TupleSpace.new(1)
   end
+  def teardown
+    # implementation-dependent
+    @ts.instance_eval{@keeper.kill if @keeper}
+  end
 end
 
 class TupleSpaceProxyTest < Test::Unit::TestCase
@@ -509,6 +533,10 @@ class TupleSpaceProxyTest < Test::Unit::TestCase
     @ts_base = Rinda::TupleSpace.new(1)
     @ts = Rinda::TupleSpaceProxy.new(@ts_base)
   end
+  def teardown
+    # implementation-dependent
+    @ts_base.instance_eval{@keeper.kill if @keeper}
+  end
 
   def test_remote_array_and_hash
     @ts.write(DRbObject.new([1, 2, 3]))
