@@ -1,4 +1,5 @@
 require 'test/unit'
+require_relative 'envutil'
 
 class TestArray < Test::Unit::TestCase
   def setup
@@ -125,27 +126,27 @@ class TestArray < Test::Unit::TestCase
 
   def test_beg_end_0
     x = [1, 2, 3, 4, 5]
-    
+
     assert_equal(1, x.first)
     assert_equal([1], x.first(1))
     assert_equal([1, 2, 3], x.first(3))
-    
+
     assert_equal(5, x.last)
     assert_equal([5], x.last(1))
     assert_equal([3, 4, 5], x.last(3))
-    
+
     assert_equal(1, x.shift)
     assert_equal([2, 3, 4], x.shift(3))
     assert_equal([5], x)
-    
+
     assert_equal([2, 3, 4, 5], x.unshift(2, 3, 4))
     assert_equal([1, 2, 3, 4, 5], x.unshift(1))
     assert_equal([1, 2, 3, 4, 5], x)
-    
+
     assert_equal(5, x.pop)
     assert_equal([3, 4], x.pop(2))
     assert_equal([1, 2], x)
-    
+
     assert_equal([1, 2, 3, 4], x.push(3, 4))
     assert_equal([1, 2, 3, 4, 5], x.push(5))
     assert_equal([1, 2, 3, 4, 5], x)
@@ -318,7 +319,7 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[10, 11, 12], a[9..11])
     assert_equal(@cls[10, 11, 12], a[-91..-89])
-    
+
     assert_nil(a[10, -3])
     # Ruby 1.8 feature change:
     # Array#[size..x] returns [] instead of nil.
@@ -487,7 +488,7 @@ class TestArray < Test::Unit::TestCase
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal([ Fixnum, String, Range], a.collect! {|e| e.class} )
     assert_equal([ Fixnum, String, Range], a)
-   
+
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal([ 99, 99, 99], a.collect! { 99 } )
     assert_equal([ 99, 99, 99], a)
@@ -535,10 +536,13 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1, 2, 3, 4],     @cls[1, 2, 3, 4].concat(@cls[]))
     assert_equal(@cls[],               @cls[].concat(@cls[]))
     assert_equal(@cls[@cls[1, 2], @cls[3, 4]], @cls[@cls[1, 2]].concat(@cls[@cls[3, 4]]))
-    
+
     a = @cls[1, 2, 3]
     a.concat(a)
     assert_equal([1, 2, 3, 1, 2, 3], a)
+
+    assert_raise(TypeError) { [0].concat(:foo) }
+    assert_raise(RuntimeError) { [0].freeze.concat(:foo) }
   end
 
   def test_count
@@ -708,7 +712,7 @@ class TestArray < Test::Unit::TestCase
     a5 = @cls[ a1, @cls[], a3 ]
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5.flatten)
     assert_equal(@cls[], @cls[].flatten)
-    assert_equal(@cls[], 
+    assert_equal(@cls[],
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
 
     assert_raise(TypeError, "[ruby-dev:31197]") { [[]].flatten("") }
@@ -719,6 +723,11 @@ class TestArray < Test::Unit::TestCase
     a7 = a6.flatten
     assert_equal(true, a7.tainted?)
     assert_equal(true, a7.untrusted?)
+
+    a8 = @cls[[1, 2], 3]
+    a9 = a8.flatten(0)
+    assert_equal(a8, a9)
+    assert_not_same(a8, a9)
   end
 
   def test_flatten!
@@ -731,11 +740,14 @@ class TestArray < Test::Unit::TestCase
 
     a5 = @cls[ a1, @cls[], a3 ]
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5.flatten!)
+    assert_nil(a5.flatten!(0), '[ruby-core:23382]')
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5)
 
     assert_equal(@cls[], @cls[].flatten)
-    assert_equal(@cls[], 
+    assert_equal(@cls[],
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
+
+    assert_nil(@cls[].flatten!(0), '[ruby-core:23382]')
   end
 
   def test_flatten_with_callcc
@@ -752,6 +764,91 @@ class TestArray < Test::Unit::TestCase
     assert_match(/reentered/, e.message, '[ruby-dev:34798]')
   end
 
+  def test_permutation_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = [1,2,3]
+    begin
+      ary.permutation {
+        callcc {|k| cont = k} unless cont
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e)
+    assert_match(/reentered/, e.message)
+  end
+
+  def test_product_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = [1,2,3]
+    begin
+      ary.product {
+        callcc {|k| cont = k} unless cont
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e)
+    assert_match(/reentered/, e.message)
+  end
+
+  def test_combination_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = [1,2,3]
+    begin
+      ary.combination(2) {
+        callcc {|k| cont = k} unless cont
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e)
+    assert_match(/reentered/, e.message)
+  end
+
+  def test_repeated_permutation_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = [1,2,3]
+    begin
+      ary.repeated_permutation(2) {
+        callcc {|k| cont = k} unless cont
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e)
+    assert_match(/reentered/, e.message)
+  end
+
+  def test_repeated_combination_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = [1,2,3]
+    begin
+      ary.repeated_combination(2) {
+        callcc {|k| cont = k} unless cont
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e)
+    assert_match(/reentered/, e.message)
+  end
+
   def test_hash
     a1 = @cls[ 'cat', 'dog' ]
     a2 = @cls[ 'cat', 'dog' ]
@@ -777,6 +874,8 @@ class TestArray < Test::Unit::TestCase
     assert_equal(4, a.index([1,2,3]))
     assert_nil(a.index('ca'))
     assert_nil(a.index([1,2]))
+
+    assert_equal(1, a.index(99) {|x| x == 'cat' })
   end
 
   def test_values_at
@@ -814,6 +913,8 @@ class TestArray < Test::Unit::TestCase
     s = a.join
     assert_equal(true, s.tainted?)
     assert_equal(true, s.untrusted?)
+  ensure
+    $, = nil
   end
 
   def test_last
@@ -835,7 +936,7 @@ class TestArray < Test::Unit::TestCase
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal(@cls[ Fixnum, String, Range], a.map! {|e| e.class} )
     assert_equal(@cls[ Fixnum, String, Range], a)
-   
+
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal(@cls[ 99, 99, 99], a.map! { 99 } )
     assert_equal(@cls[ 99, 99, 99], a)
@@ -876,17 +977,17 @@ class TestArray < Test::Unit::TestCase
     assert_equal("ABC",      @cls[ 65, 66, 67 ].pack("c3"))
     assert_equal("\377BC",   @cls[ -1, 66, 67 ].pack("c*"))
 
-    
+
     assert_equal("AB\n\x10",  @cls["4142", "0a", "12"].pack("H4H2H1"))
     assert_equal("AB\n\x02",  @cls["1424", "a0", "21"].pack("h4h2h1"))
 
-    assert_equal("abc=02def=\ncat=\n=01=\n", 
+    assert_equal("abc=02def=\ncat=\n=01=\n",
                  @cls["abc\002def", "cat", "\001"].pack("M9M3M4"))
 
     assert_equal("aGVsbG8K\n",  @cls["hello\n"].pack("m"))
     assert_equal(",:&5L;&\\*:&5L;&\\*\n",  @cls["hello\nhello\n"].pack("u"))
 
-    assert_equal("\xc2\xa9B\xe2\x89\xa0", @cls[0xa9, 0x42, 0x2260].pack("U*"))
+    assert_equal("\u{a9 42 2260}", @cls[0xa9, 0x42, 0x2260].pack("U*"))
 
 
     format = "c2x5CCxsdils_l_a6";
@@ -987,6 +1088,13 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[4, 5, 6], a)
     assert_equal(a_id, a.__id__)
     assert_equal(@cls[], a.replace(@cls[]))
+
+    fa = a.dup.freeze
+    assert_nothing_raised(RuntimeError) { a.replace(a) }
+    assert_raise(RuntimeError) { fa.replace(fa) }
+    assert_raise(ArgumentError) { fa.replace() }
+    assert_raise(TypeError) { a.replace(42) }
+    assert_raise(RuntimeError) { fa.replace(42) }
   end
 
   def test_reverse
@@ -1030,6 +1138,8 @@ class TestArray < Test::Unit::TestCase
     assert_equal(4, a.rindex([1,2,3]))
     assert_nil(a.rindex('ca'))
     assert_nil(a.rindex([1,2]))
+
+    assert_equal(3, a.rindex(99) {|x| x == [1,2,3] })
   end
 
   def test_shift
@@ -1080,7 +1190,7 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[10, 11, 12], a.slice(9..11))
     assert_equal(@cls[10, 11, 12], a.slice(-91..-89))
-    
+
     assert_nil(a.slice(-101..-1))
 
     assert_nil(a.slice(10, -3))
@@ -1126,6 +1236,9 @@ class TestArray < Test::Unit::TestCase
     a = @cls[1, 2, 3, 4, 5]
     assert_equal(nil, a.slice!(-6,2))
     assert_equal(@cls[1, 2, 3, 4, 5], a)
+
+    assert_raise(ArgumentError) { @cls[1].slice! }
+    assert_raise(ArgumentError) { @cls[1].slice!(0, 0, 0) }
   end
 
   def test_sort
@@ -1136,9 +1249,11 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[4, 3, 2, 1], a.sort { |x, y| y <=> x} )
     assert_equal(@cls[4, 1, 2, 3], a)
 
+    assert_equal(@cls[1, 2, 3, 4], a.sort { |x, y| (x - y) * (2**100) })
+
     a.fill(1)
     assert_equal(@cls[1, 1, 1, 1], a.sort)
-    
+
     assert_equal(@cls[], @cls[].sort)
   end
 
@@ -1155,6 +1270,14 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[1], @cls[1].sort!)
     assert_equal(@cls[], @cls[].sort!)
+
+    a = @cls[4, 3, 2, 1]
+    a.sort! {|m, n| a.replace([9, 8, 7, 6]); m <=> n }
+    assert_equal([1, 2, 3, 4], a)
+
+    a = @cls[4, 3, 2, 1]
+    a.sort! {|m, n| a.replace([9, 8, 7]); m <=> n }
+    assert_equal([1, 2, 3, 4], a)
   end
 
   def test_sort_with_callcc
@@ -1202,6 +1325,18 @@ class TestArray < Test::Unit::TestCase
     if (@cls == Array)
       assert_equal(a_id, a.to_ary.__id__)
     end
+
+    o = Object.new
+    def o.to_ary
+      [4, 5]
+    end
+    assert_equal([1, 2, 3, 4, 5], a.concat(o))
+
+    o = Object.new
+    def o.to_ary
+      foo_bar()
+    end
+    assert_match(/foo_bar/, assert_raise(NoMethodError) {a.concat(o)}.message)
   end
 
   def test_to_s
@@ -1220,25 +1355,127 @@ class TestArray < Test::Unit::TestCase
     $, = ":"
     a = @cls[1, 2, 3]
     assert_equal("[1, 2, 3]", a.to_s)
-
-    $, = ""
+  ensure
+    $, = nil
   end
 
   def test_uniq
+    a = []
+    b = a.uniq
+    assert_equal([], a)
+    assert_equal([], b)
+    assert_not_same(a, b)
+
+    a = [1]
+    b = a.uniq
+    assert_equal([1], a)
+    assert_equal([1], b)
+    assert_not_same(a, b)
+
+    a = [1,1]
+    b = a.uniq
+    assert_equal([1,1], a)
+    assert_equal([1], b)
+    assert_not_same(a, b)
+
+    a = [1,2]
+    b = a.uniq
+    assert_equal([1,2], a)
+    assert_equal([1,2], b)
+    assert_not_same(a, b)
+
     a = @cls[ 1, 2, 3, 2, 1, 2, 3, 4, nil ]
     b = a.dup
     assert_equal(@cls[1, 2, 3, 4, nil], a.uniq)
     assert_equal(b, a)
 
+    c = @cls["a:def", "a:xyz", "b:abc", "b:xyz", "c:jkl"]
+    d = c.dup
+    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c.uniq {|s| s[/^\w+/]})
+    assert_equal(d, c)
+
     assert_equal(@cls[1, 2, 3], @cls[1, 2, 3].uniq)
   end
 
+  def test_uniq_with_block
+    a = []
+    b = a.uniq {|v| v.even? }
+    assert_equal([], a)
+    assert_equal([], b)
+    assert_not_same(a, b)
+
+    a = [1]
+    b = a.uniq {|v| v.even? }
+    assert_equal([1], a)
+    assert_equal([1], b)
+    assert_not_same(a, b)
+
+    a = [1,3]
+    b = a.uniq {|v| v.even? }
+    assert_equal([1,3], a)
+    assert_equal([1], b)
+    assert_not_same(a, b)
+  end
+
   def test_uniq!
+    a = []
+    b = a.uniq!
+    assert_equal(nil, b)
+
+    a = [1]
+    b = a.uniq!
+    assert_equal(nil, b)
+
+    a = [1,1]
+    b = a.uniq!
+    assert_equal([1], a)
+    assert_equal([1], b)
+    assert_same(a, b)
+
+    a = [1,2]
+    b = a.uniq!
+    assert_equal([1,2], a)
+    assert_equal(nil, b)
+
     a = @cls[ 1, 2, 3, 2, 1, 2, 3, 4, nil ]
     assert_equal(@cls[1, 2, 3, 4, nil], a.uniq!)
     assert_equal(@cls[1, 2, 3, 4, nil], a)
 
+    c = @cls["a:def", "a:xyz", "b:abc", "b:xyz", "c:jkl"]
+    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c.uniq! {|s| s[/^\w+/]})
+    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c)
+
+    c = @cls["a:def", "b:abc", "c:jkl"]
+    assert_equal(nil, c.uniq! {|s| s[/^\w+/]})
+    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c)
+
     assert_nil(@cls[1, 2, 3].uniq!)
+
+    f = a.dup.freeze
+    assert_raise(ArgumentError) { a.uniq!(1) }
+    assert_raise(ArgumentError) { f.uniq!(1) }
+    assert_raise(RuntimeError) { f.uniq! }
+  end
+
+  def test_uniq_bang_with_block
+    a = []
+    b = a.uniq! {|v| v.even? }
+    assert_equal(nil, b)
+
+    a = [1]
+    b = a.uniq! {|v| v.even? }
+    assert_equal(nil, b)
+
+    a = [1,3]
+    b = a.uniq! {|v| v.even? }
+    assert_equal([1], a)
+    assert_equal([1], b)
+    assert_same(a, b)
+
+    a = [1,2]
+    b = a.uniq! {|v| v.even? }
+    assert_equal([1,2], a)
+    assert_equal(nil, b)
   end
 
   def test_unshift
@@ -1275,10 +1512,22 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[[1,1],[1,2],[2,1],[2,2]], @cls[1,2].product([1,2]))
 
     assert_equal(@cls[[1,3,5],[1,3,6],[1,4,5],[1,4,6],
-                   [2,3,5],[2,3,6],[2,4,5],[2,4,6]], 
+                   [2,3,5],[2,3,6],[2,4,5],[2,4,6]],
                  @cls[1,2].product([3,4],[5,6]))
     assert_equal(@cls[[1],[2]], @cls[1,2].product)
     assert_equal(@cls[], @cls[1,2].product([]))
+
+    bug3394 = '[ruby-dev:41540]'
+    acc = []
+    EnvUtil.under_gc_stress {[1,2].product([3,4,5],[6,8]){|array| acc << array}}
+    assert_equal([[1, 3, 6], [1, 3, 8], [1, 4, 6], [1, 4, 8], [1, 5, 6], [1, 5, 8],
+                  [2, 3, 6], [2, 3, 8], [2, 4, 6], [2, 4, 8], [2, 5, 6], [2, 5, 8]],
+                 acc, bug3394)
+
+    def (o = Object.new).to_ary; GC.start; [3,4] end
+    acc = [1,2].product(*[o]*10)
+    assert_equal([1,2].product([3,4], [3,4], [3,4], [3,4], [3,4], [3,4], [3,4], [3,4], [3,4], [3,4]),
+                 acc)
   end
 
   def test_permutation
@@ -1295,6 +1544,59 @@ class TestArray < Test::Unit::TestCase
                  "edcba".each_char.to_a.permutation(5).sort)
     assert_equal(@cls[].permutation(0).to_a, @cls[[]])
 
+    a = @cls[1, 2, 3, 4]
+    b = @cls[]
+    a.permutation {|x| b << x; a.replace(@cls[9, 8, 7, 6]) }
+    assert_equal(@cls[9, 8, 7, 6], a)
+    assert_equal(@cls[1, 2, 3, 4].permutation.to_a, b)
+  end
+
+  def test_repeated_permutation
+    a = @cls[1,2]
+    assert_equal(@cls[[]], a.repeated_permutation(0).to_a)
+    assert_equal(@cls[[1],[2]], a.repeated_permutation(1).to_a.sort)
+    assert_equal(@cls[[1,1],[1,2],[2,1],[2,2]],
+                 a.repeated_permutation(2).to_a.sort)
+    assert_equal(@cls[[1,1,1],[1,1,2],[1,2,1],[1,2,2],
+                      [2,1,1],[2,1,2],[2,2,1],[2,2,2]],
+                 a.repeated_permutation(3).to_a.sort)
+    assert_equal(@cls[], a.repeated_permutation(-1).to_a)
+    assert_equal("abcde".each_char.to_a.repeated_permutation(5).sort,
+                 "edcba".each_char.to_a.repeated_permutation(5).sort)
+    assert_equal(@cls[].repeated_permutation(0).to_a, @cls[[]])
+    assert_equal(@cls[].repeated_permutation(1).to_a, @cls[])
+
+    a = @cls[1, 2, 3, 4]
+    b = @cls[]
+    a.repeated_permutation(4) {|x| b << x; a.replace(@cls[9, 8, 7, 6]) }
+    assert_equal(@cls[9, 8, 7, 6], a)
+    assert_equal(@cls[1, 2, 3, 4].repeated_permutation(4).to_a, b)
+  end
+
+  def test_repeated_combination
+    a = @cls[1,2,3]
+    assert_equal(@cls[[]], a.repeated_combination(0).to_a)
+    assert_equal(@cls[[1],[2],[3]], a.repeated_combination(1).to_a.sort)
+    assert_equal(@cls[[1,1],[1,2],[1,3],[2,2],[2,3],[3,3]],
+                 a.repeated_combination(2).to_a.sort)
+    assert_equal(@cls[[1,1,1],[1,1,2],[1,1,3],[1,2,2],[1,2,3],
+                      [1,3,3],[2,2,2],[2,2,3],[2,3,3],[3,3,3]],
+                 a.repeated_combination(3).to_a.sort)
+    assert_equal(@cls[[1,1,1,1],[1,1,1,2],[1,1,1,3],[1,1,2,2],[1,1,2,3],
+                      [1,1,3,3],[1,2,2,2],[1,2,2,3],[1,2,3,3],[1,3,3,3],
+                      [2,2,2,2],[2,2,2,3],[2,2,3,3],[2,3,3,3],[3,3,3,3]],
+                 a.repeated_combination(4).to_a.sort)
+    assert_equal(@cls[], a.repeated_combination(-1).to_a)
+    assert_equal("abcde".each_char.to_a.repeated_combination(5).map{|a|a.sort}.sort,
+                 "edcba".each_char.to_a.repeated_combination(5).map{|a|a.sort}.sort)
+    assert_equal(@cls[].repeated_combination(0).to_a, @cls[[]])
+    assert_equal(@cls[].repeated_combination(1).to_a, @cls[])
+
+    a = @cls[1, 2, 3, 4]
+    b = @cls[]
+    a.repeated_combination(4) {|x| b << x; a.replace(@cls[9, 8, 7, 6]) }
+    assert_equal(@cls[9, 8, 7, 6], a)
+    assert_equal(@cls[1, 2, 3, 4].repeated_combination(4).to_a, b)
   end
 
   def test_take
@@ -1361,14 +1663,18 @@ class TestArray < Test::Unit::TestCase
     assert_equal([1, 1, 1], Array.new(3, 1) { 1 })
   end
 
-  def test_aset
+  def test_aset_error
     assert_raise(IndexError) { [0][-2] = 1 }
     assert_raise(IndexError) { [0][LONGP] = 2 }
     assert_raise(IndexError) { [0][(LONGP + 1) / 2 - 1] = 2 }
+    assert_raise(IndexError) { [0][LONGP..-1] = 2 }
     a = [0]
     a[2] = 4
     assert_equal([0, nil, 4], a)
     assert_raise(ArgumentError) { [0][0, 0, 0] = 0 }
+    assert_raise(ArgumentError) { [0].freeze[0, 0, 0] = 0 }
+    assert_raise(TypeError) { [0][:foo] = 0 }
+    assert_raise(RuntimeError) { [0].freeze[:foo] = 0 }
   end
 
   def test_first2
@@ -1385,8 +1691,9 @@ class TestArray < Test::Unit::TestCase
     assert_equal([2, 3], a)
   end
 
-  def test_unshift2
-    Struct.new(:a, :b, :c)
+  def test_unshift_error
+    assert_raise(RuntimeError) { [].freeze.unshift('cat') }
+    assert_raise(RuntimeError) { [].freeze.unshift() }
   end
 
   def test_aref
@@ -1445,12 +1752,20 @@ class TestArray < Test::Unit::TestCase
     assert_raise(ArgumentError) { a.insert }
     assert_equal([0, 1, 2], a.insert(-1, 2))
     assert_equal([0, 1, 3, 2], a.insert(-2, 3))
+    assert_raise(RuntimeError) { [0].freeze.insert(0)}
+    assert_raise(ArgumentError) { [0].freeze.insert }
   end
 
   def test_join2
     a = []
     a << a
-    assert_equal("[...]", a.join)
+    assert_raise(ArgumentError){a.join}
+
+    def (a = Object.new).to_ary
+      [self]
+    end
+    assert_raise(ArgumentError, '[ruby-core:24150]'){[a].join}
+    assert_equal("12345", [1,[2,[3,4],5]].join)
   end
 
   def test_to_a2
@@ -1471,6 +1786,22 @@ class TestArray < Test::Unit::TestCase
     assert_equal([0, 2], [0, 1, 2, 3].select {|x| x % 2 == 0 })
   end
 
+  # also keep_if
+  def test_select!
+    a = @cls[ 1, 2, 3, 4, 5 ]
+    assert_equal(nil, a.select! { true })
+    assert_equal(a, a.keep_if { true })
+    assert_equal(@cls[1, 2, 3, 4, 5], a)
+
+    a = @cls[ 1, 2, 3, 4, 5 ]
+    assert_equal(a, a.select! { false })
+    assert_equal(@cls[], a)
+
+    a = @cls[ 1, 2, 3, 4, 5 ]
+    assert_equal(a, a.select! { |i| i > 3 })
+    assert_equal(@cls[4, 5], a)
+  end
+
   def test_delete2
     a = [0] * 1024 + [1] + [0] * 1024
     a.delete(0)
@@ -1487,6 +1818,14 @@ class TestArray < Test::Unit::TestCase
     a = []
     [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]) {|x| a << x }
     assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]], a)
+
+    ary = Object.new
+    def ary.to_a;   [1, 2]; end
+    assert_raise(NoMethodError){ %w(a b).zip(ary) }
+    def ary.each; [3, 4].each{|e|yield e}; end
+    assert_equal([['a', 3], ['b', 4]], %w(a b).zip(ary))
+    def ary.to_ary; [5, 6]; end
+    assert_equal([['a', 5], ['b', 6]], %w(a b).zip(ary))
   end
 
   def test_transpose
@@ -1511,22 +1850,28 @@ class TestArray < Test::Unit::TestCase
     o = Object.new
     def o.to_ary; end
     def o.==(x); :foo; end
-    assert(:foo, [0, 1, 2] == o)
-    assert([0, 1, 2] != [0, 1, 3])
+    assert_equal([0, 1, 2], o)
+    assert_not_equal([0, 1, 2], [0, 1, 3])
   end
 
   def test_hash2
     a = []
     a << a
-    b = []
-    b << b
-    assert_equal(a.hash, b.hash)
+    assert_equal([[a]].hash, a.hash)
+    assert_not_equal([a, a].hash, a.hash) # Implementation dependent
   end
 
-  def test_flatten2
+  def test_flatten_error
     a = []
     a << a
     assert_raise(ArgumentError) { a.flatten }
+
+    f = [].freeze
+    assert_raise(ArgumentError) { a.flatten!(1, 2) }
+    assert_raise(TypeError) { a.flatten!(:foo) }
+    assert_raise(ArgumentError) { f.flatten!(1, 2) }
+    assert_raise(RuntimeError) { f.flatten! }
+    assert_raise(RuntimeError) { f.flatten!(:foo) }
   end
 
   def test_shuffle
@@ -1543,6 +1888,25 @@ class TestArray < Test::Unit::TestCase
         assert([0, 1, 2].include?(sample))
       }
     end
+
+    srand(0)
+    a = (1..18).to_a
+    (0..20).each do |n|
+      100.times do
+        b = a.sample(n)
+        assert_equal([n, 18].min, b.uniq.size)
+        assert_equal(a, (a | b).sort)
+        assert_equal(b.sort, (a & b).sort)
+      end
+
+      h = Hash.new(0)
+      1000.times do
+        a.sample(n).each {|x| h[x] += 1 }
+      end
+      assert_operator(h.values.min * 2, :>=, h.values.max) if n != 0
+    end
+
+    assert_raise(ArgumentError, '[ruby-core:23374]') {[1, 2].sample(-1)}
   end
 
   def test_cycle
@@ -1573,15 +1937,18 @@ class TestArray < Test::Unit::TestCase
   end
 
   def test_combination2
-    assert_raise(RangeError) do
-      (0..100).to_a.combination(50) {}
+    assert_nothing_raised do
+      (0..100).to_a.combination(50) { break }
     end
   end
 
   def test_product2
     a = (0..100).to_a
     assert_raise(RangeError) do
-      a.product(a, a, a, a, a, a, a, a, a, a) {}
+      a.product(a, a, a, a, a, a, a, a, a, a)
+    end
+    assert_nothing_raised(RangeError) do
+      a.product(a, a, a, a, a, a, a, a, a, a) { break }
     end
   end
 
@@ -1590,6 +1957,8 @@ class TestArray < Test::Unit::TestCase
 
   def test_array_subclass
     assert_equal(Array2, Array2[1,2,3].uniq.class, "[ruby-dev:34581]")
+    assert_equal(Array2, Array2[1,2][0,1].class) # embeded
+    assert_equal(Array2, Array2[*(1..100)][1..99].class) #not embeded
   end
 
   def test_inspect
@@ -1600,4 +1969,96 @@ class TestArray < Test::Unit::TestCase
     assert_equal(true, s.tainted?)
     assert_equal(true, s.untrusted?)
   end
+
+  def test_initialize2
+    a = [1] * 1000
+    a.instance_eval { initialize }
+    assert_equal([], a)
+  end
+
+  def test_shift_shared_ary
+    a = (1..100).to_a
+    b = []
+    b.replace(a)
+    assert_equal((1..10).to_a, a.shift(10))
+    assert_equal((11..100).to_a, a)
+  end
+
+  def test_replace_shared_ary
+    a = [1] * 100
+    b = []
+    b.replace(a)
+    a.replace([1, 2, 3])
+    assert_equal([1, 2, 3], a)
+    assert_equal([1] * 100, b)
+  end
+
+  def test_fill_negative_length
+    a = (1..10).to_a
+    a.fill(:foo, 5, -3)
+    assert_equal((1..10).to_a, a)
+  end
+
+  def test_slice_frozen_array
+    a = [1,2,3,4,5].freeze
+    assert_equal([1,2,3,4], a[0,4])
+    assert_equal([2,3,4,5], a[1,4])
+  end
+
+  def test_sort_by!
+    a = [1,3,5,2,4]
+    a.sort_by! {|x| -x }
+    assert_equal([5,4,3,2,1], a)
+  end
+
+  def test_rotate
+    a = [1,2,3,4,5].freeze
+    assert_equal([2,3,4,5,1], a.rotate)
+    assert_equal([5,1,2,3,4], a.rotate(-1))
+    assert_equal([3,4,5,1,2], a.rotate(2))
+    assert_equal([4,5,1,2,3], a.rotate(-2))
+    assert_equal([4,5,1,2,3], a.rotate(13))
+    assert_equal([3,4,5,1,2], a.rotate(-13))
+    a = [1].freeze
+    assert_equal([1], a.rotate)
+    assert_equal([1], a.rotate(2))
+    assert_equal([1], a.rotate(-4))
+    assert_equal([1], a.rotate(13))
+    assert_equal([1], a.rotate(-13))
+    a = [].freeze
+    assert_equal([], a.rotate)
+    assert_equal([], a.rotate(2))
+    assert_equal([], a.rotate(-4))
+    assert_equal([], a.rotate(13))
+    assert_equal([], a.rotate(-13))
+    a = [1,2,3]
+    assert_raise(ArgumentError) { a.rotate(1, 1) }
+  end
+
+  def test_rotate!
+    a = [1,2,3,4,5]
+    assert_equal([2,3,4,5,1], a.rotate!)
+    assert_equal([2,3,4,5,1], a)
+    assert_equal([4,5,1,2,3], a.rotate!(2))
+    assert_equal([5,1,2,3,4], a.rotate!(-4))
+    assert_equal([3,4,5,1,2], a.rotate!(13))
+    assert_equal([5,1,2,3,4], a.rotate!(-13))
+    a = [1]
+    assert_equal([1], a.rotate!)
+    assert_equal([1], a.rotate!(2))
+    assert_equal([1], a.rotate!(-4))
+    assert_equal([1], a.rotate!(13))
+    assert_equal([1], a.rotate!(-13))
+    a = []
+    assert_equal([], a.rotate!)
+    assert_equal([], a.rotate!(2))
+    assert_equal([], a.rotate!(-4))
+    assert_equal([], a.rotate!(13))
+    assert_equal([], a.rotate!(-13))
+    a = [].freeze
+    e = assert_raise(RuntimeError) {a.rotate!}
+    assert_match(/can't modify frozen array/, e.message)
+    a = [1,2,3]
+    assert_raise(ArgumentError) { a.rotate!(1, 1) }
+  end
 end
