@@ -3,6 +3,13 @@ require "fileutils"
 require "tmpdir"
 
 class TestFileExhaustive < Test::Unit::TestCase
+  def assert_incompatible_encoding
+    d = "\u{3042}\u{3044}".encode("utf-16le")
+    assert_raise(Encoding::CompatibilityError) {yield d}
+    m = Class.new {define_method(:to_path) {d}}
+    assert_raise(Encoding::CompatibilityError) {yield m.new}
+  end
+
   def setup
     @dir = Dir.mktmpdir("rubytest-file")
     File.chown(-1, Process.gid, @dir)
@@ -34,7 +41,7 @@ class TestFileExhaustive < Test::Unit::TestCase
   def make_file(content, file = @file)
     open(file, "w") {|fh| fh << content }
   end
-  
+
   def make_tmp_filename(prefix)
     @hardlinkfile = @dir + "/" + prefix + File.basename(__FILE__) + ".#{$$}.test"
   end
@@ -42,7 +49,7 @@ class TestFileExhaustive < Test::Unit::TestCase
   def test_path
     file = @file
 
-    assert_equal(file, File.new(file).path)
+    assert_equal(file, File.open(file) {|f| f.path})
     assert_equal(file, File.path(file))
     o = Object.new
     class << o; self; end.class_eval do
@@ -76,7 +83,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -90,9 +99,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_kind_of(String, fs1.inspect)
     end
     assert_raise(Errno::ENOENT) { File.stat(@nofile) }
-    assert_kind_of(File::Stat, File.new(@file).stat)
+    assert_kind_of(File::Stat, File.open(@file) {|f| f.stat})
     assert_raise(Errno::ENOENT) { File.lstat(@nofile) }
-    assert_kind_of(File::Stat, File.new(@file).lstat)
+    assert_kind_of(File::Stat, File.open(@file) {|f| f.lstat})
   end
 
   def test_directory_p
@@ -142,6 +151,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_readable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File.readable?(@file)))
     File.chmod(0600, @file)
@@ -151,6 +161,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_readable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File.readable_real?(@file)))
     File.chmod(0600, @file)
@@ -171,6 +182,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_writable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File.writable?(@file)))
     File.chmod(0600, @file)
@@ -180,6 +192,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_writable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File.writable_real?(@file)))
     File.chmod(0600, @file)
@@ -255,7 +268,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert(!(File.identical?(@nofile, @file)))
   end
 
-  def test_size
+  def test_s_size
     assert_integer(File.size(@dir))
     assert_equal(3, File.size(@file))
     assert_equal(0, File.size(@zerofile))
@@ -272,7 +285,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_atime
     t1 = File.atime(@file)
-    t2 = File.new(@file).atime
+    t2 = File.open(@file) {|f| f.atime}
     assert_kind_of(Time, t1)
     assert_kind_of(Time, t2)
     assert_equal(t1, t2)
@@ -281,7 +294,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_mtime
     t1 = File.mtime(@file)
-    t2 = File.new(@file).mtime
+    t2 = File.open(@file) {|f| f.mtime}
     assert_kind_of(Time, t1)
     assert_kind_of(Time, t2)
     assert_equal(t1, t2)
@@ -290,7 +303,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_ctime
     t1 = File.ctime(@file)
-    t2 = File.new(@file).ctime
+    t2 = File.open(@file) {|f| f.ctime}
     assert_kind_of(Time, t1)
     assert_kind_of(Time, t2)
     assert_equal(t1, t2)
@@ -301,7 +314,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
     assert_equal(1, File.chmod(0444, @file))
     assert_equal(0444, File.stat(@file).mode % 01000)
-    assert_equal(0, File.new(@file).chmod(0222))
+    assert_equal(0, File.open(@file) {|f| f.chmod(0222)})
     assert_equal(0222, File.stat(@file).mode % 01000)
     File.chmod(0600, @file)
     assert_raise(Errno::ENOENT) { File.chmod(0600, @nofile) }
@@ -380,13 +393,32 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(@file, File.expand_path(@file + " "))
       assert_equal(@file, File.expand_path(@file + "."))
       assert_equal(@file, File.expand_path(@file + "::$DATA"))
+      assert_match(/\Ac:\//i, File.expand_path('c:'), '[ruby-core:31591]')
     end
+    assert_kind_of(String, File.expand_path("~"))
+    assert_raise(ArgumentError) { File.expand_path("~foo_bar_baz_unknown_user_wahaha") }
+    assert_raise(ArgumentError) { File.expand_path("~foo_bar_baz_unknown_user_wahaha", "/") }
+    begin
+      bug3630 = '[ruby-core:31537]'
+      home = ENV["HOME"]
+      ENV["HOME"] = nil
+      assert_raise(ArgumentError) { File.expand_path("~") }
+      ENV["HOME"] = "~"
+      assert_raise(ArgumentError, bug3630) { File.expand_path("~") }
+      ENV["HOME"] = "."
+      assert_raise(ArgumentError, bug3630) { File.expand_path("~") }
+    ensure
+      ENV["HOME"] = home
+    end
+    assert_incompatible_encoding {|d| File.expand_path(d)}
   end
 
   def test_basename
     assert_equal(File.basename(@file).sub(/\.test$/, ""), File.basename(@file, ".test"))
-    assert_equal("", File.basename(""))
-    assert_equal("foo", File.basename("foo"))
+    assert_equal("", s = File.basename(""))
+    assert(!s.frozen?, '[ruby-core:24199]')
+    assert_equal("foo", s = File.basename("foo"))
+    assert(!s.frozen?, '[ruby-core:24199]')
     assert_equal("foo", File.basename("foo", ".ext"))
     assert_equal("foo", File.basename("foo.ext", ".ext"))
     assert_equal("foo", File.basename("foo.ext", ".*"))
@@ -403,35 +435,43 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(basename, File.basename(@file + ".", ".*"))
       assert_equal(basename, File.basename(@file + "::$DATA", ".*"))
     end
+
+    assert_incompatible_encoding {|d| File.basename(d)}
+    assert_incompatible_encoding {|d| File.basename(d, ".*")}
+    assert_raise(Encoding::CompatibilityError) {File.basename("foo.ext", ".*".encode("utf-16le"))}
   end
 
   def test_dirname
     assert(@file.start_with?(File.dirname(@file)))
     assert_equal(".", File.dirname(""))
+    assert_incompatible_encoding {|d| File.dirname(d)}
   end
 
   def test_extname
-    assert(".test", File.extname(@file))
-    assert_equal("", File.extname("foo"))
-    assert_equal("", File.extname("/foo"))
-    assert_equal("", File.extname(".foo"))
-    assert_equal("", File.extname("/.foo"))
-    assert_equal("", File.extname("bar/.foo"))
-    assert_equal("", File.extname("/bar/.foo"))
-    assert_equal(".ext", File.extname("foo.ext"))
-    assert_equal(".ext", File.extname("/foo.ext"))
-    assert_equal(".ext", File.extname(".foo.ext"))
-    assert_equal(".ext", File.extname("/.foo.ext"))
-    assert_equal(".ext", File.extname("bar/.foo.ext"))
-    assert_equal(".ext", File.extname("/bar/.foo.ext"))
-    assert_equal("", File.extname(""))
+    assert_equal(".test", File.extname(@file))
+    prefixes = ["", "/", ".", "/.", "bar/.", "/bar/."]
+    infixes = ["", " ", "."]
+    infixes2 = infixes + [".ext "]
+    appendixes = [""]
     if /cygwin|mingw|mswin|bccwin/ =~ RUBY_PLATFORM
-      assert_equal("", File.extname("foo "))
-      assert_equal(".ext", File.extname("foo.ext "))
-      assert_equal(".ext", File.extname("foo.ext."))
-      assert_equal(".ext", File.extname("foo.ext::$DATA"))
-      assert_equal("", File.extname("foo::$DATA.ext"))
+      appendixes << " " << "." << "::$DATA" << "::$DATA.bar"
     end
+    prefixes.each do |prefix|
+      appendixes.each do |appendix|
+        infixes.each do |infix|
+          path = "#{prefix}foo#{infix}#{appendix}"
+          assert_equal("", File.extname(path), "File.extname(#{path.inspect})")
+        end
+        infixes2.each do |infix|
+          path = "#{prefix}foo#{infix}.ext#{appendix}"
+          assert_equal(".ext", File.extname(path), "File.extname(#{path.inspect})")
+        end
+      end
+    end
+    bug3175 = '[ruby-core:29627]'
+    assert_equal(".rb", File.extname("/tmp//bla.rb"), bug3175)
+
+    assert_incompatible_encoding {|d| File.extname(d)}
   end
 
   def test_split
@@ -470,7 +510,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     f.close
     make_file("foo", @file)
 
-    assert_raise(IOError) { File.new(@file).truncate(0) }
+    assert_raise(IOError) { File.open(@file) {|ff| ff.truncate(0)} }
   rescue NotImplementedError
   end
 
@@ -508,7 +548,7 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(File.socket?(f), test(?S, f))
       assert_equal(File.setuid?(f), test(?u, f))
       assert_equal(File.writable?(f), test(?w, f))
-      assert_equal(File.world_writable?(f), test(?W, f))
+      assert_equal(File.writable_real?(f), test(?W, f))
       assert_equal(File.executable?(f), test(?x, f))
       assert_equal(File.executable_real?(f), test(?X, f))
       assert_equal(File.zero?(f), test(?z, f))
@@ -548,7 +588,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -611,6 +653,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_readable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File::Stat.new(@file).readable?))
     File.chmod(0600, @file)
@@ -619,6 +662,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_readable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0200, @file)
     assert(!(File::Stat.new(@file).readable_real?))
     File.chmod(0600, @file)
@@ -637,6 +681,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_writable_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File::Stat.new(@file).writable?))
     File.chmod(0600, @file)
@@ -645,6 +690,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def test_stat_writable_real_p
     return if /cygwin|mswin|bccwin|mingw|emx/ =~ RUBY_PLATFORM
+    return if Process.euid == 0
     File.chmod(0400, @file)
     assert(!(File::Stat.new(@file).writable_real?))
     File.chmod(0600, @file)
@@ -724,4 +770,18 @@ class TestFileExhaustive < Test::Unit::TestCase
       end.join
     end
   end
+
+  def test_size
+    assert_equal(3, File.open(@file) {|f| f.size })
+    File.open(@file, "a") do |f|
+      f.write("bar")
+      assert_equal(6, f.size)
+    end
+  end
+
+  def test_absolute_path
+    assert_equal(File.join(Dir.pwd, "~foo"), File.absolute_path("~foo"))
+    dir = File.expand_path("/bar")
+    assert_equal(File.join(dir, "~foo"), File.absolute_path("~foo", dir))
+  end
 end
