@@ -24,7 +24,7 @@ class TestFloat < Test::Unit::TestCase
     assert_equal(false, (x >= y))
   end
   def test_nan
-    nan = 0.0/0
+    nan = Float::NAN
     nan_test(nan, nan)
     nan_test(nan, 0)
     nan_test(nan, 1)
@@ -87,6 +87,18 @@ class TestFloat < Test::Unit::TestCase
     assert_raise(ArgumentError){Float("1__1")}
     # add expected behaviour here.
     assert_equal(10, Float("1_0"))
+
+    assert_equal([ 0.0].pack('G'), [Float(" 0x0p+0").to_f].pack('G'))
+    assert_equal([-0.0].pack('G'), [Float("-0x0p+0").to_f].pack('G'))
+    assert_equal(255.0,     Float("0Xff"))
+    assert_equal(255.5,     Float("0Xff.8"))
+    assert_equal(1.0,       Float("0X1.P+0"))
+    assert_equal(1024.0,    Float("0x1p10"))
+    assert_equal(1024.0,    Float("0x1p+10"))
+    assert_equal(0.0009765625, Float("0x1p-10"))
+    assert_equal(2.6881171418161356e+43, Float("0x1.3494a9b171bf5p+144"))
+    assert_equal(-3.720075976020836e-44, Float("-0x1.a8c1f14e2af5dp-145"))
+ 
   end
 
   def test_divmod
@@ -118,12 +130,19 @@ class TestFloat < Test::Unit::TestCase
   end
 
   def test_to_s
-    inf = 1.0 / 0.0
+    inf = Float::INFINITY
     assert_equal("Infinity", inf.to_s)
     assert_equal("-Infinity", (-inf).to_s)
     assert_equal("NaN", (inf / inf).to_s)
 
-    assert_equal("1.0e+14", 10000_00000_00000.0.to_s)
+    assert_equal("1.0e+18", 1000_00000_00000_00000.0.to_s)
+
+    bug3273 = '[ruby-core:30145]'
+    [0.21611564636388508, 0.56].each do |f|
+      s = f.to_s
+      assert_equal(f, s.to_f, bug3273)
+      assert_not_equal(f, s.chop.to_f, bug3273)
+    end
   end
 
   def test_coerce
@@ -171,10 +190,8 @@ class TestFloat < Test::Unit::TestCase
     assert_equal([1.0, 0.0], 2.0.divmod(2.0))
     assert_raise(TypeError) { 2.0.divmod(nil) }
 
-    inf = 1.0 / 0.0
-    a, b = inf.divmod(0)
-    assert(a.infinite?)
-    assert(b.nan?)
+    inf = Float::INFINITY
+    assert_raise(ZeroDivisionError) {inf.divmod(0)}
 
     a, b = (2.0**32).divmod(1.0)
     assert_equal(2**32, a)
@@ -188,8 +205,8 @@ class TestFloat < Test::Unit::TestCase
   end
 
   def test_eql
-    inf = 1.0 / 0.0
-    nan = inf / inf
+    inf = Float::INFINITY
+    nan = Float::NAN
     assert(1.0.eql?(1.0))
     assert(inf.eql?(inf))
     assert(!(nan.eql?(nan)))
@@ -202,8 +219,8 @@ class TestFloat < Test::Unit::TestCase
   end
 
   def test_cmp
-    inf = 1.0 / 0.0
-    nan = inf / inf
+    inf = Float::INFINITY
+    nan = Float::NAN
     assert_equal(0, 1.0 <=> 1.0)
     assert_equal(1, 1.0 <=> 0.0)
     assert_equal(-1, 1.0 <=> 2.0)
@@ -217,6 +234,11 @@ class TestFloat < Test::Unit::TestCase
 
     assert_equal(-1, 1.0 <=> 2**32)
 
+    assert_equal(1, inf <=> (Float::MAX.to_i*2))
+    assert_equal(-1, -inf <=> (-Float::MAX.to_i*2))
+    assert_equal(-1, (Float::MAX.to_i*2) <=> inf)
+    assert_equal(1, (-Float::MAX.to_i*2) <=> -inf)
+
     assert_raise(ArgumentError) { 1.0 > nil }
     assert_raise(ArgumentError) { 1.0 >= nil }
     assert_raise(ArgumentError) { 1.0 < nil }
@@ -229,14 +251,14 @@ class TestFloat < Test::Unit::TestCase
   end
 
   def test_infinite_p
-    inf = 1.0 / 0.0
-    assert(1, inf.infinite?)
-    assert(1, (-inf).infinite?)
+    inf = Float::INFINITY
+    assert_equal(1, inf.infinite?)
+    assert_equal(-1, (-inf).infinite?)
     assert_nil(1.0.infinite?)
   end
 
   def test_finite_p
-    inf = 1.0 / 0.0
+    inf = Float::INFINITY
     assert(!(inf.finite?))
     assert(!((-inf).finite?))
     assert(1.0.finite?)
@@ -263,7 +285,7 @@ class TestFloat < Test::Unit::TestCase
     assert_equal(-2, (-2.0).round)
     assert_equal(-2, (-2.0).truncate)
 
-    inf = 1.0/0.0
+    inf = Float::INFINITY
     assert_raise(FloatDomainError) { inf.floor }
     assert_raise(FloatDomainError) { inf.ceil }
     assert_raise(FloatDomainError) { inf.round }
@@ -275,13 +297,6 @@ class TestFloat < Test::Unit::TestCase
     assert_equal(11100.0, 11111.1.round(-2))
   end
 
-  def test_induced_from
-    assert_equal(1.0, Float.induced_from(1))
-    assert_equal(1.0, Float.induced_from(1.0))
-    assert_raise(TypeError) { Float.induced_from(nil) }
-  end
-
-
   VS = [
     18446744073709551617.0,
     18446744073709551616.0,
@@ -411,13 +426,13 @@ class TestFloat < Test::Unit::TestCase
   def test_Float
     assert_in_delta(0.125, Float("0.1_2_5"), 0.00001)
     assert_in_delta(0.125, "0.1_2_5__".to_f, 0.00001)
-    assert(Float(([1] * 10000).join).infinite?)
+    assert_equal(1, Float(([1] * 10000).join).infinite?)
     assert(!Float(([1] * 10000).join("_")).infinite?) # is it really OK?
     assert_raise(ArgumentError) { Float("1.0\x001") }
-    assert(Float("1e10_00").infinite?)
+    assert_equal(1, Float("1e10_00").infinite?)
     assert_raise(TypeError) { Float(nil) }
     o = Object.new
-    def o.to_f; inf = 1.0/0.0; inf/inf; end
+    def o.to_f; inf = Float::INFINITY; inf/inf; end
     assert(Float(o).nan?)
   end
 
@@ -430,4 +445,9 @@ class TestFloat < Test::Unit::TestCase
     end
   end
 
+  def test_sleep_with_Float
+    assert_nothing_raised("[ruby-core:23282]") do
+      sleep(0.1+0.1+0.1+0.1+0.1+0.1+0.1+0.1+0.1+0.1)
+    end
+  end
 end
