@@ -1,4 +1,6 @@
 require 'test/unit'
+require 'delegate'
+require 'timeout'
 
 class TestRange < Test::Unit::TestCase
   def test_range_string
@@ -9,6 +11,19 @@ class TestRange < Test::Unit::TestCase
     assert_equal(["a", "b"], ("a" .. "b").to_a)
   end
 
+  def test_range_numeric_string
+    assert_equal(["6", "7", "8"], ("6".."8").to_a, "[ruby-talk:343187]")
+    assert_equal(["6", "7"], ("6"..."8").to_a)
+    assert_equal(["9", "10"], ("9".."10").to_a)
+    assert_equal(["09", "10"], ("09".."10").to_a, "[ruby-dev:39361]")
+    assert_equal(["9", "10"], (SimpleDelegator.new("9").."10").to_a)
+    assert_equal(["9", "10"], ("9"..SimpleDelegator.new("10")).to_a)
+  end
+
+  def test_range_symbol
+    assert_equal([:a, :b], (:a .. :b).to_a)
+  end
+
   def test_evaluation_order
     arr = [1,2]
     r = (arr.shift)..(arr.shift)
@@ -89,6 +104,8 @@ class TestRange < Test::Unit::TestCase
     assert(r != (1..2))
     assert(r != (0..2))
     assert(r != (0...1))
+    subclass = Class.new(Range)
+    assert(r == subclass.new(0,1))
   end
 
   def test_eql
@@ -99,6 +116,8 @@ class TestRange < Test::Unit::TestCase
     assert(!r.eql?(1..2))
     assert(!r.eql?(0..2))
     assert(!r.eql?(0...1))
+    subclass = Class.new(Range)
+    assert(r.eql?(subclass.new(0,1)))
   end
 
   def test_hash
@@ -170,8 +189,12 @@ class TestRange < Test::Unit::TestCase
 
     o1 = Object.new
     o2 = Object.new
-    def o1.<=>(x); -1; end
-    def o2.<=>(x); 0; end
+    def o1.setcmp(v) @cmpresult = v end
+    o1.setcmp(-1)
+    def o1.<=>(x); @cmpresult; end
+    def o2.setcmp(v) @cmpresult = v end
+    o2.setcmp(0)
+    def o2.<=>(x); @cmpresult; end
     class << o1; self; end.class_eval do
       define_method(:succ) { o2 }
     end
@@ -187,19 +210,19 @@ class TestRange < Test::Unit::TestCase
     r2.each {|x| a << x }
     assert_equal([o1], a)
 
-    def o2.<=>(x); 1; end
+    o2.setcmp(1)
 
     a = []
     r1.each {|x| a << x }
     assert_equal([o1], a)
 
-    def o2.<=>(x); nil; end
+    o2.setcmp(nil)
 
     a = []
     r1.each {|x| a << x }
     assert_equal([o1], a)
 
-    def o1.<=>(x); nil; end
+    o1.setcmp(nil)
 
     a = []
     r2.each {|x| a << x }
@@ -251,20 +274,24 @@ class TestRange < Test::Unit::TestCase
   def test_beg_len
     o = Object.new
     assert_raise(TypeError) { [][o] }
-    def o.begin; -10; end
+    class << o; attr_accessor :begin end
+    o.begin = -10
     assert_raise(TypeError) { [][o] }
-    def o.end; 0; end
+    class << o; attr_accessor :end end
+    o.end = 0
     assert_raise(NoMethodError) { [][o] }
-    def o.exclude_end?; false; end
+    def o.exclude_end=(v) @exclude_end = v end
+    def o.exclude_end?() @exclude_end end
+    o.exclude_end = false
     assert_nil([0][o])
     assert_raise(RangeError) { [0][o] = 1 }
-    def o.begin; 10; end
-    def o.end; 10; end
+    o.begin = 10
+    o.end = 10
     assert_nil([0][o])
-    def o.begin; 0; end
+    o.begin = 0
     assert_equal([0], [0][o])
-    def o.begin; 2; end
-    def o.end; 0; end
+    o.begin = 2
+    o.end = 0
     assert_equal([], [0, 1, 2][o])
   end
 
@@ -276,4 +303,37 @@ class TestRange < Test::Unit::TestCase
     o.instance_eval { initialize(o, 1) }
     assert_equal("(... .. ...)..1", o.inspect)
   end
+
+  def test_comparison_when_recursive
+    x = CyclicRange.allocate; x.send(:initialize, x, 1)
+    y = CyclicRange.allocate; y.send(:initialize, y, 1)
+    Timeout.timeout(1) {
+      assert x == y
+      assert x.eql? y
+    }
+
+    z = CyclicRange.allocate; z.send(:initialize, z, :another)
+    Timeout.timeout(1) {
+      assert x != z
+      assert !x.eql?(z)
+    }
+
+    x = CyclicRange.allocate
+    y = CyclicRange.allocate
+    x.send(:initialize, y, 1)
+    y.send(:initialize, x, 1)
+    Timeout.timeout(1) {
+      assert x == y
+      assert x.eql?(y)
+    }
+
+    x = CyclicRange.allocate
+    z = CyclicRange.allocate
+    x.send(:initialize, z, 1)
+    z.send(:initialize, x, :other)
+    Timeout.timeout(1) {
+      assert x != z
+      assert !x.eql?(z)
+    }
+  end
 end
