@@ -1,16 +1,8 @@
-#!/usr/bin/env ruby
-#--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
-# All rights reserved.
-# See LICENSE.txt for permissions.
-#++
-
-require 'test/unit'
-require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
-require 'webrick'
-require 'zlib'
-require 'rubygems/remote_fetcher'
+require_relative 'gemutilities'
 require 'ostruct'
+require 'webrick'
+require 'rubygems/remote_fetcher'
+require 'rubygems/format'
 
 # = Testing Proxy Settings
 #
@@ -111,9 +103,14 @@ gems:
     @fetcher = Gem::RemoteFetcher.fetcher
   end
 
+  def teardown
+    super
+    Gem.configuration[:http_proxy] = nil
+  end
+
   def test_self_fetcher
     fetcher = Gem::RemoteFetcher.fetcher
-    assert_not_nil fetcher
+    refute_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
   end
 
@@ -124,7 +121,7 @@ gems:
 
     fetcher = Gem::RemoteFetcher.fetcher
 
-    assert_not_nil fetcher
+    refute_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri).to_s
   end
@@ -132,8 +129,11 @@ gems:
   def test_self_fetcher_with_proxy_URI
     proxy_uri = URI.parse 'http://proxy.example.com'
     Gem.configuration[:http_proxy] = proxy_uri
+    Gem::RemoteFetcher.fetcher = nil
+
     fetcher = Gem::RemoteFetcher.fetcher
-    assert_not_nil fetcher
+    refute_nil fetcher
+
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri)
   end
@@ -141,11 +141,11 @@ gems:
   def test_fetch_size_bad_uri
     fetcher = Gem::RemoteFetcher.new nil
 
-    e = assert_raise ArgumentError do
+    e = assert_raises ArgumentError do
       fetcher.fetch_size 'gems.example.com/yaml'
     end
 
-    assert_equal 'uri is not an HTTP URI', e.message
+    assert_equal 'uri scheme is invalid', e.message
   end
 
   def test_fetch_size_socket_error
@@ -155,7 +155,7 @@ gems:
     end
 
     uri = 'http://gems.example.com/yaml'
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_size uri
     end
 
@@ -202,7 +202,7 @@ gems:
 
     fetcher = util_fuck_with_fetcher a1_data
 
-    a1_cache_gem = File.join(@gemhome, 'cache', "#{@a1.full_name}.gem")
+    a1_cache_gem = File.join(@gemhome, 'cache', @a1.file_name)
     assert_equal a1_cache_gem, fetcher.download(@a1, 'http://gems.example.com')
     assert_equal("http://gems.example.com/gems/a-1.gem",
                  fetcher.instance_variable_get(:@test_arg).to_s)
@@ -214,20 +214,35 @@ gems:
 
     inst = Gem::RemoteFetcher.fetcher
 
-    assert_equal File.join(@gemhome, 'cache', "#{@a1.full_name}.gem"),
+    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
                  inst.download(@a1, 'http://gems.example.com')
   end
 
   def test_download_local
     FileUtils.mv @a1_gem, @tempdir
-    local_path = File.join @tempdir, "#{@a1.full_name}.gem"
+    local_path = File.join @tempdir, @a1.file_name
     inst = nil
 
     Dir.chdir @tempdir do
       inst = Gem::RemoteFetcher.fetcher
     end
 
-    assert_equal File.join(@gemhome, 'cache', "#{@a1.full_name}.gem"),
+    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
+                 inst.download(@a1, local_path)
+  end
+
+  def test_download_local_space
+    space_path = File.join @tempdir, 'space path'
+    FileUtils.mkdir space_path
+    FileUtils.mv @a1_gem, space_path
+    local_path = File.join space_path, @a1.file_name
+    inst = nil
+
+    Dir.chdir @tempdir do
+      inst = Gem::RemoteFetcher.fetcher
+    end
+
+    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
                  inst.download(@a1, local_path)
   end
 
@@ -241,7 +256,7 @@ gems:
 
     install_dir = File.join @tempdir, 'more_gems'
 
-    a1_cache_gem = File.join install_dir, 'cache', "#{@a1.full_name}.gem"
+    a1_cache_gem = File.join install_dir, 'cache', @a1.file_name
     FileUtils.mkdir_p(File.dirname(a1_cache_gem))
     actual = fetcher.download(@a1, 'http://gems.example.com', install_dir)
 
@@ -255,7 +270,7 @@ gems:
   unless win_platform? # File.chmod doesn't work
     def test_download_local_read_only
       FileUtils.mv @a1_gem, @tempdir
-      local_path = File.join @tempdir, "#{@a1.full_name}.gem"
+      local_path = File.join @tempdir, @a1.file_name
       inst = nil
       File.chmod 0555, File.join(@gemhome, 'cache')
 
@@ -263,20 +278,19 @@ gems:
         inst = Gem::RemoteFetcher.fetcher
       end
 
-      assert_equal File.join(@tempdir, "#{@a1.full_name}.gem"),
+      assert_equal File.join(@tempdir, @a1.file_name),
         inst.download(@a1, local_path)
     ensure
       File.chmod 0755, File.join(@gemhome, 'cache')
     end
-    
+
     def test_download_read_only
       File.chmod 0555, File.join(@gemhome, 'cache')
       File.chmod 0555, File.join(@gemhome)
 
       fetcher = util_fuck_with_fetcher File.read(@a1_gem)
       fetcher.download(@a1, 'http://gems.example.com')
-      assert File.exist?(File.join(@userhome, '.gem',
-                                   'cache', "#{@a1.full_name}.gem"))
+      assert File.exist?(File.join(Gem.user_dir, 'cache', @a1.file_name))
     ensure
       File.chmod 0755, File.join(@gemhome)
       File.chmod 0755, File.join(@gemhome, 'cache')
@@ -298,7 +312,7 @@ gems:
 
     fetcher = util_fuck_with_fetcher e1_data, :blow_chunks
 
-    e1_cache_gem = File.join(@gemhome, 'cache', "#{e1.full_name}.gem")
+    e1_cache_gem = File.join(@gemhome, 'cache', e1.file_name)
 
     assert_equal e1_cache_gem, fetcher.download(e1, 'http://gems.example.com')
 
@@ -307,10 +321,27 @@ gems:
     assert File.exist?(e1_cache_gem)
   end
 
+  def test_download_same_file
+    FileUtils.mv @a1_gem, @tempdir
+    local_path = File.join @tempdir, @a1.file_name
+    inst = nil
+
+    Dir.chdir @tempdir do
+      inst = Gem::RemoteFetcher.fetcher
+    end
+
+    cache_path = File.join @gemhome, 'cache', @a1.file_name
+    FileUtils.mv local_path, cache_path
+
+    gem = Gem::Format.from_file_by_path cache_path
+
+    assert_equal cache_path, inst.download(gem.spec, cache_path)
+  end
+
   def test_download_unsupported
     inst = Gem::RemoteFetcher.fetcher
 
-    e = assert_raise Gem::InstallError do
+    e = assert_raises Gem::InstallError do
       inst.download @a1, 'ftp://gems.rubyforge.org'
     end
 
@@ -391,6 +422,16 @@ gems:
     assert_equal 'foo', fetcher.fetch_path(@uri + 'foo.gz')
   end
 
+  def test_fetch_path_gzip_unmodified
+    fetcher = Gem::RemoteFetcher.new nil
+
+    def fetcher.open_uri_or_path(uri, mtime, head = nil)
+      nil
+    end
+
+    assert_equal nil, fetcher.fetch_path(@uri + 'foo.gz', Time.at(0))
+  end
+
   def test_fetch_path_io_error
     fetcher = Gem::RemoteFetcher.new nil
 
@@ -398,7 +439,7 @@ gems:
       raise EOFError
     end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -413,7 +454,7 @@ gems:
       raise SocketError
     end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -428,7 +469,7 @@ gems:
       raise Errno::ECONNREFUSED, 'connect(2)'
     end
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -441,10 +482,17 @@ gems:
     fetcher = Gem::RemoteFetcher.new nil
 
     def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      ''
+      nil
     end
 
-    assert_equal '', fetcher.fetch_path(URI.parse(@gem_repo), Time.at(0))
+    assert_equal nil, fetcher.fetch_path(URI.parse(@gem_repo), Time.at(0))
+  end
+
+  def test_get_proxy_from_env_auto_normalizes
+    fetcher = Gem::RemoteFetcher.new(nil)
+    ENV['HTTP_PROXY'] = 'fakeurl:12345'
+
+    assert_equal('http://fakeurl:12345', fetcher.get_proxy_from_env.to_s)
   end
 
   def test_get_proxy_from_env_empty
@@ -504,7 +552,7 @@ gems:
     def conn.request(req)
       unless defined? @requested then
         @requested = true
-        res = Net::HTTPRedirection.new nil, 301, nil
+        res = Net::HTTPMovedPermanently.new nil, 301, nil
         res.add_field 'Location', 'http://gems.example.com/real_path'
         res
       else
@@ -528,7 +576,7 @@ gems:
     conn = Object.new
     def conn.started?() true end
     def conn.request(req)
-      res = Net::HTTPRedirection.new nil, 301, nil
+      res = Net::HTTPMovedPermanently.new nil, 301, nil
       res.add_field 'Location', 'http://gems.example.com/redirect'
       res
     end
@@ -536,7 +584,7 @@ gems:
     conn = { 'gems.example.com:80' => conn }
     fetcher.instance_variable_set :@connections, conn
 
-    e = assert_raise Gem::RemoteFetcher::FetchError do
+    e = assert_raises Gem::RemoteFetcher::FetchError do
       fetcher.open_uri_or_path 'http://gems.example.com/redirect'
     end
 
