@@ -16,31 +16,40 @@ static inline VALUE rb_vm_set_finish_env(rb_thread_t * th);
 static inline VALUE vm_yield_with_cref(rb_thread_t *th, int argc, const VALUE *argv, const NODE *cref);
 static inline VALUE vm_yield(rb_thread_t *th, int argc, const VALUE *argv);
 static inline VALUE vm_backtrace(rb_thread_t *th, int lev);
-static NODE *vm_cref_push(rb_thread_t *th, VALUE klass, int noex);
+static int vm_backtrace_each(rb_thread_t *th, int lev, void (*init)(void *), rb_backtrace_iter_func *iter, void *arg);
+static NODE *vm_cref_push(rb_thread_t *th, VALUE klass, int noex, rb_block_t *blockptr);
 static VALUE vm_exec(rb_thread_t *th);
 static void vm_set_eval_stack(rb_thread_t * th, VALUE iseqval, const NODE *cref);
 static int vm_collect_local_variables_in_heap(rb_thread_t *th, VALUE *dfp, VALUE ary);
 
+typedef enum call_type {
+    CALL_PUBLIC,
+    CALL_FCALL,
+    CALL_VCALL,
+    CALL_TYPE_MAX
+} call_type;
+
+static VALUE send_internal(int argc, const VALUE *argv, VALUE recv, call_type scope);
+
 static inline VALUE
-vm_call0(rb_thread_t * th, VALUE klass, VALUE recv, VALUE id, ID oid,
-	 int argc, const VALUE *argv, const NODE *body, int nosuper)
+vm_call0(rb_thread_t* th, VALUE recv, VALUE id, int argc, const VALUE *argv,
+	 const rb_method_entry_t *me)
 {
+    const rb_method_definition_t *def = me->def;
     VALUE val;
-    rb_block_t *blockptr = 0;
-
-    if (0) printf("id: %s, nd: %s, argc: %d, passed: %p\n",
-		  rb_id2name(id), ruby_node_name(nd_type(body)),
-		  argc, (void *)th->passed_block);
+    VALUE klass = me->klass;
+    const rb_block_t *blockptr = 0;
 
+    if (!def) return Qnil;
     if (th->passed_block) {
 	blockptr = th->passed_block;
 	th->passed_block = 0;
     }
+
   again:
-    switch (nd_type(body)) {
-      case RUBY_VM_METHOD_NODE:{
+    switch (def->type) {
+      case VM_METHOD_TYPE_ISEQ: {
 	rb_control_frame_t *reg_cfp;
-	VALUE iseqval = (VALUE)body->nd_body;
 	int i;
 
 	rb_vm_set_finish_env(th);
@@ -53,11 +62,12 @@ vm_call0(rb_thread_t * th, VALUE klass, VALUE recv, VALUE id, ID oid,
 	    *reg_cfp->sp++ = argv[i];
 	}
 
-	vm_setup_method(th, reg_cfp, argc, blockptr, 0, iseqval, recv);
+	vm_setup_method(th, reg_cfp, recv, argc, blockptr, 0 /* flag */, me);
 	val = vm_exec(th);
 	break;
       }
-      case NODE_CFUNC: {
+      case VM_METHOD_TYPE_NOTIMPLEMENTED:
+      case VM_METHOD_TYPE_CFUNC: {
 	EXEC_EVENT_HOOK(th, RUBY_EVENT_C_CALL, recv, id, klass);
 	{
 	    rb_control_frame_t *reg_cfp = th->cfp;
@@ -65,96 +75,113 @@ vm_call0(rb_thread_t * th, VALUE klass, VALUE recv, VALUE id, ID oid,
 		vm_push_frame(th, 0, VM_FRAME_MAGIC_CFUNC,
 			      recv, (VALUE)blockptr, 0, reg_cfp->sp, 0, 1);
 
-	    cfp->method_id = oid;
-	    cfp->method_class = klass;
-
-	    val = call_cfunc(body->nd_cfnc, recv, body->nd_argc, argc, argv);
+	    cfp->me = me;
+	    val = call_cfunc(def->body.cfunc.func, recv, def->body.cfunc.argc, argc, argv);
 
 	    if (reg_cfp != th->cfp + 1) {
-		SDR2(reg_cfp);
-		SDR2(th->cfp-5);
 		rb_bug("cfp consistency error - call0");
-		th->cfp = reg_cfp;
 	    }
 	    vm_pop_frame(th);
 	}
 	EXEC_EVENT_HOOK(th, RUBY_EVENT_C_RETURN, recv, id, klass);
 	break;
       }
-      case NODE_ATTRSET:{
+      case VM_METHOD_TYPE_ATTRSET: {
 	if (argc != 1) {
 	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", argc);
 	}
-	val = rb_ivar_set(recv, body->nd_vid, argv[0]);
+	val = rb_ivar_set(recv, def->body.attr.id, argv[0]);
 	break;
       }
-      case NODE_IVAR: {
+      case VM_METHOD_TYPE_IVAR: {
 	if (argc != 0) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)",
-		     argc);
+	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)", argc);
 	}
-	val = rb_attr_get(recv, body->nd_vid);
+	val = rb_attr_get(recv, def->body.attr.id);
 	break;
       }
-      case NODE_BMETHOD:{
-	val = vm_call_bmethod(th, oid, body->nd_cval,
-			      recv, klass, argc, (VALUE *)argv, blockptr);
+      case VM_METHOD_TYPE_BMETHOD: {
+	val = vm_call_bmethod(th, recv, argc, argv, blockptr, me);
 	break;
       }
-      case NODE_ZSUPER:{
+      case VM_METHOD_TYPE_ZSUPER: {
 	klass = RCLASS_SUPER(klass);
-	if (!klass || !(body = rb_method_node(klass, id))) {
-	    return method_missing(recv, id, argc, argv, 0);
+	if (!klass || !(me = rb_method_entry(klass, id))) {
+	    return method_missing(recv, id, argc, argv, NOEX_SUPER);
 	}
 	RUBY_VM_CHECK_INTS();
-	nosuper = CALL_SUPER;
-	body = body->nd_body;
+	if (!(def = me->def)) return Qnil;
 	goto again;
       }
+      case VM_METHOD_TYPE_MISSING: {
+	VALUE new_args = rb_ary_new4(argc, argv);
+
+	RB_GC_GUARD(new_args);
+	rb_ary_unshift(new_args, ID2SYM(id));
+	return rb_funcall2(recv, idMethodMissing,
+			   argc+1, RARRAY_PTR(new_args));
+      }
+      case VM_METHOD_TYPE_OPTIMIZED: {
+	switch (def->body.optimize_type) {
+	  case OPTIMIZED_METHOD_TYPE_SEND:
+	    val = send_internal(argc, argv, recv, CALL_FCALL);
+	    break;
+	  case OPTIMIZED_METHOD_TYPE_CALL: {
+	    rb_proc_t *proc;
+	    GetProcPtr(recv, proc);
+	    val = rb_vm_invoke_proc(th, proc, proc->block.self, argc, argv, blockptr);
+	    break;
+	  }
+	  default:
+	    rb_bug("vm_call0: unsupported optimized method type (%d)", def->body.optimize_type);
+	    val = Qundef;
+	    break;
+	}
+	break;
+      }
       default:
-	rb_bug("unsupported: vm_call0(%s)", ruby_node_name(nd_type(body)));
+	rb_bug("vm_call0: unsupported method type (%d)", def->type);
+	val = Qundef;
     }
     RUBY_VM_CHECK_INTS();
     return val;
 }
 
 VALUE
-rb_vm_call(rb_thread_t * th, VALUE klass, VALUE recv, VALUE id, ID oid,
-	   int argc, const VALUE *argv, const NODE *body, int nosuper)
+rb_vm_call(rb_thread_t *th, VALUE recv, VALUE id, int argc, const VALUE *argv,
+	   const rb_method_entry_t *me)
 {
-    return vm_call0(th, klass, recv, id, oid, argc, argv, body, nosuper);
+    return vm_call0(th, recv, id, argc, argv, me);
 }
 
 static inline VALUE
-vm_call_super(rb_thread_t * const th, const int argc, const VALUE * const argv)
+vm_call_super(rb_thread_t *th, int argc, const VALUE *argv)
 {
     VALUE recv = th->cfp->self;
     VALUE klass;
     ID id;
-    NODE *body;
+    rb_method_entry_t *me;
     rb_control_frame_t *cfp = th->cfp;
 
     if (!cfp->iseq) {
-	klass = cfp->method_class;
+	klass = cfp->me->klass;
 	klass = RCLASS_SUPER(klass);
 
 	if (klass == 0) {
-	    klass = vm_search_normal_superclass(cfp->method_class, recv);
+	    klass = vm_search_normal_superclass(cfp->me->klass, recv);
 	}
-
-	id = cfp->method_id;
+	id = cfp->me->def->original_id;
     }
     else {
 	rb_bug("vm_call_super: should not be reached");
     }
 
-    body = rb_method_node(klass, id);	/* this returns NODE_METHOD */
-    if (!body) {
-	return method_missing(recv, id, argc, argv, 0);
+    me = rb_method_entry(klass, id);
+    if (!me) {
+	return method_missing(recv, id, argc, argv, NOEX_SUPER);
     }
 
-    return vm_call0(th, klass, recv, id, (ID)body->nd_file,
-		    argc, argv, body->nd_body, CALL_SUPER);
+    return vm_call0(th, recv, id, argc, argv, me);
 }
 
 VALUE
@@ -175,58 +202,203 @@ stack_check(void)
     }
 }
 
+static inline rb_method_entry_t *rb_search_method_entry(VALUE recv, ID mid);
+static inline int rb_method_call_status(rb_thread_t *th, rb_method_entry_t *me, call_type scope, VALUE self);
+#define NOEX_OK NOEX_NOSUPER
+
+/*!
+ * \internal
+ * calls the specified method.
+ *
+ * This function is called by functions in rb_call* family.
+ * \param recv   receiver of the method
+ * \param mid    an ID that represents the name of the method
+ * \param argc   the number of method arguments
+ * \param argv   a pointer to an array of method arguments
+ * \param scope
+ * \param self   self in the caller. Qundef means the current control frame's self.
+ *
+ * \note \a self is used in order to controlling access to protected methods.
+ */
 static inline VALUE
-rb_call0(VALUE klass, VALUE recv, ID mid, int argc, const VALUE *argv,
-	 int scope, VALUE self)
+rb_call0(VALUE recv, ID mid, int argc, const VALUE *argv,
+	 call_type scope, VALUE self)
 {
-    NODE *body, *method;
-    int noex;
-    ID id = mid;
-    struct cache_entry *ent;
+    rb_method_entry_t *me = rb_search_method_entry(recv, mid);
     rb_thread_t *th = GET_THREAD();
+    int call_status = rb_method_call_status(th, me, scope, self);
+
+    if (call_status != NOEX_OK) {
+	return method_missing(recv, mid, argc, argv, call_status);
+    }
+    stack_check();
+    return vm_call0(th, recv, mid, argc, argv, me);
+}
+
+struct rescue_funcall_args {
+    VALUE recv;
+    VALUE sym;
+    int argc;
+    VALUE *argv;
+};
+
+static VALUE
+check_funcall_exec(struct rescue_funcall_args *args)
+{
+    VALUE new_args = rb_ary_new4(args->argc, args->argv);
+
+    RB_GC_GUARD(new_args);
+    rb_ary_unshift(new_args, args->sym);
+    return rb_funcall2(args->recv, idMethodMissing,
+		       args->argc+1, RARRAY_PTR(new_args));
+}
+
+static VALUE
+check_funcall_failed(struct rescue_funcall_args *args, VALUE e)
+{
+    if (rb_respond_to(args->recv, SYM2ID(args->sym))) {
+	rb_exc_raise(e);
+    }
+    return Qundef;
+}
+
+static VALUE
+check_funcall(VALUE recv, ID mid, int argc, VALUE *argv)
+{
+    rb_method_entry_t *me = rb_search_method_entry(recv, mid);
+    rb_thread_t *th = GET_THREAD();
+    int call_status = rb_method_call_status(th, me, CALL_FCALL, Qundef);
+
+    if (call_status != NOEX_OK) {
+	if (rb_method_basic_definition_p(CLASS_OF(recv), idMethodMissing)) {
+	    return Qundef;
+	}
+	else {
+	    struct rescue_funcall_args args;
+
+	    th->method_missing_reason = 0;
+	    args.recv = recv;
+	    args.sym = ID2SYM(mid);
+	    args.argc = argc;
+	    args.argv = argv;
+	    return rb_rescue2(check_funcall_exec, (VALUE)&args,
+			      check_funcall_failed, (VALUE)&args,
+			      rb_eNoMethodError, (VALUE)0);
+	}
+    }
+    stack_check();
+    return vm_call0(th, recv, mid, argc, argv, me);
+}
+
+VALUE
+rb_check_funcall(VALUE recv, ID mid, int argc, VALUE *argv)
+{
+    return check_funcall(recv, mid, argc, argv);
+}
+
+static const char *
+rb_type_str(enum ruby_value_type type)
+{
+#define type_case(t) case t: return #t;
+    switch (type) {
+      type_case(T_NONE)
+      type_case(T_OBJECT)
+      type_case(T_CLASS)
+      type_case(T_MODULE)
+      type_case(T_FLOAT)
+      type_case(T_STRING)
+      type_case(T_REGEXP)
+      type_case(T_ARRAY)
+      type_case(T_HASH)
+      type_case(T_STRUCT)
+      type_case(T_BIGNUM)
+      type_case(T_FILE)
+      type_case(T_DATA)
+      type_case(T_MATCH)
+      type_case(T_COMPLEX)
+      type_case(T_RATIONAL)
+      type_case(T_NIL)
+      type_case(T_TRUE)
+      type_case(T_FALSE)
+      type_case(T_SYMBOL)
+      type_case(T_FIXNUM)
+      type_case(T_UNDEF)
+      type_case(T_NODE)
+      type_case(T_ICLASS)
+      type_case(T_ZOMBIE)
+      default: return NULL;
+    }
+#undef type_case
+}
+
+static inline rb_method_entry_t *
+rb_search_method_entry(VALUE recv, ID mid)
+{
+    VALUE klass = CLASS_OF(recv);
 
     if (!klass) {
-	rb_raise(rb_eNotImpError,
-		 "method `%s' called on terminated object (%p)",
-		 rb_id2name(mid), (void *)recv);
+        VALUE flags, klass;
+        if (IMMEDIATE_P(recv)) {
+            rb_raise(rb_eNotImpError,
+                     "method `%s' called on unexpected immediate object (%p)",
+                     rb_id2name(mid), (void *)recv);
+        }
+        flags = RBASIC(recv)->flags;
+        klass = RBASIC(recv)->klass;
+        if (flags == 0) {
+            rb_raise(rb_eNotImpError,
+                     "method `%s' called on terminated object"
+                     " (%p flags=0x%"PRIxVALUE" klass=0x%"PRIxVALUE")",
+                     rb_id2name(mid), (void *)recv, flags, klass);
+        }
+        else {
+            int type = BUILTIN_TYPE(recv);
+            const char *typestr = rb_type_str(type);
+            if (typestr && T_OBJECT <= type && type < T_NIL)
+                rb_raise(rb_eNotImpError,
+                         "method `%s' called on hidden %s object"
+                         " (%p flags=0x%"PRIxVALUE" klass=0x%"PRIxVALUE")",
+                         rb_id2name(mid), typestr, (void *)recv, flags, klass);
+            if (typestr)
+                rb_raise(rb_eNotImpError,
+                         "method `%s' called on unexpected %s object"
+                         " (%p flags=0x%"PRIxVALUE" klass=0x%"PRIxVALUE")",
+                         rb_id2name(mid), typestr, (void *)recv, flags, klass);
+            else
+                rb_raise(rb_eNotImpError,
+                         "method `%s' called on broken T_???" "(0x%02x) object"
+                         " (%p flags=0x%"PRIxVALUE" klass=0x%"PRIxVALUE")",
+                         rb_id2name(mid), type, (void *)recv, flags, klass);
+        }
     }
-    /* is it in the method cache? */
-    ent = cache + EXPR1(klass, mid);
+    return rb_method_entry(klass, mid);
+}
 
-    if (ent->mid == mid && ent->klass == klass) {
-	if (!ent->method)
-	    return method_missing(recv, mid, argc, argv,
-				  scope == 2 ? NOEX_VCALL : 0);
-	id = ent->mid0;
-	noex = ent->method->nd_noex;
-	klass = ent->method->nd_clss;
-	body = ent->method->nd_body;
-    }
-    else if ((method = rb_get_method_body(klass, id, &id)) != 0) {
-	noex = method->nd_noex;
-	klass = method->nd_clss;
-	body = method->nd_body;
-    }
-    else {
-	if (scope == 3) {
-	    return method_missing(recv, mid, argc, argv, NOEX_SUPER);
-	}
-	return method_missing(recv, mid, argc, argv,
-			      scope == 2 ? NOEX_VCALL : 0);
-    }
-    
+static inline int
+rb_method_call_status(rb_thread_t *th, rb_method_entry_t *me, call_type scope, VALUE self)
+{
+    VALUE klass;
+    ID oid;
+    int noex;
 
-    if (mid != idMethodMissing) {
+    if (UNDEFINED_METHOD_ENTRY_P(me)) {
+	return scope == CALL_VCALL ? NOEX_VCALL : 0;
+    }
+    klass = me->klass;
+    oid = me->def->original_id;
+    noex = me->flag;
+
+    if (oid != idMethodMissing) {
 	/* receiver specified form for private method */
 	if (UNLIKELY(noex)) {
-	    if (((noex & NOEX_MASK) & NOEX_PRIVATE) && scope == 0) {
-		return method_missing(recv, mid, argc, argv, NOEX_PRIVATE);
+	    if (((noex & NOEX_MASK) & NOEX_PRIVATE) && scope == CALL_PUBLIC) {
+		return NOEX_PRIVATE;
 	    }
 
 	    /* self must be kind of a specified form for protected method */
-	    if (((noex & NOEX_MASK) & NOEX_PROTECTED) && scope == 0) {
+	    if (((noex & NOEX_MASK) & NOEX_PROTECTED) && scope == CALL_PUBLIC) {
 		VALUE defined_class = klass;
-		
+
 		if (TYPE(defined_class) == T_ICLASS) {
 		    defined_class = RBASIC(defined_class)->klass;
 		}
@@ -234,25 +406,36 @@ rb_call0(VALUE klass, VALUE recv, ID mid, int argc, const VALUE *argv,
 		if (self == Qundef) {
 		    self = th->cfp->self;
 		}
-		if (!rb_obj_is_kind_of(self, rb_class_real(defined_class))) {
-		    return method_missing(recv, mid, argc, argv, NOEX_PROTECTED);
+		if (!rb_obj_is_kind_of(self, defined_class)) {
+		    return NOEX_PROTECTED;
 		}
 	    }
 
 	    if (NOEX_SAFE(noex) > th->safe_level) {
-		rb_raise(rb_eSecurityError, "calling insecure method: %s", rb_id2name(mid));
+		rb_raise(rb_eSecurityError, "calling insecure method: %s",
+			 rb_id2name(me->called_id));
 	    }
 	}
     }
-
-    stack_check();
-    return vm_call0(th, klass, recv, mid, id, argc, argv, body, noex & NOEX_NOSUPER);
+    return NOEX_OK;
 }
 
+
+/*!
+ * \internal
+ * calls the specified method.
+ *
+ * This function is called by functions in rb_call* family.
+ * \param recv   receiver
+ * \param mid    an ID that represents the name of the method
+ * \param argc   the number of method arguments
+ * \param argv   a pointer to an array of method arguments
+ * \param scope
+ */
 static inline VALUE
-rb_call(VALUE klass, VALUE recv, ID mid, int argc, const VALUE *argv, int scope)
+rb_call(VALUE recv, ID mid, int argc, const VALUE *argv, call_type scope)
 {
-    return rb_call0(klass, recv, mid, argc, argv, scope, Qundef);
+    return rb_call0(recv, mid, argc, argv, scope, Qundef);
 }
 
 NORETURN(static void raise_method_missing(rb_thread_t *th, int argc, const VALUE *argv,
@@ -260,7 +443,7 @@ NORETURN(static void raise_method_missing(rb_thread_t *th, int argc, const VALUE
 
 /*
  *  call-seq:
- *     obj.method_missing(symbol [, *args] )   => result
+ *     obj.method_missing(symbol [, *args] )   -> result
  *
  *  Invoked by Ruby when <i>obj</i> is sent a message it cannot handle.
  *  <i>symbol</i> is the symbol for the method called, and <i>args</i>
@@ -336,9 +519,16 @@ raise_method_missing(rb_thread_t *th, int argc, const VALUE *argv, VALUE obj,
 
     {
 	int n = 0;
+	VALUE mesg;
 	VALUE args[3];
-	args[n++] = rb_funcall(rb_const_get(exc, rb_intern("message")), '!',
-			       3, rb_str_new2(format), obj, argv[0]);
+
+	mesg = rb_const_get(exc, rb_intern("message"));
+	if (rb_method_basic_definition_p(CLASS_OF(mesg), '!')) {
+	    args[n++] = rb_name_err_mesg_new(mesg, rb_str_new2(format), obj, argv[0]);
+	}
+	else {
+	    args[n++] = rb_funcall(mesg, '!', 3, rb_str_new2(format), obj, argv[0]);
+	}
 	args[n++] = argv[0];
 	if (exc == rb_eNoMethodError) {
 	    args[n++] = rb_ary_new4(argc - 1, argv + 1);
@@ -379,6 +569,9 @@ method_missing(VALUE obj, ID id, int argc, const VALUE *argv, int call_status)
     nargv[0] = ID2SYM(id);
     MEMCPY(nargv + 1, argv, VALUE, argc);
 
+    if (rb_method_basic_definition_p(CLASS_OF(obj) , idMethodMissing)) {
+	raise_method_missing(th, argc+1, nargv, obj, call_status | NOEX_MISSING);
+    }
     result = rb_funcall2(obj, idMethodMissing, argc + 1, nargv);
     if (argv_ary) rb_ary_clear(argv_ary);
     return result;
@@ -392,18 +585,35 @@ rb_raise_method_missing(rb_thread_t *th, int argc, VALUE *argv,
     raise_method_missing(th, argc, argv, obj, call_status | NOEX_MISSING);
 }
 
+/*!
+ * Calls a method
+ * \param recv   receiver of the method
+ * \param mid    an ID that represents the name of the method
+ * \param args   an Array object which contains method arguments
+ *
+ * \pre \a args must refer an Array object.
+ */
 VALUE
 rb_apply(VALUE recv, ID mid, VALUE args)
 {
     int argc;
     VALUE *argv;
 
-    argc = RARRAY_LEN(args);	/* Assigns LONG, but argc is INT */
+    argc = RARRAY_LENINT(args);
     argv = ALLOCA_N(VALUE, argc);
     MEMCPY(argv, RARRAY_PTR(args), VALUE, argc);
-    return rb_call(CLASS_OF(recv), recv, mid, argc, argv, CALL_FCALL);
+    return rb_call(recv, mid, argc, argv, CALL_FCALL);
 }
 
+/*!
+ * Calls a method
+ * \param recv   receiver of the method
+ * \param mid    an ID that represents the name of the method
+ * \param n      the number of arguments
+ * \param ...    arbitrary number of method arguments
+ *
+ * \pre each of arguments after \a n must be a VALUE.
+ */
 VALUE
 rb_funcall(VALUE recv, ID mid, int n, ...)
 {
@@ -424,23 +634,39 @@ rb_funcall(VALUE recv, ID mid, int n, ...)
     else {
 	argv = 0;
     }
-    return rb_call(CLASS_OF(recv), recv, mid, n, argv, CALL_FCALL);
+    return rb_call(recv, mid, n, argv, CALL_FCALL);
 }
 
+/*!
+ * Calls a method
+ * \param recv   receiver of the method
+ * \param mid    an ID that represents the name of the method
+ * \param argc   the number of arguments
+ * \param argv   pointer to an array of method arguments
+ */
 VALUE
 rb_funcall2(VALUE recv, ID mid, int argc, const VALUE *argv)
 {
-    return rb_call(CLASS_OF(recv), recv, mid, argc, argv, CALL_FCALL);
+    return rb_call(recv, mid, argc, argv, CALL_FCALL);
 }
 
+/*!
+ * Calls a method.
+ *
+ * Same as rb_funcall2 but this function can call only public methods.
+ * \param recv   receiver of the method
+ * \param mid    an ID that represents the name of the method
+ * \param argc   the number of arguments
+ * \param argv   pointer to an array of method arguments
+ */
 VALUE
 rb_funcall3(VALUE recv, ID mid, int argc, const VALUE *argv)
 {
-    return rb_call(CLASS_OF(recv), recv, mid, argc, argv, CALL_PUBLIC);
+    return rb_call(recv, mid, argc, argv, CALL_PUBLIC);
 }
 
 static VALUE
-send_internal(int argc, VALUE *argv, VALUE recv, int scope)
+send_internal(int argc, const VALUE *argv, VALUE recv, call_type scope)
 {
     VALUE vid;
     VALUE self = RUBY_VM_PREVIOUS_CONTROL_FRAME(GET_THREAD()->cfp)->self;
@@ -453,13 +679,13 @@ send_internal(int argc, VALUE *argv, VALUE recv, int scope)
     vid = *argv++; argc--;
     PASS_PASSED_BLOCK_TH(th);
 
-    return rb_call0(CLASS_OF(recv), recv, rb_to_id(vid), argc, argv, scope, self);
+    return rb_call0(recv, rb_to_id(vid), argc, argv, scope, self);
 }
 
 /*
  *  call-seq:
- *     obj.send(symbol [, args...])        => obj
- *     obj.__send__(symbol [, args...])      => obj
+ *     obj.send(symbol [, args...])        -> obj
+ *     obj.__send__(symbol [, args...])      -> obj
  *
  *  Invokes the method identified by _symbol_, passing it any
  *  arguments specified. You can use <code>__send__</code> if the name
@@ -477,12 +703,12 @@ send_internal(int argc, VALUE *argv, VALUE recv, int scope)
 VALUE
 rb_f_send(int argc, VALUE *argv, VALUE recv)
 {
-    return send_internal(argc, argv, recv, NOEX_NOSUPER | NOEX_PRIVATE);
+    return send_internal(argc, argv, recv, CALL_FCALL);
 }
 
 /*
  *  call-seq:
- *     obj.public_send(symbol [, args...])  => obj
+ *     obj.public_send(symbol [, args...])  -> obj
  *
  *  Invokes the method identified by _symbol_, passing it any
  *  arguments specified. Unlike send, public_send calls public
@@ -494,7 +720,7 @@ rb_f_send(int argc, VALUE *argv, VALUE recv)
 VALUE
 rb_f_public_send(int argc, VALUE *argv, VALUE recv)
 {
-    return send_internal(argc, argv, recv, NOEX_PUBLIC);
+    return send_internal(argc, argv, recv, CALL_PUBLIC);
 }
 
 /* yield */
@@ -552,7 +778,7 @@ rb_yield_splat(VALUE values)
     if (NIL_P(tmp)) {
         rb_raise(rb_eArgError, "not an array");
     }
-    v = rb_yield_0(RARRAY_LEN(tmp), RARRAY_PTR(tmp));
+    v = rb_yield_0(RARRAY_LENINT(tmp), RARRAY_PTR(tmp));
     return v;
 }
 
@@ -567,10 +793,13 @@ loop_i(void)
 
 /*
  *  call-seq:
- *     loop {|| block }
+ *     loop { block }
+ *     loop            -> an_enumerator
  *
  *  Repeatedly executes the block.
  *
+ *  If no block is given, an enumerator is returned instead.
+ *
  *     loop do
  *       print "Input: "
  *       line = gets
@@ -582,12 +811,18 @@ loop_i(void)
  */
 
 static VALUE
-rb_f_loop(void)
+rb_f_loop(VALUE self)
 {
+    RETURN_ENUMERATOR(self, 0, 0);
     rb_rescue2(loop_i, (VALUE)0, 0, 0, rb_eStopIteration, (VALUE)0);
     return Qnil;		/* dummy */
 }
 
+#if VMDEBUG
+static const char *
+vm_frametype_name(const rb_control_frame_t *cfp);
+#endif
+
 VALUE
 rb_iterate(VALUE (* it_proc) (VALUE), VALUE data1,
 	   VALUE (* bl_proc) (ANYARGS), VALUE data2)
@@ -596,16 +831,22 @@ rb_iterate(VALUE (* it_proc) (VALUE), VALUE data1,
     volatile VALUE retval = Qnil;
     NODE *node = NEW_IFUNC(bl_proc, data2);
     rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *cfp = th->cfp;
+    rb_control_frame_t *volatile cfp = th->cfp;
 
     TH_PUSH_TAG(th);
     state = TH_EXEC_TAG();
     if (state == 0) {
       iter_retry:
 	{
-	    rb_block_t *blockptr = RUBY_VM_GET_BLOCK_PTR_IN_CFP(th->cfp);
-	    blockptr->iseq = (void *)node;
-	    blockptr->proc = 0;
+	    rb_block_t *blockptr;
+	    if (bl_proc) {
+		blockptr = RUBY_VM_GET_BLOCK_PTR_IN_CFP(th->cfp);
+		blockptr->iseq = (void *)node;
+		blockptr->proc = 0;
+	    }
+	    else {
+		blockptr = GC_GUARDED_PTR_REF(th->cfp->lfp[0]);
+	    }
 	    th->passed_block = blockptr;
 	}
 	retval = (*it_proc) (data1);
@@ -620,7 +861,19 @@ rb_iterate(VALUE (* it_proc) (VALUE), VALUE data1,
 		state = 0;
 		th->state = 0;
 		th->errinfo = Qnil;
-		th->cfp = cfp;
+
+		/* check skipped frame */
+		while (th->cfp != cfp) {
+#if VMDEBUG
+		    printf("skipped frame: %s\n", vm_frametype_name(th->cfp));
+#endif
+		    if (UNLIKELY(VM_FRAME_TYPE(th->cfp) == VM_FRAME_MAGIC_CFUNC)) {
+			const rb_method_entry_t *me = th->cfp->me;
+			EXEC_EVENT_HOOK(th, RUBY_EVENT_C_RETURN, th->cfp->self, me->called_id, me->klass);
+		    }
+
+		    th->cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(th->cfp);
+		}
 	    }
 	    else{
 		/* SDR(); printf("%p, %p\n", cdfp, escape_dfp); */
@@ -663,8 +916,7 @@ iterate_method(VALUE obj)
     const struct iter_method_arg * arg =
       (struct iter_method_arg *) obj;
 
-    return rb_call(CLASS_OF(arg->obj), arg->obj, arg->mid,
-		   arg->argc, arg->argv, CALL_FCALL);
+    return rb_call(arg->obj, arg->mid, arg->argc, arg->argv, CALL_FCALL);
 }
 
 VALUE
@@ -683,11 +935,11 @@ rb_block_call(VALUE obj, ID mid, int argc, VALUE * argv,
 VALUE
 rb_each(VALUE obj)
 {
-    return rb_call(CLASS_OF(obj), obj, idEach, 0, 0, CALL_FCALL);
+    return rb_call(obj, idEach, 0, 0, CALL_FCALL);
 }
 
 static VALUE
-eval_string_with_cref(VALUE self, VALUE src, VALUE scope, NODE *cref, const char *file, int line)
+eval_string_with_cref(VALUE self, VALUE src, VALUE scope, NODE *cref, const char *volatile file, volatile int line)
 {
     int state;
     VALUE result = Qundef;
@@ -715,6 +967,10 @@ eval_string_with_cref(VALUE self, VALUE src, VALUE scope, NODE *cref, const char
 	    if (rb_obj_is_kind_of(scope, rb_cBinding)) {
 		GetBindingPtr(scope, bind);
 		envval = bind->env;
+		if (strcmp(file, "(eval)") == 0 && bind->filename != Qnil) {
+		    file = RSTRING_PTR(bind->filename);
+		    line = bind->line_no;
+		}
 	    }
 	    else {
 		rb_raise(rb_eTypeError,
@@ -754,7 +1010,7 @@ eval_string_with_cref(VALUE self, VALUE src, VALUE scope, NODE *cref, const char
 
 	/* save new env */
 	GetISeqPtr(iseqval, iseq);
-	if (bind && iseq->local_size > 0) {
+	if (bind && iseq->local_table_size > 0) {
 	    bind->env = rb_vm_make_env_object(th, th->cfp);
 	}
 
@@ -807,14 +1063,13 @@ eval_string(VALUE self, VALUE src, VALUE scope, const char *file, int line)
 
 /*
  *  call-seq:
- *     eval(string [, binding [, filename [,lineno]]])  => obj
+ *     eval(string [, binding [, filename [,lineno]]])  -> obj
  *
  *  Evaluates the Ruby expression(s) in <em>string</em>. If
- *  <em>binding</em> is given, the evaluation is performed in its
- *  context. The binding may be a <code>Binding</code> object or a
- *  <code>Proc</code> object. If the optional <em>filename</em> and
- *  <em>lineno</em> parameters are present, they will be used when
- *  reporting syntax errors.
+ *  <em>binding</em> is given, which must be a <code>Binding</code>
+ *  object, the evaluation is performed in its context. If the
+ *  optional <em>filename</em> and <em>lineno</em> parameters are
+ *  present, they will be used when reporting syntax errors.
  *
  *     def getBinding(str)
  *       return binding
@@ -908,7 +1163,7 @@ rb_eval_cmd(VALUE cmd, VALUE arg, int level)
 	PUSH_TAG();
 	rb_set_safe_level_force(level);
 	if ((state = EXEC_TAG()) == 0) {
-	    val = rb_funcall2(cmd, rb_intern("call"), RARRAY_LEN(arg),
+	    val = rb_funcall2(cmd, rb_intern("call"), RARRAY_LENINT(arg),
 			      RARRAY_PTR(arg));
 	}
 	POP_TAG();
@@ -938,19 +1193,21 @@ yield_under(VALUE under, VALUE self, VALUE values)
 {
     rb_thread_t *th = GET_THREAD();
     rb_block_t block, *blockptr;
-    NODE *cref = vm_cref_push(th, under, NOEX_PUBLIC);
+    NODE *cref;
 
     if ((blockptr = GC_GUARDED_PTR_REF(th->cfp->lfp[0])) != 0) {
 	block = *blockptr;
 	block.self = self;
 	th->cfp->lfp[0] = GC_GUARDED_PTR(&block);
     }
+    cref = vm_cref_push(th, under, NOEX_PUBLIC, blockptr);
+    cref->flags |= NODE_FL_CREF_PUSHED_BY_EVAL;
 
     if (values == Qundef) {
-	return vm_yield_with_cref(th, 0, 0, cref);
+	return vm_yield_with_cref(th, 1, &self, cref);
     }
     else {
-	return vm_yield_with_cref(th, RARRAY_LEN(values), RARRAY_PTR(values), cref);
+	return vm_yield_with_cref(th, RARRAY_LENINT(values), RARRAY_PTR(values), cref);
     }
 }
 
@@ -958,7 +1215,7 @@ yield_under(VALUE under, VALUE self, VALUE values)
 static VALUE
 eval_under(VALUE under, VALUE self, VALUE src, const char *file, int line)
 {
-    NODE *cref = vm_cref_push(GET_THREAD(), under, NOEX_PUBLIC);
+    NODE *cref = vm_cref_push(GET_THREAD(), under, NOEX_PUBLIC, NULL);
 
     if (rb_safe_level() >= 4) {
 	StringValue(src);
@@ -1011,8 +1268,8 @@ specific_eval(int argc, VALUE *argv, VALUE klass, VALUE self)
 
 /*
  *  call-seq:
- *     obj.instance_eval(string [, filename [, lineno]] )   => obj
- *     obj.instance_eval {| | block }                       => obj
+ *     obj.instance_eval(string [, filename [, lineno]] )   -> obj
+ *     obj.instance_eval {| | block }                       -> obj
  *
  *  Evaluates a string containing Ruby source code, or the given block,
  *  within the context of the receiver (_obj_). In order to set the
@@ -1048,7 +1305,7 @@ rb_obj_instance_eval(int argc, VALUE *argv, VALUE self)
 
 /*
  *  call-seq:
- *     obj.instance_exec(arg...) {|var...| block }                       => obj
+ *     obj.instance_exec(arg...) {|var...| block }                       -> obj
  *
  *  Executes the given block within the context of the receiver
  *  (_obj_). In order to set the context, the variable +self+ is set
@@ -1080,8 +1337,8 @@ rb_obj_instance_exec(int argc, VALUE *argv, VALUE self)
 
 /*
  *  call-seq:
- *     mod.class_eval(string [, filename [, lineno]])  => obj
- *     mod.module_eval {|| block }                     => obj
+ *     mod.class_eval(string [, filename [, lineno]])  -> obj
+ *     mod.module_eval {|| block }                     -> obj
  *
  *  Evaluates the string or block in the context of _mod_. This can
  *  be used to add methods to a class. <code>module_eval</code> returns
@@ -1110,8 +1367,8 @@ rb_mod_module_eval(int argc, VALUE *argv, VALUE mod)
 
 /*
  *  call-seq:
- *     mod.module_exec(arg...) {|var...| block }       => obj
- *     mod.class_exec(arg...) {|var...| block }        => obj
+ *     mod.module_exec(arg...) {|var...| block }       -> obj
+ *     mod.class_exec(arg...) {|var...| block }        -> obj
  *
  *  Evaluates the given block in the context of the class/module.
  *  The method defined in the block will belong to the receiver.
@@ -1134,15 +1391,13 @@ rb_mod_module_exec(int argc, VALUE *argv, VALUE mod)
     return yield_under(mod, mod, rb_ary_new4(argc, argv));
 }
 
-NORETURN(static VALUE rb_f_throw _((int, VALUE *)));
-
 /*
  *  call-seq:
- *     throw(symbol [, obj])
+ *     throw(tag [, obj])
  *
  *  Transfers control to the end of the active +catch+ block
- *  waiting for _symbol_. Raises +NameError+ if there
- *  is no +catch+ block for the symbol. The optional second
+ *  waiting for _tag_. Raises +ArgumentError+ if there
+ *  is no +catch+ block for the _tag_. The optional second
  *  parameter supplies a return value for the +catch+ block,
  *  which otherwise defaults to +nil+. For examples, see
  *  <code>Kernel::catch</code>.
@@ -1152,10 +1407,18 @@ static VALUE
 rb_f_throw(int argc, VALUE *argv)
 {
     VALUE tag, value;
+
+    rb_scan_args(argc, argv, "11", &tag, &value);
+    rb_throw_obj(tag, value);
+    return Qnil;		/* not reached */
+}
+
+void
+rb_throw_obj(VALUE tag, VALUE value)
+{
     rb_thread_t *th = GET_THREAD();
     struct rb_vm_tag *tt = th->tag;
 
-    rb_scan_args(argc, argv, "11", &tag, &value);
     while (tt) {
 	if (tt->tag == tag) {
 	    tt->retval = value;
@@ -1171,39 +1434,28 @@ rb_f_throw(int argc, VALUE *argv)
     th->errinfo = NEW_THROW_OBJECT(tag, 0, TAG_THROW);
 
     JUMP_TAG(TAG_THROW);
-#ifndef __GNUC__
-    return Qnil;		/* not reached */
-#endif
 }
 
 void
 rb_throw(const char *tag, VALUE val)
 {
-    VALUE argv[2];
-
-    argv[0] = ID2SYM(rb_intern(tag));
-    argv[1] = val;
-    rb_f_throw(2, argv);
+    rb_throw_obj(ID2SYM(rb_intern(tag)), val);
 }
 
-void
-rb_throw_obj(VALUE tag, VALUE val)
+static VALUE
+catch_i(VALUE tag, VALUE data)
 {
-    VALUE argv[2];
-
-    argv[0] = tag;
-    argv[1] = val;
-    rb_f_throw(2, argv);
+    return rb_yield_0(1, &tag);
 }
 
 /*
  *  call-seq:
- *     catch(symbol) {| | block }  > obj
+ *     catch([arg]) {|tag| block }  -> obj
  *
  *  +catch+ executes its block. If a +throw+ is
  *  executed, Ruby searches up its stack for a +catch+ block
  *  with a tag corresponding to the +throw+'s
- *  _symbol_. If found, that block is terminated, and
+ *  _tag_. If found, that block is terminated, and
  *  +catch+ returns the value given to +throw+. If
  *  +throw+ is not called, the block terminates normally, and
  *  the value of +catch+ is the value of the last expression
@@ -1225,16 +1477,18 @@ rb_throw_obj(VALUE tag, VALUE val)
  *     2
  *     1
  *     0
+ *
+ *  when _arg_ is given, +catch+ yields it as is, or when no
+ *  _arg_ is given, +catch+ assigns a new unique object to
+ *  +throw+.  this is useful for nested +catch+.  _arg_ can
+ *  be an arbitrary object, not only Symbol.
+ *
  */
 
 static VALUE
 rb_f_catch(int argc, VALUE *argv)
 {
     VALUE tag;
-    int state;
-    VALUE val = Qnil;		/* OK */
-    rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *saved_cfp = th->cfp;
 
     if (argc == 0) {
 	tag = rb_obj_alloc(rb_cObject);
@@ -1242,12 +1496,31 @@ rb_f_catch(int argc, VALUE *argv)
     else {
 	rb_scan_args(argc, argv, "01", &tag);
     }
+    return rb_catch_obj(tag, catch_i, 0);
+}
+
+VALUE
+rb_catch(const char *tag, VALUE (*func)(), VALUE data)
+{
+    VALUE vtag = tag ? ID2SYM(rb_intern(tag)) : rb_obj_alloc(rb_cObject);
+    return rb_catch_obj(vtag, func, data);
+}
+
+VALUE
+rb_catch_obj(VALUE tag, VALUE (*func)(), VALUE data)
+{
+    int state;
+    volatile VALUE val = Qnil;		/* OK */
+    rb_thread_t *th = GET_THREAD();
+    rb_control_frame_t *saved_cfp = th->cfp;
+
     PUSH_TAG();
 
     th->tag->tag = tag;
 
     if ((state = EXEC_TAG()) == 0) {
-	val = rb_yield_0(1, &tag);
+	/* call with argc=1, argv = [tag], block = Qnil to insure compatibility */
+	val = (*func)(tag, data, 1, &tag, Qnil);
     }
     else if (state == TAG_THROW && RNODE(th->errinfo)->u1.value == tag) {
 	th->cfp = saved_cfp;
@@ -1262,36 +1535,9 @@ rb_f_catch(int argc, VALUE *argv)
     return val;
 }
 
-static VALUE
-catch_null_i(VALUE dmy)
-{
-    return rb_funcall(Qnil, rb_intern("catch"), 0, 0);
-}
-
-static VALUE
-catch_i(VALUE tag)
-{
-    return rb_funcall(Qnil, rb_intern("catch"), 1, tag);
-}
-
-VALUE
-rb_catch(const char *tag, VALUE (*func)(), VALUE data)
-{
-    if (!tag) {
-	return rb_iterate(catch_null_i, 0, func, data);
-    }
-    return rb_iterate(catch_i, ID2SYM(rb_intern(tag)), func, data);
-}
-
-VALUE
-rb_catch_obj(VALUE tag, VALUE (*func)(), VALUE data)
-{
-    return rb_iterate((VALUE (*)_((VALUE)))catch_i, tag, func, data);
-}
-
 /*
  *  call-seq:
- *     caller(start=1)    => array
+ *     caller(start=1)    -> array or nil
  *
  *  Returns the current execution stack---an array containing strings in
  *  the form ``<em>file:line</em>'' or ``<em>file:line: in
@@ -1299,6 +1545,9 @@ rb_catch_obj(VALUE tag, VALUE (*func)(), VALUE data)
  *  determines the number of initial stack entries to omit from the
  *  result.
  *
+ *  Returns +nil+ if _start_ is greater than the size of
+ *  current execution stack.
+ *
  *     def a(skip)
  *       caller(skip)
  *     end
@@ -1308,10 +1557,12 @@ rb_catch_obj(VALUE tag, VALUE (*func)(), VALUE data)
  *     def c(skip)
  *       b(skip)
  *     end
- *     c(0)   #=> ["prog:2:in `a'", "prog:5:in `b'", "prog:8:in `c'", "prog:10"]
- *     c(1)   #=> ["prog:5:in `b'", "prog:8:in `c'", "prog:11"]
- *     c(2)   #=> ["prog:8:in `c'", "prog:12"]
- *     c(3)   #=> ["prog:13"]
+ *     c(0)   #=> ["prog:2:in `a'", "prog:5:in `b'", "prog:8:in `c'", "prog:10:in `<main>'"]
+ *     c(1)   #=> ["prog:5:in `b'", "prog:8:in `c'", "prog:11:in `<main>'"]
+ *     c(2)   #=> ["prog:8:in `c'", "prog:12:in `<main>'"]
+ *     c(3)   #=> ["prog:13:in `<main>'"]
+ *     c(4)   #=> []
+ *     c(5)   #=> nil
  */
 
 static VALUE
@@ -1332,16 +1583,26 @@ rb_f_caller(int argc, VALUE *argv)
     return vm_backtrace(GET_THREAD(), lev);
 }
 
+static int
+print_backtrace(void *arg, VALUE file, int line, VALUE method)
+{
+    FILE *fp = arg;
+    const char *filename = NIL_P(file) ? "ruby" : RSTRING_PTR(file);
+    if (NIL_P(method)) {
+	fprintf(fp, "\tfrom %s:%d:in unknown method\n",
+		filename, line);
+    }
+    else {
+	fprintf(fp, "\tfrom %s:%d:in `%s'\n",
+		filename, line, RSTRING_PTR(method));
+    }
+    return FALSE;
+}
+
 void
 rb_backtrace(void)
 {
-    long i;
-    VALUE ary;
-
-    ary = vm_backtrace(GET_THREAD(), -1);
-    for (i = 0; i < RARRAY_LEN(ary); i++) {
-	printf("\tfrom %s\n", RSTRING_PTR(RARRAY_PTR(ary)[i]));
-    }
+    vm_backtrace_each(GET_THREAD(), -1, NULL, print_backtrace, stderr);
 }
 
 VALUE
@@ -1350,9 +1611,34 @@ rb_make_backtrace(void)
     return vm_backtrace(GET_THREAD(), -1);
 }
 
+VALUE
+rb_thread_backtrace(VALUE thval)
+{
+    rb_thread_t *th;
+    GetThreadPtr(thval, th);
+
+    switch (th->status) {
+      case THREAD_RUNNABLE:
+      case THREAD_STOPPED:
+      case THREAD_STOPPED_FOREVER:
+	break;
+      case THREAD_TO_KILL:
+      case THREAD_KILLED:
+	return Qnil;
+    }
+
+    return vm_backtrace(th, 0);
+}
+
+int
+rb_backtrace_each(rb_backtrace_iter_func *iter, void *arg)
+{
+    return vm_backtrace_each(GET_THREAD(), -1, NULL, iter, arg);
+}
+
 /*
  *  call-seq:
- *     local_variables    => array
+ *     local_variables    -> array
  *
  *  Returns the names of the current local variables.
  *
@@ -1360,7 +1646,7 @@ rb_make_backtrace(void)
  *     for i in 1..10
  *        # ...
  *     end
- *     local_variables   #=> ["fred", "i"]
+ *     local_variables   #=> [:fred, :i]
  */
 
 static VALUE
@@ -1407,8 +1693,8 @@ rb_f_local_variables(void)
 
 /*
  *  call-seq:
- *     block_given?   => true or false
- *     iterator?      => true or false
+ *     block_given?   -> true or false
+ *     iterator?      -> true or false
  *
  *  Returns <code>true</code> if <code>yield</code> would execute a
  *  block in the current context. The <code>iterator?</code> form
@@ -1444,6 +1730,16 @@ rb_f_block_given_p(void)
     }
 }
 
+VALUE
+rb_current_realfilepath(void)
+{
+    rb_thread_t *th = GET_THREAD();
+    rb_control_frame_t *cfp = th->cfp;
+    cfp = vm_get_ruby_level_caller_cfp(th, RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp));
+    if (cfp != 0) return cfp->iseq->filepath;
+    return Qnil;
+}
+
 void
 Init_vm_eval(void)
 {
@@ -1461,8 +1757,15 @@ Init_vm_eval(void)
     rb_define_method(rb_cBasicObject, "instance_exec", rb_obj_instance_exec, -1);
     rb_define_private_method(rb_cBasicObject, "method_missing", rb_method_missing, -1);
 
+#if 1
+    rb_add_method(rb_cBasicObject, rb_intern("__send__"),
+		  VM_METHOD_TYPE_OPTIMIZED, (void *)OPTIMIZED_METHOD_TYPE_SEND, 0);
+    rb_add_method(rb_mKernel, rb_intern("send"),
+		  VM_METHOD_TYPE_OPTIMIZED, (void *)OPTIMIZED_METHOD_TYPE_SEND, 0);
+#else
     rb_define_method(rb_cBasicObject, "__send__", rb_f_send, -1);
     rb_define_method(rb_mKernel, "send", rb_f_send, -1);
+#endif
     rb_define_method(rb_mKernel, "public_send", rb_f_public_send, -1);
 
     rb_define_method(rb_cModule, "module_exec", rb_mod_module_exec, -1);
