@@ -2,7 +2,7 @@
 
   vm_insnhelper.c - instruction helper functions.
 
-  $Author: yugui $
+  $Author: nobu $
 
   Copyright (C) 2007 Koichi Sasada
 
@@ -26,11 +26,15 @@ vm_push_frame(rb_thread_t * th, const rb_iseq_t * iseq,
 	      const VALUE *pc, VALUE *sp, VALUE *lfp,
 	      int local_size)
 {
-    rb_control_frame_t * const cfp = th->cfp = th->cfp - 1;
+    rb_control_frame_t * const cfp = th->cfp - 1;
     int i;
 
+    if ((void *)(sp + local_size) >= (void *)cfp) {
+	rb_exc_raise(sysstack_error);
+    }
+    th->cfp = cfp;
     /* setup vm value stack */
-    
+
     /* nil initialize */
     for (i=0; i < local_size; i++) {
 	*sp = Qnil;
@@ -55,6 +59,7 @@ vm_push_frame(rb_thread_t * th, const rb_iseq_t * iseq,
     cfp->lfp = lfp;
     cfp->dfp = sp;
     cfp->proc = 0;
+    cfp->me = 0;
 
 #define COLLECT_PROFILE 0
 #if COLLECT_PROFILE
@@ -98,11 +103,37 @@ vm_pop_frame(rb_thread_t *th)
 
 /* method dispatch */
 
+NORETURN(static void argument_error(const rb_iseq_t *iseq, int miss_argc, int correct_argc));
+static void
+argument_error(const rb_iseq_t *iseq, int miss_argc, int correct_argc)
+{
+    VALUE mesg = rb_sprintf("wrong number of arguments (%d for %d)", miss_argc, correct_argc);
+    VALUE exc = rb_exc_new3(rb_eArgError, mesg);
+    VALUE bt = rb_make_backtrace();
+    VALUE err_line = 0;
+
+    if (iseq) {
+	int line_no = 1;
+
+	if (iseq->insn_info_size) {
+	    line_no = iseq->insn_info_table[0].line_no;
+	}
+
+	err_line = rb_sprintf("%s:%d:in `%s'",
+			      RSTRING_PTR(iseq->filename),
+			      line_no, RSTRING_PTR(iseq->name));
+	rb_funcall(bt, rb_intern("unshift"), 1, err_line);
+    }
+
+    rb_funcall(exc, rb_intern("set_backtrace"), 1, bt);
+    rb_exc_raise(exc);
+}
+
 #define VM_CALLEE_SETUP_ARG(ret, th, iseq, orig_argc, orig_argv, block) \
     if (LIKELY(iseq->arg_simple & 0x01)) { \
 	/* simple check */ \
 	if (orig_argc != iseq->argc) { \
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)", orig_argc, iseq->argc); \
+	    argument_error(iseq, orig_argc, iseq->argc); \
 	} \
 	ret = 0; \
     } \
@@ -118,14 +149,13 @@ vm_callee_setup_arg_complex(rb_thread_t *th, const rb_iseq_t * iseq,
     const int m = iseq->argc;
     int argc = orig_argc;
     VALUE *argv = orig_argv;
-    int opt_pc = 0;
+    rb_num_t opt_pc = 0;
 
     th->mark_stack_len = argc + iseq->arg_size;
 
     /* mandatory */
     if (argc < (m + iseq->arg_post_len)) { /* check with post arg */
-	rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)",
-		 argc, m + iseq->arg_post_len);
+	argument_error(iseq, argc, m + iseq->arg_post_len);
     }
 
     argv += m;
@@ -148,8 +178,7 @@ vm_callee_setup_arg_complex(rb_thread_t *th, const rb_iseq_t * iseq,
 	const int opts = iseq->arg_opts - 1 /* no opt */;
 
 	if (iseq->arg_rest == -1 && argc > opts) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)",
-		     orig_argc, m + opts + iseq->arg_post_len);
+	    argument_error(iseq, orig_argc, m + opts + iseq->arg_post_len);
 	}
 
 	if (argc > opts) {
@@ -179,8 +208,7 @@ vm_callee_setup_arg_complex(rb_thread_t *th, const rb_iseq_t * iseq,
 	const rb_block_t *blockptr = *block;
 
 	if (argc != 0) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)",
-		     orig_argc, m + iseq->arg_post_len);
+	    argument_error(iseq, orig_argc, m + iseq->arg_post_len);
 	}
 
 	if (blockptr) {
@@ -200,7 +228,7 @@ vm_callee_setup_arg_complex(rb_thread_t *th, const rb_iseq_t * iseq,
     }
 
     th->mark_stack_len = 0;
-    return opt_pc;
+    return (int)opt_pc;
 }
 
 static inline int
@@ -251,7 +279,7 @@ caller_setup_args(const rb_thread_t *th, rb_control_frame_t *cfp, VALUE flag,
 	    /* do nothing */
 	}
 	else {
-	    int len = RARRAY_LEN(tmp);
+	    long len = RARRAY_LEN(tmp);
 	    ptr = RARRAY_PTR(tmp);
 	    cfp->sp -= 1;
 
@@ -350,86 +378,84 @@ call_cfunc(VALUE (*func)(), VALUE recv,
 	break;
       default:
 	rb_raise(rb_eArgError, "too many arguments(%d)", len);
-	break;
+	return Qundef; /* not reached */
     }
-    return Qnil;		/* not reached */
 }
 
 static inline VALUE
 vm_call_cfunc(rb_thread_t *th, rb_control_frame_t *reg_cfp,
-	      int num, ID id, ID oid, VALUE recv, VALUE klass,
-	      VALUE flag, const NODE *mn, const rb_block_t *blockptr)
+	      int num, VALUE recv, const rb_block_t *blockptr,
+	      const rb_method_entry_t *me)
 {
-    VALUE val;
+    VALUE val = 0;
+    const rb_method_definition_t *def = me->def;
+    rb_control_frame_t *cfp;
 
-    EXEC_EVENT_HOOK(th, RUBY_EVENT_C_CALL, recv, id, klass);
-    {
-	rb_control_frame_t *cfp =
-	    vm_push_frame(th, 0, VM_FRAME_MAGIC_CFUNC,
-			  recv, (VALUE) blockptr, 0, reg_cfp->sp, 0, 1);
+    EXEC_EVENT_HOOK(th, RUBY_EVENT_C_CALL, recv, me->called_id, me->klass);
 
-	cfp->method_id = oid;
-	cfp->method_class = klass;
+    cfp = vm_push_frame(th, 0, VM_FRAME_MAGIC_CFUNC,
+			recv, (VALUE) blockptr, 0, reg_cfp->sp, 0, 1);
+    cfp->me = me;
+    reg_cfp->sp -= num + 1;
 
-	reg_cfp->sp -= num + 1;
+    val = call_cfunc(def->body.cfunc.func, recv, (int)def->body.cfunc.argc, num, reg_cfp->sp + 1);
 
-	val = call_cfunc(mn->nd_cfnc, recv, mn->nd_argc, num, reg_cfp->sp + 1);
-
-	if (reg_cfp != th->cfp + 1) {
-	    rb_bug("cfp consistency error - send");
-	}
-
-	vm_pop_frame(th);
+    if (reg_cfp != th->cfp + 1) {
+	rb_bug("cfp consistency error - send");
     }
-    EXEC_EVENT_HOOK(th, RUBY_EVENT_C_RETURN, recv, id, klass);
+
+    vm_pop_frame(th);
+
+    EXEC_EVENT_HOOK(th, RUBY_EVENT_C_RETURN, recv, me->called_id, me->klass);
 
     return val;
 }
 
 static inline VALUE
-vm_call_bmethod(rb_thread_t *th, ID id, VALUE procval, VALUE recv,
-		VALUE klass, int argc, VALUE *argv, rb_block_t *blockptr)
+vm_call_bmethod(rb_thread_t *th, VALUE recv, int argc, const VALUE *argv,
+		const rb_block_t *blockptr, const rb_method_entry_t *me)
 {
-    rb_control_frame_t *cfp = th->cfp;
     rb_proc_t *proc;
     VALUE val;
 
     /* control block frame */
-    (cfp-2)->method_id = id;
-    (cfp-2)->method_class = klass;
+    th->passed_me = me;
 
-    GetProcPtr(procval, proc);
+    GetProcPtr(me->def->body.proc, proc);
     val = rb_vm_invoke_proc(th, proc, recv, argc, argv, blockptr);
     return val;
 }
 
-static inline VALUE
-vm_method_missing(rb_thread_t *th, ID id, VALUE recv,
-		  int num, rb_block_t *blockptr, int opt)
+static inline void
+vm_method_missing_args(rb_thread_t *th, VALUE *argv,
+		       int num, const rb_block_t *blockptr, int opt)
 {
-    VALUE val;
     rb_control_frame_t * const reg_cfp = th->cfp;
-    VALUE *argv = ALLOCA_N(VALUE, num + 1);
     MEMCPY(argv, STACK_ADDR_FROM_TOP(num + 1), VALUE, num + 1);
-    argv[0] = ID2SYM(id);
     th->method_missing_reason = opt;
     th->passed_block = blockptr;
     POPN(num + 1);
-    val = rb_funcall2(recv, idMethodMissing, num + 1, argv);
-    return val;
+}
+
+static inline VALUE
+vm_method_missing(rb_thread_t *th, ID id, VALUE recv,
+		  int num, const rb_block_t *blockptr, int opt)
+{
+    VALUE *argv = ALLOCA_N(VALUE, num + 1);
+    vm_method_missing_args(th, argv, num, blockptr, opt);
+    argv[0] = ID2SYM(id);
+    return rb_funcall2(recv, idMethodMissing, num + 1, argv);
 }
 
 static inline void
 vm_setup_method(rb_thread_t *th, rb_control_frame_t *cfp,
-		const int argc, const rb_block_t *blockptr, const VALUE flag,
-		const VALUE iseqval, const VALUE recv)
+		VALUE recv, int argc, const rb_block_t *blockptr, VALUE flag,
+		const rb_method_entry_t *me)
 {
-    rb_iseq_t *iseq;
     int opt_pc, i;
     VALUE *sp, *rsp = cfp->sp - argc;
+    rb_iseq_t *iseq = me->def->body.iseq;
 
-    /* TODO: eliminate it */
-    GetISeqPtr(iseqval, iseq);
     VM_CALLEE_SETUP_ARG(opt_pc, th, iseq, argc, rsp, &blockptr);
 
     /* stack overflow check */
@@ -476,69 +502,112 @@ vm_setup_method(rb_thread_t *th, rb_control_frame_t *cfp,
 }
 
 static inline VALUE
-vm_call_method(rb_thread_t * const th, rb_control_frame_t * const cfp,
-	       const int num, rb_block_t * const blockptr, const VALUE flag,
-	       const ID id, const NODE * mn, const VALUE recv)
+vm_call_method(rb_thread_t *th, rb_control_frame_t *cfp,
+	       int num, const rb_block_t *blockptr, VALUE flag,
+	       ID id, const rb_method_entry_t *me, VALUE recv)
 {
     VALUE val;
 
   start_method_dispatch:
 
-    if (mn != 0) {
-	if ((mn->nd_noex == 0)) {
-	    /* dispatch method */
-	    NODE *node;
-
+    if (me != 0) {
+	if ((me->flag == 0)) {
 	  normal_method_dispatch:
-
-	    node = mn->nd_body;
-
-	    switch (nd_type(node)) {
-	      case RUBY_VM_METHOD_NODE:{
-		vm_setup_method(th, cfp, num, blockptr, flag, (VALUE)node->nd_body, recv);
+	    switch (me->def->type) {
+	      case VM_METHOD_TYPE_ISEQ:{
+		vm_setup_method(th, cfp, recv, num, blockptr, flag, me);
 		return Qundef;
 	      }
-	      case NODE_CFUNC:{
-		val = vm_call_cfunc(th, cfp, num, id, (ID)mn->nd_file, recv, mn->nd_clss, flag, node, blockptr);
+	      case VM_METHOD_TYPE_NOTIMPLEMENTED:
+	      case VM_METHOD_TYPE_CFUNC:{
+		val = vm_call_cfunc(th, cfp, num, recv, blockptr, me);
 		break;
 	      }
-	      case NODE_ATTRSET:{
-		val = rb_ivar_set(recv, node->nd_vid, *(cfp->sp - 1));
+	      case VM_METHOD_TYPE_ATTRSET:{
+		if (num != 1) {
+		    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)", num);
+		}
+		val = rb_ivar_set(recv, me->def->body.attr.id, *(cfp->sp - 1));
 		cfp->sp -= 2;
 		break;
 	      }
-	      case NODE_IVAR:{
+	      case VM_METHOD_TYPE_IVAR:{
 		if (num != 0) {
-		    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)",
-			     num);
+		    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)", num);
 		}
-		val = rb_attr_get(recv, node->nd_vid);
+		val = rb_attr_get(recv, me->def->body.attr.id);
 		cfp->sp -= 1;
 		break;
 	      }
-	      case NODE_BMETHOD:{
+	      case VM_METHOD_TYPE_MISSING:{
+		VALUE *argv = ALLOCA_N(VALUE, num+1);
+		argv[0] = ID2SYM(me->def->original_id);
+		MEMCPY(argv+1, cfp->sp - num, VALUE, num);
+		cfp->sp += - num - 1;
+		val = rb_funcall2(recv, rb_intern("method_missing"), num+1, argv);
+		break;
+	      }
+	      case VM_METHOD_TYPE_BMETHOD:{
 		VALUE *argv = ALLOCA_N(VALUE, num);
 		MEMCPY(argv, cfp->sp - num, VALUE, num);
 		cfp->sp += - num - 1;
-		val = vm_call_bmethod(th, (ID)mn->nd_file, node->nd_cval, recv, mn->nd_clss, num, argv, blockptr);
+		val = vm_call_bmethod(th, recv, num, argv, blockptr, me);
 		break;
 	      }
-	      case NODE_ZSUPER:{
-		VALUE klass;
-		klass = RCLASS_SUPER(mn->nd_clss);
-		mn = rb_method_node(klass, id);
+	      case VM_METHOD_TYPE_ZSUPER:{
+		VALUE klass = RCLASS_SUPER(me->klass);
+		me = rb_method_entry(klass, id);
 
-		if (mn != 0) {
+		if (me != 0) {
 		    goto normal_method_dispatch;
 		}
 		else {
 		    goto start_method_dispatch;
 		}
 	      }
+	      case VM_METHOD_TYPE_OPTIMIZED:{
+		switch (me->def->body.optimize_type) {
+		  case OPTIMIZED_METHOD_TYPE_SEND: {
+		    rb_control_frame_t *reg_cfp = cfp;
+		    rb_num_t i = num - 1;
+		    VALUE sym;
+
+		    if (num == 0) {
+			rb_raise(rb_eArgError, "no method name given");
+		    }
+
+		    sym = TOPN(i);
+		    id = SYMBOL_P(sym) ? SYM2ID(sym) : rb_to_id(sym);
+		    /* shift arguments */
+		    if (i > 0) {
+			MEMMOVE(&TOPN(i), &TOPN(i-1), VALUE, i);
+		    }
+		    me = rb_method_entry(CLASS_OF(recv), id);
+		    num -= 1;
+		    DEC_SP(1);
+		    flag |= VM_CALL_FCALL_BIT | VM_CALL_OPT_SEND_BIT;
+
+		    goto start_method_dispatch;
+		  }
+		  case OPTIMIZED_METHOD_TYPE_CALL: {
+		    rb_proc_t *proc;
+		    int argc = num;
+		    VALUE *argv = ALLOCA_N(VALUE, num);
+		    GetProcPtr(recv, proc);
+		    MEMCPY(argv, cfp->sp - num, VALUE, num);
+		    cfp->sp -= num + 1;
+
+		    val = rb_vm_invoke_proc(th, proc, proc->block.self, argc, argv, blockptr);
+		    break;
+		  }
+		  default:
+		    rb_bug("eval_invoke_method: unsupported optimized method type (%d)",
+			   me->def->body.optimize_type);
+		}
+		break;
+	      }
 	      default:{
-		printf("node: %s\n", ruby_node_name(nd_type(node)));
-		rb_bug("eval_invoke_method: unreachable");
-		/* unreachable */
+		rb_bug("eval_invoke_method: unsupported method type (%d)", me->def->type);
 		break;
 	      }
 	    }
@@ -547,7 +616,7 @@ vm_call_method(rb_thread_t * const th, rb_control_frame_t * const cfp,
 	    int noex_safe;
 
 	    if (!(flag & VM_CALL_FCALL_BIT) &&
-		(mn->nd_noex & NOEX_MASK) & NOEX_PRIVATE) {
+		(me->flag & NOEX_MASK) & NOEX_PRIVATE) {
 		int stat = NOEX_PRIVATE;
 
 		if (flag & VM_CALL_VCALL_BIT) {
@@ -555,22 +624,21 @@ vm_call_method(rb_thread_t * const th, rb_control_frame_t * const cfp,
 		}
 		val = vm_method_missing(th, id, recv, num, blockptr, stat);
 	    }
-	    else if (((mn->nd_noex & NOEX_MASK) & NOEX_PROTECTED) &&
-		     !(flag & VM_CALL_SEND_BIT)) {
-		VALUE defined_class = mn->nd_clss;
+	    else if (!(flag & VM_CALL_OPT_SEND_BIT) && (me->flag & NOEX_MASK) & NOEX_PROTECTED) {
+		VALUE defined_class = me->klass;
 
 		if (TYPE(defined_class) == T_ICLASS) {
 		    defined_class = RBASIC(defined_class)->klass;
 		}
 
-		if (!rb_obj_is_kind_of(cfp->self, rb_class_real(defined_class))) {
+		if (!rb_obj_is_kind_of(cfp->self, defined_class)) {
 		    val = vm_method_missing(th, id, recv, num, blockptr, NOEX_PROTECTED);
 		}
 		else {
 		    goto normal_method_dispatch;
 		}
 	    }
-	    else if ((noex_safe = NOEX_SAFE(mn->nd_noex)) > th->safe_level &&
+	    else if ((noex_safe = NOEX_SAFE(me->flag)) > th->safe_level &&
 		     (noex_safe > 2)) {
 		rb_raise(rb_eSecurityError, "calling insecure method: %s", rb_id2name(id));
 	    }
@@ -581,17 +649,19 @@ vm_call_method(rb_thread_t * const th, rb_control_frame_t * const cfp,
     }
     else {
 	/* method missing */
+	int stat = 0;
+	if (flag & VM_CALL_VCALL_BIT) {
+	    stat |= NOEX_VCALL;
+	}
+	if (flag & VM_CALL_SUPER_BIT) {
+	    stat |= NOEX_SUPER;
+	}
 	if (id == idMethodMissing) {
-	    rb_bug("method missing");
+	    VALUE *argv = ALLOCA_N(VALUE, num);
+	    vm_method_missing_args(th, argv, num - 1, 0, stat);
+	    rb_raise_method_missing(th, num, argv, recv, stat);
 	}
 	else {
-	    int stat = 0;
-	    if (flag & VM_CALL_VCALL_BIT) {
-		stat |= NOEX_VCALL;
-	    }
-	    if (flag & VM_CALL_SUPER_BIT) {
-		stat |= NOEX_SUPER;
-	    }
 	    val = vm_method_missing(th, id, recv, num, blockptr, stat);
 	}
     }
@@ -600,33 +670,6 @@ vm_call_method(rb_thread_t * const th, rb_control_frame_t * const cfp,
     return val;
 }
 
-static inline void
-vm_send_optimize(rb_control_frame_t * const reg_cfp, NODE ** const mn,
-		 rb_num_t * const flag, rb_num_t * const num,
-		 ID * const id, const VALUE klass)
-{
-    if (*mn && nd_type((*mn)->nd_body) == NODE_CFUNC) {
-	NODE *node = (*mn)->nd_body;
-	extern VALUE rb_f_send(int argc, VALUE *argv, VALUE recv);
-
-	if (node->nd_cfnc == rb_f_send) {
-	    int i = *num - 1;
-	    VALUE sym = TOPN(i);
-	    *id = SYMBOL_P(sym) ? SYM2ID(sym) : rb_to_id(sym);
-
-	    /* shift arguments */
-	    if (i > 0) {
-		MEMMOVE(&TOPN(i), &TOPN(i-1), VALUE, i);
-	    }
-
-	    *mn = rb_method_node(klass, *id);
-	    *num -= 1;
-	    DEC_SP(1);
-	    *flag |= VM_CALL_FCALL_BIT;
-	}
-    }
-}
-
 /* yield */
 
 static inline int
@@ -690,10 +733,10 @@ vm_yield_with_cfunc(rb_thread_t *th, const rb_block_t *block,
  * @pre iseq is block style (not lambda style)
  */
 static inline int
-vm_yield_setup_block_args_complex(rb_thread_t *th, const rb_iseq_t * iseq,
-	int argc, VALUE * argv)
+vm_yield_setup_block_args_complex(rb_thread_t *th, const rb_iseq_t *iseq,
+				  int argc, VALUE *argv)
 {
-    int opt_pc = 0;
+    rb_num_t opt_pc = 0;
     int i;
     const int m = iseq->argc;
     const int r = iseq->arg_rest;
@@ -743,18 +786,18 @@ vm_yield_setup_block_args_complex(rb_thread_t *th, const rb_iseq_t * iseq,
 	argv[start + i] = Qnil;
     }
 
-    return opt_pc;
+    return (int)opt_pc;
 }
 
 static inline int
 vm_yield_setup_block_args(rb_thread_t *th, const rb_iseq_t * iseq,
-	int orig_argc, VALUE * argv,
-	const rb_block_t *blockptr)
+			  int orig_argc, VALUE *argv,
+			  const rb_block_t *blockptr)
 {
     int i;
     int argc = orig_argc;
     const int m = iseq->argc;
-    VALUE ary;
+    VALUE ary, arg0;
     int opt_pc = 0;
 
     th->mark_stack_len = argc;
@@ -764,15 +807,19 @@ vm_yield_setup_block_args(rb_thread_t *th, const rb_iseq_t * iseq,
      *  => {|a|} => a = [1, 2]
      *  => {|a, b|} => a, b = [1, 2]
      */
+    arg0 = argv[0];
     if (!(iseq->arg_simple & 0x02) &&          /* exclude {|a|} */
             (m + iseq->arg_post_len) > 0 &&    /* this process is meaningful */
-            argc == 1 && !NIL_P(ary = rb_check_array_type(argv[0]))) { /* rhs is only an array */
-        th->mark_stack_len = argc = RARRAY_LEN(ary);
+            argc == 1 && !NIL_P(ary = rb_check_array_type(arg0))) { /* rhs is only an array */
+        th->mark_stack_len = argc = RARRAY_LENINT(ary);
 
         CHECK_STACK_OVERFLOW(th->cfp, argc);
 
         MEMCPY(argv, RARRAY_PTR(ary), VALUE, argc);
     }
+    else {
+        argv[0] = arg0;
+    }
 
     for (i=argc; i<m; i++) {
         argv[i] = Qnil;
@@ -791,7 +838,7 @@ vm_yield_setup_block_args(rb_thread_t *th, const rb_iseq_t * iseq,
     else {
         int r = iseq->arg_rest;
 
-        if (iseq->arg_post_len || 
+        if (iseq->arg_post_len ||
                 iseq->arg_opts) { /* TODO: implement simple version for (iseq->arg_post_len==0 && iseq->arg_opts > 0) */
 	    opt_pc = vm_yield_setup_block_args_complex(th, iseq, argc, argv);
         }
@@ -818,7 +865,12 @@ vm_yield_setup_block_args(rb_thread_t *th, const rb_iseq_t * iseq,
         VALUE procval = Qnil;
 
         if (blockptr) {
-            procval = blockptr->proc;
+	    if (blockptr->proc == 0) {
+		procval = rb_vm_make_proc(th, blockptr, rb_cProc);
+	    }
+	    else {
+		procval = blockptr->proc;
+	    }
         }
 
         argv[iseq->arg_block] = procval;
@@ -858,11 +910,12 @@ vm_yield_setup_args(rb_thread_t * const th, const rb_iseq_t *iseq,
 static VALUE
 vm_invoke_block(rb_thread_t *th, rb_control_frame_t *reg_cfp, rb_num_t num, rb_num_t flag)
 {
-    rb_block_t * const block = GET_BLOCK_PTR();
+    const rb_block_t *block = GET_BLOCK_PTR();
     rb_iseq_t *iseq;
-    int argc = num;
+    int argc = (int)num;
+    VALUE type = GET_ISEQ()->local_iseq->type;
 
-    if (GET_ISEQ()->local_iseq->type != ISEQ_TYPE_METHOD || block == 0) {
+    if ((type != ISEQ_TYPE_METHOD && type != ISEQ_TYPE_CLASS) || block == 0) {
 	rb_vm_localjump_error("no block given (yield)", Qnil, 0);
     }
     iseq = block->iseq;
@@ -992,7 +1045,7 @@ vm_getspecial(rb_thread_t *th, VALUE *lfp, VALUE key, rb_num_t type)
 	    }
 	}
 	else {
-	    val = rb_reg_nth_match(type >> 1, backref);
+	    val = rb_reg_nth_match((int)(type >> 1), backref);
 	}
     }
     return val;
@@ -1022,14 +1075,17 @@ vm_get_cref(const rb_iseq_t *iseq, const VALUE *lfp, const VALUE *dfp)
 }
 
 static NODE *
-vm_cref_push(rb_thread_t *th, VALUE klass, int noex)
+vm_cref_push(rb_thread_t *th, VALUE klass, int noex, rb_block_t *blockptr)
 {
     rb_control_frame_t *cfp = vm_get_ruby_level_caller_cfp(th, th->cfp);
     NODE *cref = NEW_BLOCK(klass);
     cref->nd_file = 0;
     cref->nd_visi = noex;
 
-    if (cfp) {
+    if (blockptr) {
+	cref->nd_next = vm_get_cref(blockptr->iseq, blockptr->lfp, blockptr->dfp);
+    }
+    else if (cfp) {
 	cref->nd_next = vm_get_cref(cfp->iseq, cfp->lfp, cfp->dfp);
     }
 
@@ -1052,6 +1108,23 @@ vm_get_cbase(const rb_iseq_t *iseq, const VALUE *lfp, const VALUE *dfp)
     return klass;
 }
 
+static inline VALUE
+vm_get_const_base(const rb_iseq_t *iseq, const VALUE *lfp, const VALUE *dfp)
+{
+    NODE *cref = vm_get_cref(iseq, lfp, dfp);
+    VALUE klass = Qundef;
+
+    while (cref) {
+	if (!(cref->flags & NODE_FL_CREF_PUSHED_BY_EVAL) &&
+	    (klass = cref->nd_clss) != 0) {
+	    break;
+	}
+	cref = cref->nd_next;
+    }
+
+    return klass;
+}
+
 static inline void
 vm_check_if_namespace(VALUE klass)
 {
@@ -1073,19 +1146,26 @@ vm_get_ev_const(rb_thread_t *th, const rb_iseq_t *iseq,
 
     if (orig_klass == Qnil) {
 	/* in current lexical scope */
-	const NODE *root_cref = vm_get_cref(iseq, th->cfp->lfp, th->cfp->dfp);
-	const NODE *cref = root_cref;
+	const NODE *cref = vm_get_cref(iseq, th->cfp->lfp, th->cfp->dfp);
+	const NODE *root_cref = NULL;
 	VALUE klass = orig_klass;
 
 	while (cref && cref->nd_next) {
-	    klass = cref->nd_clss;
+	    if (!(cref->flags & NODE_FL_CREF_PUSHED_BY_EVAL)) {
+		klass = cref->nd_clss;
+		if (root_cref == NULL)
+		    root_cref = cref;
+	    }
 	    cref = cref->nd_next;
 
 	    if (!NIL_P(klass)) {
+		VALUE am = 0;
 	      search_continue:
 		if (RCLASS_IV_TBL(klass) &&
 		    st_lookup(RCLASS_IV_TBL(klass), id, &val)) {
 		    if (val == Qundef) {
+			if (am == klass) break;
+			am = klass;
 			rb_autoload_load(klass, id);
 			goto search_continue;
 		    }
@@ -1102,8 +1182,10 @@ vm_get_ev_const(rb_thread_t *th, const rb_iseq_t *iseq,
 	}
 
 	/* search self */
-	klass = root_cref->nd_clss;
-	if (NIL_P(klass)) {
+	if (root_cref && !NIL_P(root_cref->nd_clss)) {
+	    klass = root_cref->nd_clss;
+	}
+	else {
 	    klass = CLASS_OF(th->cfp->self);
 	}
 
@@ -1131,7 +1213,8 @@ vm_get_cvar_base(NODE *cref)
     VALUE klass;
 
     while (cref && cref->nd_next &&
-	   (NIL_P(cref->nd_clss) || FL_TEST(cref->nd_clss, FL_SINGLETON))) {
+	   (NIL_P(cref->nd_clss) || FL_TEST(cref->nd_clss, FL_SINGLETON) ||
+	    (cref->flags & NODE_FL_CREF_PUSHED_BY_EVAL))) {
 	cref = cref->nd_next;
 
 	if (!cref->nd_next) {
@@ -1147,28 +1230,118 @@ vm_get_cvar_base(NODE *cref)
     return klass;
 }
 
-static inline NODE *
-vm_method_search(VALUE id, VALUE klass, IC ic)
-{
-    NODE *mn;
 
-#if OPT_INLINE_METHOD_CACHE
-    {
-	if (LIKELY(klass == ic->ic_class) &&
-	    LIKELY(GET_VM_STATE_VERSION() == ic->ic_vmstat)) {
-	    mn = ic->ic_method;
+#ifndef USE_IC_FOR_IVAR
+#define USE_IC_FOR_IVAR 1
+#endif
+
+static VALUE
+vm_getivar(VALUE obj, ID id, IC ic)
+{
+#if USE_IC_FOR_IVAR
+    if (TYPE(obj) ==  T_OBJECT) {
+	VALUE val = Qundef;
+	VALUE klass = RBASIC(obj)->klass;
+
+	if (ic->ic_class == klass) {
+	    long index = ic->ic_value.index;
+	    long len = ROBJECT_NUMIV(obj);
+	    VALUE *ptr = ROBJECT_IVPTR(obj);
+
+	    if (index < len) {
+		val = ptr[index];
+	    }
 	}
 	else {
-	    mn = rb_method_node(klass, id);
-	    ic->ic_class = klass;
-	    ic->ic_method = mn;
-	    ic->ic_vmstat = GET_VM_STATE_VERSION();
+	    st_data_t index;
+	    long len = ROBJECT_NUMIV(obj);
+	    VALUE *ptr = ROBJECT_IVPTR(obj);
+	    struct st_table *iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+
+	    if (iv_index_tbl) {
+		if (st_lookup(iv_index_tbl, id, &index)) {
+		    if ((long)index < len) {
+			val = ptr[index];
+		    }
+		    ic->ic_class = klass;
+		    ic->ic_value.index = index;
+		}
+	    }
 	}
+	if (UNLIKELY(val == Qundef)) {
+	    rb_warning("instance variable %s not initialized", rb_id2name(id));
+	    val = Qnil;
+	}
+	return val;
+    }
+    else {
+	return rb_ivar_get(obj, id);
     }
 #else
-    mn = rb_method_node(klass, id);
+    return rb_ivar_get(obj, id);
 #endif
-    return mn;
+}
+
+static void
+vm_setivar(VALUE obj, ID id, VALUE val, IC ic)
+{
+#if USE_IC_FOR_IVAR
+    if (!OBJ_UNTRUSTED(obj) && rb_safe_level() >= 4) {
+	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
+    }
+    if (OBJ_FROZEN(obj)) {
+	rb_error_frozen("object");
+    }
+
+    if (TYPE(obj) == T_OBJECT) {
+	VALUE klass = RBASIC(obj)->klass;
+	st_data_t index;
+
+	if (ic->ic_class == klass) {
+	    long index = ic->ic_value.index;
+	    long len = ROBJECT_NUMIV(obj);
+	    VALUE *ptr = ROBJECT_IVPTR(obj);
+
+	    if (index < len) {
+		ptr[index] = val;
+		return; /* inline cache hit */
+	    }
+	}
+	else {
+	    struct st_table *iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+
+	    if (iv_index_tbl && st_lookup(iv_index_tbl, (st_data_t)id, &index)) {
+		ic->ic_class = klass;
+		ic->ic_value.index = index;
+	    }
+	    /* fall through */
+	}
+    }
+    rb_ivar_set(obj, id, val);
+#else
+    rb_ivar_set(obj, id, val);
+#endif
+}
+
+static inline const rb_method_entry_t *
+vm_method_search(VALUE id, VALUE klass, IC ic)
+{
+    rb_method_entry_t *me;
+#if OPT_INLINE_METHOD_CACHE
+    if (LIKELY(klass == ic->ic_class) &&
+	LIKELY(GET_VM_STATE_VERSION() == ic->ic_vmstat)) {
+	me = ic->ic_value.method;
+    }
+    else {
+	me = rb_method_entry(klass, id);
+	ic->ic_class = klass;
+	ic->ic_value.method = me;
+	ic->ic_vmstat = GET_VM_STATE_VERSION();
+    }
+#else
+    me = rb_method_entry(klass, id);
+#endif
+    return me;
 }
 
 static inline VALUE
@@ -1214,6 +1387,11 @@ vm_search_superclass(rb_control_frame_t *reg_cfp, rb_iseq_t *ip,
 	/* defined by Module#define_method() */
 	rb_control_frame_t *lcfp = GET_CFP();
 
+	if (!sigval) {
+	    /* zsuper */
+	    rb_raise(rb_eRuntimeError, "implicit argument passing of super from method defined by define_method() is not supported. Specify all arguments explicitly.");
+	}
+
 	while (lcfp->iseq != ip) {
 	    VALUE *tdfp = GET_PREV_DFP(lcfp->dfp);
 	    while (1) {
@@ -1224,13 +1402,13 @@ vm_search_superclass(rb_control_frame_t *reg_cfp, rb_iseq_t *ip,
 	    }
 	}
 
-	id = lcfp->method_id;
-	klass = vm_search_normal_superclass(lcfp->method_class, recv);
-
-	if (sigval == Qfalse) {
-	    /* zsuper */
-	    rb_raise(rb_eRuntimeError, "implicit argument passing of super from method defined by define_method() is not supported. Specify all arguments explicitly.");
+	/* temporary measure for [Bug #2420] [Bug #3136] */
+	if (!lcfp->me) {
+	    rb_raise(rb_eNoMethodError, "super called outside of method");
 	}
+
+	id = lcfp->me->def->original_id;
+	klass = vm_search_normal_superclass(lcfp->me->klass, recv);
     }
     else {
 	klass = vm_search_normal_superclass(ip->klass, recv);
@@ -1244,13 +1422,12 @@ static VALUE
 vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 	 rb_num_t throw_state, VALUE throwobj)
 {
-    rb_num_t state = throw_state & 0xff;
-    rb_num_t flag = throw_state & 0x8000;
+    int state = (int)(throw_state & 0xff);
+    int flag = (int)(throw_state & 0x8000);
     rb_num_t level = throw_state >> 16;
 
     if (state != 0) {
 	VALUE *pt = 0;
-	int i;
 	if (flag != 0) {
 	    pt = (void *) 1;
 	}
@@ -1263,6 +1440,11 @@ vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 
 	      search_parent:
 		if (cfp->iseq->type != ISEQ_TYPE_BLOCK) {
+		    if (cfp->iseq->type == ISEQ_TYPE_CLASS) {
+			cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+			dfp = cfp->dfp;
+			goto search_parent;
+		    }
 		    dfp = GC_GUARDED_PTR_REF((VALUE *) *dfp);
 		    base_iseq = base_iseq->parent_iseq;
 
@@ -1319,6 +1501,7 @@ vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 		}
 	    }
 	    else if (state == TAG_RETRY) {
+		rb_num_t i;
 		pt = GC_GUARDED_PTR_REF((VALUE *) * GET_DFP());
 		for (i = 0; i < level; i++) {
 		    pt = GC_GUARDED_PTR_REF((VALUE *) * pt);
@@ -1327,10 +1510,17 @@ vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 	    else if (state == TAG_RETURN) {
 		rb_control_frame_t *cfp = GET_CFP();
 		VALUE *dfp = GET_DFP();
-		VALUE * const lfp = GET_LFP();
+		VALUE *lfp = GET_LFP();
 
 		/* check orphan and get dfp */
 		while ((VALUE *) cfp < th->stack + th->stack_size) {
+		    if (!lfp) {
+			lfp = cfp->lfp;
+		    }
+		    if (cfp->dfp == lfp && cfp->iseq->type == ISEQ_TYPE_CLASS) {
+			lfp = 0;
+		    }
+
 		    if (cfp->lfp == lfp) {
 			if (VM_FRAME_TYPE(cfp) == VM_FRAME_MAGIC_LAMBDA) {
 			    VALUE *tdfp = dfp;
@@ -1341,7 +1531,7 @@ vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 				    dfp = cfp->dfp;
 				    goto valid_return;
 				}
-				tdfp = GC_GUARDED_PTR_REF((VALUE *)*dfp);
+				tdfp = GC_GUARDED_PTR_REF((VALUE *)*tdfp);
 			    }
 			}
 		    }
@@ -1388,13 +1578,13 @@ vm_throw(rb_thread_t *th, rb_control_frame_t *reg_cfp,
 }
 
 static inline void
-vm_expandarray(rb_control_frame_t *cfp, VALUE ary, int num, int flag)
+vm_expandarray(rb_control_frame_t *cfp, VALUE ary, rb_num_t num, int flag)
 {
     int is_splat = flag & 0x01;
-    int space_size = num + is_splat;
+    rb_num_t space_size = num + is_splat;
     VALUE *base = cfp->sp, *ptr;
     volatile VALUE tmp_ary;
-    int len;
+    rb_num_t len;
 
     if (TYPE(ary) != T_ARRAY) {
 	ary = rb_ary_to_ary(ary);
@@ -1404,11 +1594,11 @@ vm_expandarray(rb_control_frame_t *cfp, VALUE ary, int num, int flag)
 
     tmp_ary = ary;
     ptr = RARRAY_PTR(ary);
-    len = RARRAY_LEN(ary);
+    len = (rb_num_t)RARRAY_LEN(ary);
 
     if (flag & 0x02) {
 	/* post: ..., nil ,ary[-1], ..., ary[0..-num] # top */
-	int i = 0, j;
+	rb_num_t i = 0, j;
 
 	if (len < num) {
 	    for (i=0; i<num-len; i++) {
@@ -1425,7 +1615,7 @@ vm_expandarray(rb_control_frame_t *cfp, VALUE ary, int num, int flag)
     }
     else {
 	/* normal: ary[num..-1], ary[num-2], ary[num-3], ..., ary[0] # top */
-	int i;
+	rb_num_t i;
 	VALUE *bptr = &base[space_size - 1];
 
 	for (i=0; i<num; i++) {
@@ -1449,10 +1639,10 @@ vm_expandarray(rb_control_frame_t *cfp, VALUE ary, int num, int flag)
 }
 
 static inline int
-check_cfunc(const NODE *mn, VALUE (*func)())
+check_cfunc(const rb_method_entry_t *me, VALUE (*func)())
 {
-    if (mn && nd_type(mn->nd_body) == NODE_CFUNC &&
-	mn->nd_body->nd_cfnc == func) {
+    if (me && me->def->type == VM_METHOD_TYPE_CFUNC &&
+	me->def->body.cfunc.func == func) {
 	return 1;
     }
     else {
@@ -1467,16 +1657,9 @@ inline
 VALUE
 opt_eq_func(VALUE recv, VALUE obj, IC ic)
 {
-    VALUE val = Qundef;
-
     if (FIXNUM_2_P(recv, obj) &&
 	BASIC_OP_UNREDEFINED_P(BOP_EQ)) {
-	if (recv == obj) {
-	    val = Qtrue;
-	}
-	else {
-	    val = Qfalse;
-	}
+	return (recv == obj) ? Qtrue : Qfalse;
     }
     else if (!SPECIAL_CONST_P(recv) && !SPECIAL_CONST_P(obj)) {
 	if (HEAP_CLASS_OF(recv) == rb_cFloat &&
@@ -1486,29 +1669,45 @@ opt_eq_func(VALUE recv, VALUE obj, IC ic)
 	    double b = RFLOAT_VALUE(obj);
 
 	    if (isnan(a) || isnan(b)) {
-		val = Qfalse;
-	    }
-	    else if (a == b) {
-		val = Qtrue;
-	    }
-	    else {
-		val = Qfalse;
+		return Qfalse;
 	    }
+	    return  (a == b) ? Qtrue : Qfalse;
 	}
 	else if (HEAP_CLASS_OF(recv) == rb_cString &&
 		 HEAP_CLASS_OF(obj) == rb_cString &&
 		 BASIC_OP_UNREDEFINED_P(BOP_EQ)) {
-	    val = rb_str_equal(recv, obj);
-	}
-	else {
-	    NODE *mn = vm_method_search(idEq, CLASS_OF(recv), ic);
-	    extern VALUE rb_obj_equal(VALUE obj1, VALUE obj2);
-
-	    if (check_cfunc(mn, rb_obj_equal)) {
-		return recv == obj ? Qtrue : Qfalse;
-	    }
+	    return rb_str_equal(recv, obj);
 	}
     }
 
-    return val;
+    {
+	const rb_method_entry_t *me = vm_method_search(idEq, CLASS_OF(recv), ic);
+	extern VALUE rb_obj_equal(VALUE obj1, VALUE obj2);
+
+	if (check_cfunc(me, rb_obj_equal)) {
+	    return recv == obj ? Qtrue : Qfalse;
+	}
+    }
+
+    return Qundef;
 }
+
+struct opt_case_dispatch_i_arg {
+    VALUE obj;
+    int label;
+};
+
+static int
+opt_case_dispatch_i(st_data_t key, st_data_t data, st_data_t p)
+{
+    struct opt_case_dispatch_i_arg *arg = (void *)p;
+
+    if (RTEST(rb_funcall((VALUE)key, idEqq, 1, arg->obj))) {
+	arg->label = FIX2INT((VALUE)data);
+	return ST_STOP;
+    }
+    else {
+	return ST_CONTINUE;
+    }
+}
+
