@@ -3,7 +3,7 @@
 
   vm_opts.h - VM optimize option
 
-  $Author: nobu $
+  $Author: akr $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -18,7 +18,7 @@
  * Following definitions are default values.
  */
 
-#define OPT_TRACE_INSTRUCTION        0
+#define OPT_TRACE_INSTRUCTION        1
 #define OPT_TAILCALL_OPTIMIZATION    0
 #define OPT_PEEPHOLE_OPTIMIZATION    1
 #define OPT_SPECIALISED_INSTRUCTION  1
@@ -31,6 +31,7 @@
 
 /* C compiler depend */
 #define OPT_DIRECT_THREADED_CODE     1
+#define OPT_TOKEN_THREADED_CODE      0
 #define OPT_CALL_THREADED_CODE       0
 
 /* VM running option */
